!------------------------------------------------------------------------------
!    This code is part of the MondoSCF suite of programs for linear scaling
!    electronic structure theory and ab initio molecular dynamics.
!
!    Copyright (2004). The Regents of the University of California. This
!    material was produced under U.S. Government contract W-7405-ENG-36
!    for Los Alamos National Laboratory, which is operated by the University
!    of California for the U.S. Department of Energy. The U.S. Government has
!    rights to use, reproduce, and distribute this software.  NEITHER THE
!    GOVERNMENT NOR THE UNIVERSITY MAKES ANY WARRANTY, EXPRESS OR IMPLIED,
!    OR ASSUMES ANY LIABILITY FOR THE USE OF THIS SOFTWARE.
!
!    This program is free software; you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by the
!    Free Software Foundation; either version 2 of the License, or (at your
!    option) any later version. Accordingly, this program is distributed in
!    the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
!    the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
!    PURPOSE. See the GNU General Public License at www.gnu.org for details.
!
!    While you may do as you like with this software, the GNU license requires
!    that you clearly mark derivative software.  In addition, you are encouraged
!    to return derivative works to the MondoSCF group for review, and possible
!    disemination in future releases.
!------------------------------------------------------------------------------
! ----------------------------------------------------------
! COMPUTES THE INTEGRAL CLASS (f f|f sp)
! ----------------------------------------------------------
   SUBROUTINE IntB10101002(PrmBufB,LBra,PrmBufK,LKet,ACInfo,BDInfo, &
                              OA,LDA,OB,LDB,OC,LDC,OD,LDD,PBC,INTGRL)
      USE DerivedTypes
      USE VScratchB
      USE GlobalScalars
      USE ShellPairStruct
      USE GammaF10
      IMPLICIT REAL(DOUBLE) (W)
      INTEGER        :: LBra,LKet,CDOffSet
      REAL(DOUBLE)   :: PrmBufB(10,LBra),PrmBufK(10,LKet)
      TYPE(SmallAtomInfo) :: ACInfo,BDInfo
      TYPE(PBCInfo) :: PBC
      REAL(DOUBLE)  :: INTGRL(*)
      REAL(DOUBLE)  :: Ax,Ay,Az,Bx,By,Bz,Cx,Cy,Cz
      REAL(DOUBLE)  :: Dx,Dy,Dz,Qx,Qy,Qz,Px,Py,Pz
      REAL(DOUBLE)  :: PQx,PQy,PQz,FPQx,FPQy,FPQz
      REAL(DOUBLE)  :: Zeta,Eta,Omega,Up,Uq,Upq
      REAL(DOUBLE)  :: T,ET,TwoT,InvT,SqInvT
      REAL(DOUBLE)  :: VRR(84,35,0:10)
      REAL(DOUBLE)  :: HRR(84,45,4)
      INTEGER       :: OffSet,OA,LDA,OB,LDB,OC,LDC,OD,LDD,I,J,K,L
      EXTERNAL InitDbl
      CALL InitDbl(84*45,HRR(1,1,1))
      Ax=ACInfo%Atm1X
      Ay=ACInfo%Atm1Y
      Az=ACInfo%Atm1Z
      Bx=ACInfo%Atm2X
      By=ACInfo%Atm2Y
      Bz=ACInfo%Atm2Z
      Cx=BDInfo%Atm1X
      Cy=BDInfo%Atm1Y
      Cz=BDInfo%Atm1Z
      Dx=BDInfo%Atm2X
      Dy=BDInfo%Atm2Y
      Dz=BDInfo%Atm2Z
      ABx=Ax-Bx
      ABy=Ay-By
      ABz=Az-Bz
      CDx=Cx-Dx
      CDy=Cy-Dy
      CDz=Cz-Dz
      DO J=1,LKet ! K^2 VRR |N0) loop
         Eta=PrmBufK(1,J)
         Qx=PrmBufK(2,J)
         Qy=PrmBufK(3,J)
         Qz=PrmBufK(4,J)
         Uq=PrmBufK(5,J)
         FnSpK=PrmBufK(6,J)
         QCx=Qx-Cx
         QCy=Qy-Cy
         QCz=Qz-Cz
         DO K=1,LBra ! K^2 VRR (M0| loop
            Zeta=PrmBufB(1,K)
            Px=PrmBufB(2,K)
            Py=PrmBufB(3,K)
            Pz=PrmBufB(4,K)
            Up=PrmBufB(5,K)
            r1xZpE=One/(Zeta+Eta)
            Upq=SQRT(r1xZpE)*Up*Uq
            HfxZpE=Half/(Zeta+Eta)
            r1x2E=Half/Eta
            r1x2Z=Half/Zeta
            ExZpE=Eta*r1xZpE
            ZxZpE=Zeta*r1xZpE
            Omega=Eta*Zeta*r1xZpE
            PAx=Px-Ax
            PAy=Py-Ay
            PAz=Pz-Az
            PQx=Px-Qx
            PQy=Py-Qy
            PQz=Pz-Qz
            ! Begin Minimum Image Convention
            FPQx = PQx*PBC%InvBoxSh%D(1,1)+PQy*PBC%InvBoxSh%D(1,2)+PQz*PBC%InvBoxSh%D(1,3)
            FPQy = PQy*PBC%InvBoxSh%D(2,2)+PQz*PBC%InvBoxSh%D(2,3)
            FPQz = PQz*PBC%InvBoxSh%D(3,3)
            IF(PBC%AutoW%I(1)==1)FPQx=FPQx-ANINT(FPQx-SIGN(1D-15,FPQx))
            IF(PBC%AutoW%I(2)==1)FPQy=FPQy-ANINT(FPQy-SIGN(1D-15,FPQy))
            IF(PBC%AutoW%I(3)==1)FPQz=FPQz-ANINT(FPQz-SIGN(1D-15,FPQz))
            PQx=FPQx*PBC%BoxShape%D(1,1)+FPQy*PBC%BoxShape%D(1,2)+FPQz*PBC%BoxShape%D(1,3)
            PQy=FPQy*PBC%BoxShape%D(2,2)+FPQz*PBC%BoxShape%D(2,3)
            PQz=FPQz*PBC%BoxShape%D(3,3)
            ! End MIC
            WPx = -Eta*PQx*r1xZpE
            WPy = -Eta*PQy*r1xZpE
            WPz = -Eta*PQz*r1xZpE
            WQx = Zeta*PQx*r1xZpE
            WQy = Zeta*PQy*r1xZpE
            WQz = Zeta*PQz*r1xZpE
            T=Omega*(PQx*PQx+PQy*PQy+PQz*PQz)
            IF(T<Gamma_Switch)THEN
              L=AINT(T*Gamma_Grid)
              ET=EXP(-T)
              TwoT=Two*T
              W10=(F10_0(L)+T*(F10_1(L)+T*(F10_2(L)+T*(F10_3(L)+T*F10_4(L)))))
              W9=+5.263157894736842D-02*(TwoT*W10+ET)
              W8=+5.882352941176471D-02*(TwoT*W9+ET)
              W7=+6.666666666666666D-02*(TwoT*W8+ET)
              W6=+7.692307692307693D-02*(TwoT*W7+ET)
              W5=+9.090909090909090D-02*(TwoT*W6+ET)
              W4=+1.111111111111111D-01*(TwoT*W5+ET)
              W3=+1.428571428571428D-01*(TwoT*W4+ET)
              W2=+2.000000000000000D-01*(TwoT*W3+ET)
              W1=+3.333333333333333D-01*(TwoT*W2+ET)
              W0=TwoT*W1+ET
              VRR(1,1,0)=Upq*W0
              VRR(1,1,1)=Upq*W1
              VRR(1,1,2)=Upq*W2
              VRR(1,1,3)=Upq*W3
              VRR(1,1,4)=Upq*W4
              VRR(1,1,5)=Upq*W5
              VRR(1,1,6)=Upq*W6
              VRR(1,1,7)=Upq*W7
              VRR(1,1,8)=Upq*W8
              VRR(1,1,9)=Upq*W9
              VRR(1,1,10)=Upq*W10
            ELSE
              InvT=One/T
              SqInvT=DSQRT(InvT)
              VRR(1,1,0)=+8.862269254527580D-01*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,1)=+4.431134627263790D-01*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,2)=+6.646701940895685D-01*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,3)=+1.661675485223921D+00*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,4)=+5.815864198283724D+00*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,5)=+2.617138889227676D+01*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,6)=+1.439426389075222D+02*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,7)=+9.356271528988940D+02*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,8)=+7.017203646741708D+03*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,9)=+5.964623099730450D+04*Upq*SqInvT
              SqInvT=SqInvT*InvT
              VRR(1,1,10)=+5.666391944743928D+05*Upq*SqInvT
            ENDIF
            ! Generating (p0|s0)^(9)
            VRR(2,1,9)=PAx*VRR(1,1,9)+WPx*VRR(1,1,10)
            VRR(3,1,9)=PAy*VRR(1,1,9)+WPy*VRR(1,1,10)
            VRR(4,1,9)=PAz*VRR(1,1,9)+WPz*VRR(1,1,10)
            ! Generating (p0|s0)^(8)
            VRR(2,1,8)=PAx*VRR(1,1,8)+WPx*VRR(1,1,9)
            VRR(3,1,8)=PAy*VRR(1,1,8)+WPy*VRR(1,1,9)
            VRR(4,1,8)=PAz*VRR(1,1,8)+WPz*VRR(1,1,9)
            ! Generating (p0|s0)^(7)
            VRR(2,1,7)=PAx*VRR(1,1,7)+WPx*VRR(1,1,8)
            VRR(3,1,7)=PAy*VRR(1,1,7)+WPy*VRR(1,1,8)
            VRR(4,1,7)=PAz*VRR(1,1,7)+WPz*VRR(1,1,8)
            ! Generating (p0|s0)^(6)
            VRR(2,1,6)=PAx*VRR(1,1,6)+WPx*VRR(1,1,7)
            VRR(3,1,6)=PAy*VRR(1,1,6)+WPy*VRR(1,1,7)
            VRR(4,1,6)=PAz*VRR(1,1,6)+WPz*VRR(1,1,7)
            ! Generating (p0|s0)^(5)
            VRR(2,1,5)=PAx*VRR(1,1,5)+WPx*VRR(1,1,6)
            VRR(3,1,5)=PAy*VRR(1,1,5)+WPy*VRR(1,1,6)
            VRR(4,1,5)=PAz*VRR(1,1,5)+WPz*VRR(1,1,6)
            ! Generating (p0|s0)^(4)
            VRR(2,1,4)=PAx*VRR(1,1,4)+WPx*VRR(1,1,5)
            VRR(3,1,4)=PAy*VRR(1,1,4)+WPy*VRR(1,1,5)
            VRR(4,1,4)=PAz*VRR(1,1,4)+WPz*VRR(1,1,5)
            ! Generating (p0|s0)^(3)
            VRR(2,1,3)=PAx*VRR(1,1,3)+WPx*VRR(1,1,4)
            VRR(3,1,3)=PAy*VRR(1,1,3)+WPy*VRR(1,1,4)
            VRR(4,1,3)=PAz*VRR(1,1,3)+WPz*VRR(1,1,4)
            ! Generating (p0|s0)^(2)
            VRR(2,1,2)=PAx*VRR(1,1,2)+WPx*VRR(1,1,3)
            VRR(3,1,2)=PAy*VRR(1,1,2)+WPy*VRR(1,1,3)
            VRR(4,1,2)=PAz*VRR(1,1,2)+WPz*VRR(1,1,3)
            ! Generating (p0|s0)^(1)
            VRR(2,1,1)=PAx*VRR(1,1,1)+WPx*VRR(1,1,2)
            VRR(3,1,1)=PAy*VRR(1,1,1)+WPy*VRR(1,1,2)
            VRR(4,1,1)=PAz*VRR(1,1,1)+WPz*VRR(1,1,2)
            ! Generating (p0|s0)^(0)
            VRR(2,1,0)=PAx*VRR(1,1,0)+WPx*VRR(1,1,1)
            VRR(3,1,0)=PAy*VRR(1,1,0)+WPy*VRR(1,1,1)
            VRR(4,1,0)=PAz*VRR(1,1,0)+WPz*VRR(1,1,1)
            ! Generating (d0|s0)^(8)
            VRR(5,1,8)=PAx*VRR(2,1,8)+r1x2Z*(VRR(1,1,8)-ExZpE*VRR(1,1,9))+WPx*VRR(2,1,9)
            VRR(6,1,8)=PAx*VRR(3,1,8)+WPx*VRR(3,1,9)
            VRR(7,1,8)=PAy*VRR(3,1,8)+r1x2Z*(VRR(1,1,8)-ExZpE*VRR(1,1,9))+WPy*VRR(3,1,9)
            VRR(8,1,8)=PAx*VRR(4,1,8)+WPx*VRR(4,1,9)
            VRR(9,1,8)=PAy*VRR(4,1,8)+WPy*VRR(4,1,9)
            VRR(10,1,8)=PAz*VRR(4,1,8)+r1x2Z*(VRR(1,1,8)-ExZpE*VRR(1,1,9))+WPz*VRR(4,1,9)
            ! Generating (d0|s0)^(7)
            VRR(5,1,7)=PAx*VRR(2,1,7)+r1x2Z*(VRR(1,1,7)-ExZpE*VRR(1,1,8))+WPx*VRR(2,1,8)
            VRR(6,1,7)=PAx*VRR(3,1,7)+WPx*VRR(3,1,8)
            VRR(7,1,7)=PAy*VRR(3,1,7)+r1x2Z*(VRR(1,1,7)-ExZpE*VRR(1,1,8))+WPy*VRR(3,1,8)
            VRR(8,1,7)=PAx*VRR(4,1,7)+WPx*VRR(4,1,8)
            VRR(9,1,7)=PAy*VRR(4,1,7)+WPy*VRR(4,1,8)
            VRR(10,1,7)=PAz*VRR(4,1,7)+r1x2Z*(VRR(1,1,7)-ExZpE*VRR(1,1,8))+WPz*VRR(4,1,8)
            ! Generating (d0|s0)^(6)
            VRR(5,1,6)=PAx*VRR(2,1,6)+r1x2Z*(VRR(1,1,6)-ExZpE*VRR(1,1,7))+WPx*VRR(2,1,7)
            VRR(6,1,6)=PAx*VRR(3,1,6)+WPx*VRR(3,1,7)
            VRR(7,1,6)=PAy*VRR(3,1,6)+r1x2Z*(VRR(1,1,6)-ExZpE*VRR(1,1,7))+WPy*VRR(3,1,7)
            VRR(8,1,6)=PAx*VRR(4,1,6)+WPx*VRR(4,1,7)
            VRR(9,1,6)=PAy*VRR(4,1,6)+WPy*VRR(4,1,7)
            VRR(10,1,6)=PAz*VRR(4,1,6)+r1x2Z*(VRR(1,1,6)-ExZpE*VRR(1,1,7))+WPz*VRR(4,1,7)
            ! Generating (d0|s0)^(5)
            VRR(5,1,5)=PAx*VRR(2,1,5)+r1x2Z*(VRR(1,1,5)-ExZpE*VRR(1,1,6))+WPx*VRR(2,1,6)
            VRR(6,1,5)=PAx*VRR(3,1,5)+WPx*VRR(3,1,6)
            VRR(7,1,5)=PAy*VRR(3,1,5)+r1x2Z*(VRR(1,1,5)-ExZpE*VRR(1,1,6))+WPy*VRR(3,1,6)
            VRR(8,1,5)=PAx*VRR(4,1,5)+WPx*VRR(4,1,6)
            VRR(9,1,5)=PAy*VRR(4,1,5)+WPy*VRR(4,1,6)
            VRR(10,1,5)=PAz*VRR(4,1,5)+r1x2Z*(VRR(1,1,5)-ExZpE*VRR(1,1,6))+WPz*VRR(4,1,6)
            ! Generating (d0|s0)^(4)
            VRR(5,1,4)=PAx*VRR(2,1,4)+r1x2Z*(VRR(1,1,4)-ExZpE*VRR(1,1,5))+WPx*VRR(2,1,5)
            VRR(6,1,4)=PAx*VRR(3,1,4)+WPx*VRR(3,1,5)
            VRR(7,1,4)=PAy*VRR(3,1,4)+r1x2Z*(VRR(1,1,4)-ExZpE*VRR(1,1,5))+WPy*VRR(3,1,5)
            VRR(8,1,4)=PAx*VRR(4,1,4)+WPx*VRR(4,1,5)
            VRR(9,1,4)=PAy*VRR(4,1,4)+WPy*VRR(4,1,5)
            VRR(10,1,4)=PAz*VRR(4,1,4)+r1x2Z*(VRR(1,1,4)-ExZpE*VRR(1,1,5))+WPz*VRR(4,1,5)
            ! Generating (d0|s0)^(3)
            VRR(5,1,3)=PAx*VRR(2,1,3)+r1x2Z*(VRR(1,1,3)-ExZpE*VRR(1,1,4))+WPx*VRR(2,1,4)
            VRR(6,1,3)=PAx*VRR(3,1,3)+WPx*VRR(3,1,4)
            VRR(7,1,3)=PAy*VRR(3,1,3)+r1x2Z*(VRR(1,1,3)-ExZpE*VRR(1,1,4))+WPy*VRR(3,1,4)
            VRR(8,1,3)=PAx*VRR(4,1,3)+WPx*VRR(4,1,4)
            VRR(9,1,3)=PAy*VRR(4,1,3)+WPy*VRR(4,1,4)
            VRR(10,1,3)=PAz*VRR(4,1,3)+r1x2Z*(VRR(1,1,3)-ExZpE*VRR(1,1,4))+WPz*VRR(4,1,4)
            ! Generating (d0|s0)^(2)
            VRR(5,1,2)=PAx*VRR(2,1,2)+r1x2Z*(VRR(1,1,2)-ExZpE*VRR(1,1,3))+WPx*VRR(2,1,3)
            VRR(6,1,2)=PAx*VRR(3,1,2)+WPx*VRR(3,1,3)
            VRR(7,1,2)=PAy*VRR(3,1,2)+r1x2Z*(VRR(1,1,2)-ExZpE*VRR(1,1,3))+WPy*VRR(3,1,3)
            VRR(8,1,2)=PAx*VRR(4,1,2)+WPx*VRR(4,1,3)
            VRR(9,1,2)=PAy*VRR(4,1,2)+WPy*VRR(4,1,3)
            VRR(10,1,2)=PAz*VRR(4,1,2)+r1x2Z*(VRR(1,1,2)-ExZpE*VRR(1,1,3))+WPz*VRR(4,1,3)
            ! Generating (d0|s0)^(1)
            VRR(5,1,1)=PAx*VRR(2,1,1)+r1x2Z*(VRR(1,1,1)-ExZpE*VRR(1,1,2))+WPx*VRR(2,1,2)
            VRR(6,1,1)=PAx*VRR(3,1,1)+WPx*VRR(3,1,2)
            VRR(7,1,1)=PAy*VRR(3,1,1)+r1x2Z*(VRR(1,1,1)-ExZpE*VRR(1,1,2))+WPy*VRR(3,1,2)
            VRR(8,1,1)=PAx*VRR(4,1,1)+WPx*VRR(4,1,2)
            VRR(9,1,1)=PAy*VRR(4,1,1)+WPy*VRR(4,1,2)
            VRR(10,1,1)=PAz*VRR(4,1,1)+r1x2Z*(VRR(1,1,1)-ExZpE*VRR(1,1,2))+WPz*VRR(4,1,2)
            ! Generating (d0|s0)^(0)
            VRR(5,1,0)=PAx*VRR(2,1,0)+r1x2Z*(VRR(1,1,0)-ExZpE*VRR(1,1,1))+WPx*VRR(2,1,1)
            VRR(6,1,0)=PAx*VRR(3,1,0)+WPx*VRR(3,1,1)
            VRR(7,1,0)=PAy*VRR(3,1,0)+r1x2Z*(VRR(1,1,0)-ExZpE*VRR(1,1,1))+WPy*VRR(3,1,1)
            VRR(8,1,0)=PAx*VRR(4,1,0)+WPx*VRR(4,1,1)
            VRR(9,1,0)=PAy*VRR(4,1,0)+WPy*VRR(4,1,1)
            VRR(10,1,0)=PAz*VRR(4,1,0)+r1x2Z*(VRR(1,1,0)-ExZpE*VRR(1,1,1))+WPz*VRR(4,1,1)
            ! Generating (f0|s0)^(7)
            CALL VRRf0s0(84,35,VRR(1,1,7),VRR(1,1,8))
            ! Generating (f0|s0)^(6)
            CALL VRRf0s0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (f0|s0)^(5)
            CALL VRRf0s0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (f0|s0)^(4)
            CALL VRRf0s0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (f0|s0)^(3)
            CALL VRRf0s0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (f0|s0)^(2)
            CALL VRRf0s0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (f0|s0)^(1)
            CALL VRRf0s0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (f0|s0)^(0)
            CALL VRRf0s0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (g0|s0)^(6)
            CALL VRRg0s0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (g0|s0)^(5)
            CALL VRRg0s0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (g0|s0)^(4)
            CALL VRRg0s0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (g0|s0)^(3)
            CALL VRRg0s0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (g0|s0)^(2)
            CALL VRRg0s0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (g0|s0)^(1)
            CALL VRRg0s0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (g0|s0)^(0)
            CALL VRRg0s0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (h0|s0)^(5)
            CALL VRRh0s0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (h0|s0)^(4)
            CALL VRRh0s0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (h0|s0)^(3)
            CALL VRRh0s0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (h0|s0)^(2)
            CALL VRRh0s0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (h0|s0)^(1)
            CALL VRRh0s0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (h0|s0)^(0)
            CALL VRRh0s0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (i0|s0)^(4)
            CALL VRRi0s0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (i0|s0)^(3)
            CALL VRRi0s0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (i0|s0)^(2)
            CALL VRRi0s0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (i0|s0)^(1)
            CALL VRRi0s0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (i0|s0)^(0)
            CALL VRRi0s0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (s0|p0)^(9)
            VRR(1,2,9)=QCx*VRR(1,1,9)+WQx*VRR(1,1,10)
            VRR(1,3,9)=QCy*VRR(1,1,9)+WQy*VRR(1,1,10)
            VRR(1,4,9)=QCz*VRR(1,1,9)+WQz*VRR(1,1,10)
            ! Generating (s0|p0)^(8)
            VRR(1,2,8)=QCx*VRR(1,1,8)+WQx*VRR(1,1,9)
            VRR(1,3,8)=QCy*VRR(1,1,8)+WQy*VRR(1,1,9)
            VRR(1,4,8)=QCz*VRR(1,1,8)+WQz*VRR(1,1,9)
            ! Generating (s0|p0)^(7)
            VRR(1,2,7)=QCx*VRR(1,1,7)+WQx*VRR(1,1,8)
            VRR(1,3,7)=QCy*VRR(1,1,7)+WQy*VRR(1,1,8)
            VRR(1,4,7)=QCz*VRR(1,1,7)+WQz*VRR(1,1,8)
            ! Generating (s0|p0)^(6)
            VRR(1,2,6)=QCx*VRR(1,1,6)+WQx*VRR(1,1,7)
            VRR(1,3,6)=QCy*VRR(1,1,6)+WQy*VRR(1,1,7)
            VRR(1,4,6)=QCz*VRR(1,1,6)+WQz*VRR(1,1,7)
            ! Generating (s0|p0)^(5)
            VRR(1,2,5)=QCx*VRR(1,1,5)+WQx*VRR(1,1,6)
            VRR(1,3,5)=QCy*VRR(1,1,5)+WQy*VRR(1,1,6)
            VRR(1,4,5)=QCz*VRR(1,1,5)+WQz*VRR(1,1,6)
            ! Generating (s0|p0)^(4)
            VRR(1,2,4)=QCx*VRR(1,1,4)+WQx*VRR(1,1,5)
            VRR(1,3,4)=QCy*VRR(1,1,4)+WQy*VRR(1,1,5)
            VRR(1,4,4)=QCz*VRR(1,1,4)+WQz*VRR(1,1,5)
            ! Generating (s0|p0)^(3)
            VRR(1,2,3)=QCx*VRR(1,1,3)+WQx*VRR(1,1,4)
            VRR(1,3,3)=QCy*VRR(1,1,3)+WQy*VRR(1,1,4)
            VRR(1,4,3)=QCz*VRR(1,1,3)+WQz*VRR(1,1,4)
            ! Generating (s0|p0)^(2)
            VRR(1,2,2)=QCx*VRR(1,1,2)+WQx*VRR(1,1,3)
            VRR(1,3,2)=QCy*VRR(1,1,2)+WQy*VRR(1,1,3)
            VRR(1,4,2)=QCz*VRR(1,1,2)+WQz*VRR(1,1,3)
            ! Generating (s0|p0)^(1)
            VRR(1,2,1)=QCx*VRR(1,1,1)+WQx*VRR(1,1,2)
            VRR(1,3,1)=QCy*VRR(1,1,1)+WQy*VRR(1,1,2)
            VRR(1,4,1)=QCz*VRR(1,1,1)+WQz*VRR(1,1,2)
            ! Generating (s0|p0)^(0)
            VRR(1,2,0)=QCx*VRR(1,1,0)+WQx*VRR(1,1,1)
            VRR(1,3,0)=QCy*VRR(1,1,0)+WQy*VRR(1,1,1)
            VRR(1,4,0)=QCz*VRR(1,1,0)+WQz*VRR(1,1,1)
            ! Generating (p0|p0)^(8)
            VRR(2,2,8)=QCx*VRR(2,1,8)+HfxZpE*VRR(1,1,9)+WQx*VRR(2,1,9)
            VRR(2,3,8)=QCy*VRR(2,1,8)+WQy*VRR(2,1,9)
            VRR(2,4,8)=QCz*VRR(2,1,8)+WQz*VRR(2,1,9)
            VRR(3,2,8)=QCx*VRR(3,1,8)+WQx*VRR(3,1,9)
            VRR(3,3,8)=QCy*VRR(3,1,8)+HfxZpE*VRR(1,1,9)+WQy*VRR(3,1,9)
            VRR(3,4,8)=QCz*VRR(3,1,8)+WQz*VRR(3,1,9)
            VRR(4,2,8)=QCx*VRR(4,1,8)+WQx*VRR(4,1,9)
            VRR(4,3,8)=QCy*VRR(4,1,8)+WQy*VRR(4,1,9)
            VRR(4,4,8)=QCz*VRR(4,1,8)+HfxZpE*VRR(1,1,9)+WQz*VRR(4,1,9)
            ! Generating (p0|p0)^(7)
            VRR(2,2,7)=QCx*VRR(2,1,7)+HfxZpE*VRR(1,1,8)+WQx*VRR(2,1,8)
            VRR(2,3,7)=QCy*VRR(2,1,7)+WQy*VRR(2,1,8)
            VRR(2,4,7)=QCz*VRR(2,1,7)+WQz*VRR(2,1,8)
            VRR(3,2,7)=QCx*VRR(3,1,7)+WQx*VRR(3,1,8)
            VRR(3,3,7)=QCy*VRR(3,1,7)+HfxZpE*VRR(1,1,8)+WQy*VRR(3,1,8)
            VRR(3,4,7)=QCz*VRR(3,1,7)+WQz*VRR(3,1,8)
            VRR(4,2,7)=QCx*VRR(4,1,7)+WQx*VRR(4,1,8)
            VRR(4,3,7)=QCy*VRR(4,1,7)+WQy*VRR(4,1,8)
            VRR(4,4,7)=QCz*VRR(4,1,7)+HfxZpE*VRR(1,1,8)+WQz*VRR(4,1,8)
            ! Generating (p0|p0)^(6)
            VRR(2,2,6)=QCx*VRR(2,1,6)+HfxZpE*VRR(1,1,7)+WQx*VRR(2,1,7)
            VRR(2,3,6)=QCy*VRR(2,1,6)+WQy*VRR(2,1,7)
            VRR(2,4,6)=QCz*VRR(2,1,6)+WQz*VRR(2,1,7)
            VRR(3,2,6)=QCx*VRR(3,1,6)+WQx*VRR(3,1,7)
            VRR(3,3,6)=QCy*VRR(3,1,6)+HfxZpE*VRR(1,1,7)+WQy*VRR(3,1,7)
            VRR(3,4,6)=QCz*VRR(3,1,6)+WQz*VRR(3,1,7)
            VRR(4,2,6)=QCx*VRR(4,1,6)+WQx*VRR(4,1,7)
            VRR(4,3,6)=QCy*VRR(4,1,6)+WQy*VRR(4,1,7)
            VRR(4,4,6)=QCz*VRR(4,1,6)+HfxZpE*VRR(1,1,7)+WQz*VRR(4,1,7)
            ! Generating (p0|p0)^(5)
            VRR(2,2,5)=QCx*VRR(2,1,5)+HfxZpE*VRR(1,1,6)+WQx*VRR(2,1,6)
            VRR(2,3,5)=QCy*VRR(2,1,5)+WQy*VRR(2,1,6)
            VRR(2,4,5)=QCz*VRR(2,1,5)+WQz*VRR(2,1,6)
            VRR(3,2,5)=QCx*VRR(3,1,5)+WQx*VRR(3,1,6)
            VRR(3,3,5)=QCy*VRR(3,1,5)+HfxZpE*VRR(1,1,6)+WQy*VRR(3,1,6)
            VRR(3,4,5)=QCz*VRR(3,1,5)+WQz*VRR(3,1,6)
            VRR(4,2,5)=QCx*VRR(4,1,5)+WQx*VRR(4,1,6)
            VRR(4,3,5)=QCy*VRR(4,1,5)+WQy*VRR(4,1,6)
            VRR(4,4,5)=QCz*VRR(4,1,5)+HfxZpE*VRR(1,1,6)+WQz*VRR(4,1,6)
            ! Generating (p0|p0)^(4)
            VRR(2,2,4)=QCx*VRR(2,1,4)+HfxZpE*VRR(1,1,5)+WQx*VRR(2,1,5)
            VRR(2,3,4)=QCy*VRR(2,1,4)+WQy*VRR(2,1,5)
            VRR(2,4,4)=QCz*VRR(2,1,4)+WQz*VRR(2,1,5)
            VRR(3,2,4)=QCx*VRR(3,1,4)+WQx*VRR(3,1,5)
            VRR(3,3,4)=QCy*VRR(3,1,4)+HfxZpE*VRR(1,1,5)+WQy*VRR(3,1,5)
            VRR(3,4,4)=QCz*VRR(3,1,4)+WQz*VRR(3,1,5)
            VRR(4,2,4)=QCx*VRR(4,1,4)+WQx*VRR(4,1,5)
            VRR(4,3,4)=QCy*VRR(4,1,4)+WQy*VRR(4,1,5)
            VRR(4,4,4)=QCz*VRR(4,1,4)+HfxZpE*VRR(1,1,5)+WQz*VRR(4,1,5)
            ! Generating (p0|p0)^(3)
            VRR(2,2,3)=QCx*VRR(2,1,3)+HfxZpE*VRR(1,1,4)+WQx*VRR(2,1,4)
            VRR(2,3,3)=QCy*VRR(2,1,3)+WQy*VRR(2,1,4)
            VRR(2,4,3)=QCz*VRR(2,1,3)+WQz*VRR(2,1,4)
            VRR(3,2,3)=QCx*VRR(3,1,3)+WQx*VRR(3,1,4)
            VRR(3,3,3)=QCy*VRR(3,1,3)+HfxZpE*VRR(1,1,4)+WQy*VRR(3,1,4)
            VRR(3,4,3)=QCz*VRR(3,1,3)+WQz*VRR(3,1,4)
            VRR(4,2,3)=QCx*VRR(4,1,3)+WQx*VRR(4,1,4)
            VRR(4,3,3)=QCy*VRR(4,1,3)+WQy*VRR(4,1,4)
            VRR(4,4,3)=QCz*VRR(4,1,3)+HfxZpE*VRR(1,1,4)+WQz*VRR(4,1,4)
            ! Generating (p0|p0)^(2)
            VRR(2,2,2)=QCx*VRR(2,1,2)+HfxZpE*VRR(1,1,3)+WQx*VRR(2,1,3)
            VRR(2,3,2)=QCy*VRR(2,1,2)+WQy*VRR(2,1,3)
            VRR(2,4,2)=QCz*VRR(2,1,2)+WQz*VRR(2,1,3)
            VRR(3,2,2)=QCx*VRR(3,1,2)+WQx*VRR(3,1,3)
            VRR(3,3,2)=QCy*VRR(3,1,2)+HfxZpE*VRR(1,1,3)+WQy*VRR(3,1,3)
            VRR(3,4,2)=QCz*VRR(3,1,2)+WQz*VRR(3,1,3)
            VRR(4,2,2)=QCx*VRR(4,1,2)+WQx*VRR(4,1,3)
            VRR(4,3,2)=QCy*VRR(4,1,2)+WQy*VRR(4,1,3)
            VRR(4,4,2)=QCz*VRR(4,1,2)+HfxZpE*VRR(1,1,3)+WQz*VRR(4,1,3)
            ! Generating (p0|p0)^(1)
            VRR(2,2,1)=QCx*VRR(2,1,1)+HfxZpE*VRR(1,1,2)+WQx*VRR(2,1,2)
            VRR(2,3,1)=QCy*VRR(2,1,1)+WQy*VRR(2,1,2)
            VRR(2,4,1)=QCz*VRR(2,1,1)+WQz*VRR(2,1,2)
            VRR(3,2,1)=QCx*VRR(3,1,1)+WQx*VRR(3,1,2)
            VRR(3,3,1)=QCy*VRR(3,1,1)+HfxZpE*VRR(1,1,2)+WQy*VRR(3,1,2)
            VRR(3,4,1)=QCz*VRR(3,1,1)+WQz*VRR(3,1,2)
            VRR(4,2,1)=QCx*VRR(4,1,1)+WQx*VRR(4,1,2)
            VRR(4,3,1)=QCy*VRR(4,1,1)+WQy*VRR(4,1,2)
            VRR(4,4,1)=QCz*VRR(4,1,1)+HfxZpE*VRR(1,1,2)+WQz*VRR(4,1,2)
            ! Generating (p0|p0)^(0)
            VRR(2,2,0)=QCx*VRR(2,1,0)+HfxZpE*VRR(1,1,1)+WQx*VRR(2,1,1)
            VRR(2,3,0)=QCy*VRR(2,1,0)+WQy*VRR(2,1,1)
            VRR(2,4,0)=QCz*VRR(2,1,0)+WQz*VRR(2,1,1)
            VRR(3,2,0)=QCx*VRR(3,1,0)+WQx*VRR(3,1,1)
            VRR(3,3,0)=QCy*VRR(3,1,0)+HfxZpE*VRR(1,1,1)+WQy*VRR(3,1,1)
            VRR(3,4,0)=QCz*VRR(3,1,0)+WQz*VRR(3,1,1)
            VRR(4,2,0)=QCx*VRR(4,1,0)+WQx*VRR(4,1,1)
            VRR(4,3,0)=QCy*VRR(4,1,0)+WQy*VRR(4,1,1)
            VRR(4,4,0)=QCz*VRR(4,1,0)+HfxZpE*VRR(1,1,1)+WQz*VRR(4,1,1)
            ! Generating (d0|p0)^(7)
            CALL VRRd0p0(84,35,VRR(1,1,7),VRR(1,1,8))
            ! Generating (d0|p0)^(6)
            CALL VRRd0p0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (d0|p0)^(5)
            CALL VRRd0p0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (d0|p0)^(4)
            CALL VRRd0p0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (d0|p0)^(3)
            CALL VRRd0p0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (d0|p0)^(2)
            CALL VRRd0p0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (d0|p0)^(1)
            CALL VRRd0p0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (d0|p0)^(0)
            CALL VRRd0p0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (f0|p0)^(6)
            CALL VRRf0p0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (f0|p0)^(5)
            CALL VRRf0p0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (f0|p0)^(4)
            CALL VRRf0p0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (f0|p0)^(3)
            CALL VRRf0p0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (f0|p0)^(2)
            CALL VRRf0p0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (f0|p0)^(1)
            CALL VRRf0p0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (f0|p0)^(0)
            CALL VRRf0p0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (g0|p0)^(5)
            CALL VRRg0p0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (g0|p0)^(4)
            CALL VRRg0p0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (g0|p0)^(3)
            CALL VRRg0p0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (g0|p0)^(2)
            CALL VRRg0p0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (g0|p0)^(1)
            CALL VRRg0p0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (g0|p0)^(0)
            CALL VRRg0p0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (h0|p0)^(4)
            CALL VRRh0p0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (h0|p0)^(3)
            CALL VRRh0p0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (h0|p0)^(2)
            CALL VRRh0p0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (h0|p0)^(1)
            CALL VRRh0p0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (h0|p0)^(0)
            CALL VRRh0p0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (i0|p0)^(3)
            CALL VRRi0p0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (i0|p0)^(2)
            CALL VRRi0p0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (i0|p0)^(1)
            CALL VRRi0p0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (i0|p0)^(0)
            CALL VRRi0p0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (s0|d0)^(8)
            CALL VRRs0d0(84,35,VRR(1,1,8),VRR(1,1,9))
            ! Generating (s0|d0)^(7)
            CALL VRRs0d0(84,35,VRR(1,1,7),VRR(1,1,8))
            ! Generating (s0|d0)^(6)
            CALL VRRs0d0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (s0|d0)^(5)
            CALL VRRs0d0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (s0|d0)^(4)
            CALL VRRs0d0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (s0|d0)^(3)
            CALL VRRs0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (s0|d0)^(2)
            CALL VRRs0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (s0|d0)^(1)
            CALL VRRs0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (s0|d0)^(0)
            CALL VRRs0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (p0|d0)^(7)
            CALL VRRp0d0(84,35,VRR(1,1,7),VRR(1,1,8))
            ! Generating (p0|d0)^(6)
            CALL VRRp0d0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (p0|d0)^(5)
            CALL VRRp0d0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (p0|d0)^(4)
            CALL VRRp0d0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (p0|d0)^(3)
            CALL VRRp0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (p0|d0)^(2)
            CALL VRRp0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (p0|d0)^(1)
            CALL VRRp0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (p0|d0)^(0)
            CALL VRRp0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (d0|d0)^(6)
            CALL VRRd0d0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (d0|d0)^(5)
            CALL VRRd0d0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (d0|d0)^(4)
            CALL VRRd0d0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (d0|d0)^(3)
            CALL VRRd0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (d0|d0)^(2)
            CALL VRRd0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (d0|d0)^(1)
            CALL VRRd0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (d0|d0)^(0)
            CALL VRRd0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (f0|d0)^(5)
            CALL VRRf0d0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (f0|d0)^(4)
            CALL VRRf0d0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (f0|d0)^(3)
            CALL VRRf0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (f0|d0)^(2)
            CALL VRRf0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (f0|d0)^(1)
            CALL VRRf0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (f0|d0)^(0)
            CALL VRRf0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (g0|d0)^(4)
            CALL VRRg0d0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (g0|d0)^(3)
            CALL VRRg0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (g0|d0)^(2)
            CALL VRRg0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (g0|d0)^(1)
            CALL VRRg0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (g0|d0)^(0)
            CALL VRRg0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (h0|d0)^(3)
            CALL VRRh0d0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (h0|d0)^(2)
            CALL VRRh0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (h0|d0)^(1)
            CALL VRRh0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (h0|d0)^(0)
            CALL VRRh0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (i0|d0)^(2)
            CALL VRRi0d0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (i0|d0)^(1)
            CALL VRRi0d0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (i0|d0)^(0)
            CALL VRRi0d0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (s0|f0)^(7)
            CALL VRRs0f0(84,35,VRR(1,1,7),VRR(1,1,8))
            ! Generating (s0|f0)^(6)
            CALL VRRs0f0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (s0|f0)^(5)
            CALL VRRs0f0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (s0|f0)^(4)
            CALL VRRs0f0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (s0|f0)^(3)
            CALL VRRs0f0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (s0|f0)^(2)
            CALL VRRs0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (s0|f0)^(1)
            CALL VRRs0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (s0|f0)^(0)
            CALL VRRs0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (p0|f0)^(6)
            CALL VRRp0f0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (p0|f0)^(5)
            CALL VRRp0f0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (p0|f0)^(4)
            CALL VRRp0f0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (p0|f0)^(3)
            CALL VRRp0f0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (p0|f0)^(2)
            CALL VRRp0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (p0|f0)^(1)
            CALL VRRp0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (p0|f0)^(0)
            CALL VRRp0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (d0|f0)^(5)
            CALL VRRd0f0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (d0|f0)^(4)
            CALL VRRd0f0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (d0|f0)^(3)
            CALL VRRd0f0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (d0|f0)^(2)
            CALL VRRd0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (d0|f0)^(1)
            CALL VRRd0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (d0|f0)^(0)
            CALL VRRd0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (f0|f0)^(4)
            CALL VRRf0f0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (f0|f0)^(3)
            CALL VRRf0f0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (f0|f0)^(2)
            CALL VRRf0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (f0|f0)^(1)
            CALL VRRf0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (f0|f0)^(0)
            CALL VRRf0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (g0|f0)^(3)
            CALL VRRg0f0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (g0|f0)^(2)
            CALL VRRg0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (g0|f0)^(1)
            CALL VRRg0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (g0|f0)^(0)
            CALL VRRg0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (h0|f0)^(2)
            CALL VRRh0f0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (h0|f0)^(1)
            CALL VRRh0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (h0|f0)^(0)
            CALL VRRh0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (i0|f0)^(1)
            CALL VRRi0f0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (i0|f0)^(0)
            CALL VRRi0f0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (s0|g0)^(6)
            CALL VRRs0g0(84,35,VRR(1,1,6),VRR(1,1,7))
            ! Generating (s0|g0)^(5)
            CALL VRRs0g0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (s0|g0)^(4)
            CALL VRRs0g0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (s0|g0)^(3)
            CALL VRRs0g0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (s0|g0)^(2)
            CALL VRRs0g0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (s0|g0)^(1)
            CALL VRRs0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (s0|g0)^(0)
            CALL VRRs0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (p0|g0)^(5)
            CALL VRRp0g0(84,35,VRR(1,1,5),VRR(1,1,6))
            ! Generating (p0|g0)^(4)
            CALL VRRp0g0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (p0|g0)^(3)
            CALL VRRp0g0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (p0|g0)^(2)
            CALL VRRp0g0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (p0|g0)^(1)
            CALL VRRp0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (p0|g0)^(0)
            CALL VRRp0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (d0|g0)^(4)
            CALL VRRd0g0(84,35,VRR(1,1,4),VRR(1,1,5))
            ! Generating (d0|g0)^(3)
            CALL VRRd0g0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (d0|g0)^(2)
            CALL VRRd0g0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (d0|g0)^(1)
            CALL VRRd0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (d0|g0)^(0)
            CALL VRRd0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (f0|g0)^(3)
            CALL VRRf0g0(84,35,VRR(1,1,3),VRR(1,1,4))
            ! Generating (f0|g0)^(2)
            CALL VRRf0g0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (f0|g0)^(1)
            CALL VRRf0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (f0|g0)^(0)
            CALL VRRf0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (g0|g0)^(2)
            CALL VRRg0g0(84,35,VRR(1,1,2),VRR(1,1,3))
            ! Generating (g0|g0)^(1)
            CALL VRRg0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (g0|g0)^(0)
            CALL VRRg0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (h0|g0)^(1)
            CALL VRRh0g0(84,35,VRR(1,1,1),VRR(1,1,2))
            ! Generating (h0|g0)^(0)
            CALL VRRh0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Generating (i0|g0)^(0)
            CALL VRRi0g0(84,35,VRR(1,1,0),VRR(1,1,1))
            ! Contracting ...
            CALL CNTRCT1010102(VRR,HRR)
         ENDDO ! (M0| loop
      ENDDO ! |N0) loop
      ! Generating (f,0|f,sp)^(0)
      CALL KetHRR102(84,HRR)
      ! Generating (f,f|f,sp)^(0)
      DO L=1,4
         DO K=11,20
            CDOffSet=(OC+K-11)*LDC+(OD+L-1)*LDD
            CALL BraHRR1010(OA,OB,LDA,LDB,CDOffSet,HRR(1,K,L),INTGRL)
          ENDDO
      ENDDO
    END SUBROUTINE IntB10101002
    SUBROUTINE CNTRCT1010102(VRR,HRR)
      USE DerivedTypes
      USE VScratchB
      REAL(DOUBLE)  :: VRR(84,35,0:10)
      REAL(DOUBLE)  :: HRR(84,45,4)
      HRR(1,1,1)=HRR(1,1,1)+VRR(1,1,0)
      HRR(1,2,1)=HRR(1,2,1)+VRR(1,2,0)
      HRR(1,3,1)=HRR(1,3,1)+VRR(1,3,0)
      HRR(1,4,1)=HRR(1,4,1)+VRR(1,4,0)
      HRR(1,5,1)=HRR(1,5,1)+VRR(1,5,0)
      HRR(1,6,1)=HRR(1,6,1)+VRR(1,6,0)
      HRR(1,7,1)=HRR(1,7,1)+VRR(1,7,0)
      HRR(1,8,1)=HRR(1,8,1)+VRR(1,8,0)
      HRR(1,9,1)=HRR(1,9,1)+VRR(1,9,0)
      HRR(1,10,1)=HRR(1,10,1)+VRR(1,10,0)
      HRR(1,11,1)=HRR(1,11,1)+VRR(1,11,0)
      HRR(1,36,1)=HRR(1,36,1)+FnSpK*VRR(1,11,0)
      HRR(1,12,1)=HRR(1,12,1)+VRR(1,12,0)
      HRR(1,37,1)=HRR(1,37,1)+FnSpK*VRR(1,12,0)
      HRR(1,13,1)=HRR(1,13,1)+VRR(1,13,0)
      HRR(1,38,1)=HRR(1,38,1)+FnSpK*VRR(1,13,0)
      HRR(1,14,1)=HRR(1,14,1)+VRR(1,14,0)
      HRR(1,39,1)=HRR(1,39,1)+FnSpK*VRR(1,14,0)
      HRR(1,15,1)=HRR(1,15,1)+VRR(1,15,0)
      HRR(1,40,1)=HRR(1,40,1)+FnSpK*VRR(1,15,0)
      HRR(1,16,1)=HRR(1,16,1)+VRR(1,16,0)
      HRR(1,41,1)=HRR(1,41,1)+FnSpK*VRR(1,16,0)
      HRR(1,17,1)=HRR(1,17,1)+VRR(1,17,0)
      HRR(1,42,1)=HRR(1,42,1)+FnSpK*VRR(1,17,0)
      HRR(1,18,1)=HRR(1,18,1)+VRR(1,18,0)
      HRR(1,43,1)=HRR(1,43,1)+FnSpK*VRR(1,18,0)
      HRR(1,19,1)=HRR(1,19,1)+VRR(1,19,0)
      HRR(1,44,1)=HRR(1,44,1)+FnSpK*VRR(1,19,0)
      HRR(1,20,1)=HRR(1,20,1)+VRR(1,20,0)
      HRR(1,45,1)=HRR(1,45,1)+FnSpK*VRR(1,20,0)
      HRR(1,21,1)=HRR(1,21,1)+VRR(1,21,0)
      HRR(1,22,1)=HRR(1,22,1)+VRR(1,22,0)
      HRR(1,23,1)=HRR(1,23,1)+VRR(1,23,0)
      HRR(1,24,1)=HRR(1,24,1)+VRR(1,24,0)
      HRR(1,25,1)=HRR(1,25,1)+VRR(1,25,0)
      HRR(1,26,1)=HRR(1,26,1)+VRR(1,26,0)
      HRR(1,27,1)=HRR(1,27,1)+VRR(1,27,0)
      HRR(1,28,1)=HRR(1,28,1)+VRR(1,28,0)
      HRR(1,29,1)=HRR(1,29,1)+VRR(1,29,0)
      HRR(1,30,1)=HRR(1,30,1)+VRR(1,30,0)
      HRR(1,31,1)=HRR(1,31,1)+VRR(1,31,0)
      HRR(1,32,1)=HRR(1,32,1)+VRR(1,32,0)
      HRR(1,33,1)=HRR(1,33,1)+VRR(1,33,0)
      HRR(1,34,1)=HRR(1,34,1)+VRR(1,34,0)
      HRR(1,35,1)=HRR(1,35,1)+VRR(1,35,0)
      HRR(2,1,1)=HRR(2,1,1)+VRR(2,1,0)
      HRR(2,2,1)=HRR(2,2,1)+VRR(2,2,0)
      HRR(2,3,1)=HRR(2,3,1)+VRR(2,3,0)
      HRR(2,4,1)=HRR(2,4,1)+VRR(2,4,0)
      HRR(2,5,1)=HRR(2,5,1)+VRR(2,5,0)
      HRR(2,6,1)=HRR(2,6,1)+VRR(2,6,0)
      HRR(2,7,1)=HRR(2,7,1)+VRR(2,7,0)
      HRR(2,8,1)=HRR(2,8,1)+VRR(2,8,0)
      HRR(2,9,1)=HRR(2,9,1)+VRR(2,9,0)
      HRR(2,10,1)=HRR(2,10,1)+VRR(2,10,0)
      HRR(2,11,1)=HRR(2,11,1)+VRR(2,11,0)
      HRR(2,36,1)=HRR(2,36,1)+FnSpK*VRR(2,11,0)
      HRR(2,12,1)=HRR(2,12,1)+VRR(2,12,0)
      HRR(2,37,1)=HRR(2,37,1)+FnSpK*VRR(2,12,0)
      HRR(2,13,1)=HRR(2,13,1)+VRR(2,13,0)
      HRR(2,38,1)=HRR(2,38,1)+FnSpK*VRR(2,13,0)
      HRR(2,14,1)=HRR(2,14,1)+VRR(2,14,0)
      HRR(2,39,1)=HRR(2,39,1)+FnSpK*VRR(2,14,0)
      HRR(2,15,1)=HRR(2,15,1)+VRR(2,15,0)
      HRR(2,40,1)=HRR(2,40,1)+FnSpK*VRR(2,15,0)
      HRR(2,16,1)=HRR(2,16,1)+VRR(2,16,0)
      HRR(2,41,1)=HRR(2,41,1)+FnSpK*VRR(2,16,0)
      HRR(2,17,1)=HRR(2,17,1)+VRR(2,17,0)
      HRR(2,42,1)=HRR(2,42,1)+FnSpK*VRR(2,17,0)
      HRR(2,18,1)=HRR(2,18,1)+VRR(2,18,0)
      HRR(2,43,1)=HRR(2,43,1)+FnSpK*VRR(2,18,0)
      HRR(2,19,1)=HRR(2,19,1)+VRR(2,19,0)
      HRR(2,44,1)=HRR(2,44,1)+FnSpK*VRR(2,19,0)
      HRR(2,20,1)=HRR(2,20,1)+VRR(2,20,0)
      HRR(2,45,1)=HRR(2,45,1)+FnSpK*VRR(2,20,0)
      HRR(2,21,1)=HRR(2,21,1)+VRR(2,21,0)
      HRR(2,22,1)=HRR(2,22,1)+VRR(2,22,0)
      HRR(2,23,1)=HRR(2,23,1)+VRR(2,23,0)
      HRR(2,24,1)=HRR(2,24,1)+VRR(2,24,0)
      HRR(2,25,1)=HRR(2,25,1)+VRR(2,25,0)
      HRR(2,26,1)=HRR(2,26,1)+VRR(2,26,0)
      HRR(2,27,1)=HRR(2,27,1)+VRR(2,27,0)
      HRR(2,28,1)=HRR(2,28,1)+VRR(2,28,0)
      HRR(2,29,1)=HRR(2,29,1)+VRR(2,29,0)
      HRR(2,30,1)=HRR(2,30,1)+VRR(2,30,0)
      HRR(2,31,1)=HRR(2,31,1)+VRR(2,31,0)
      HRR(2,32,1)=HRR(2,32,1)+VRR(2,32,0)
      HRR(2,33,1)=HRR(2,33,1)+VRR(2,33,0)
      HRR(2,34,1)=HRR(2,34,1)+VRR(2,34,0)
      HRR(2,35,1)=HRR(2,35,1)+VRR(2,35,0)
      HRR(3,1,1)=HRR(3,1,1)+VRR(3,1,0)
      HRR(3,2,1)=HRR(3,2,1)+VRR(3,2,0)
      HRR(3,3,1)=HRR(3,3,1)+VRR(3,3,0)
      HRR(3,4,1)=HRR(3,4,1)+VRR(3,4,0)
      HRR(3,5,1)=HRR(3,5,1)+VRR(3,5,0)
      HRR(3,6,1)=HRR(3,6,1)+VRR(3,6,0)
      HRR(3,7,1)=HRR(3,7,1)+VRR(3,7,0)
      HRR(3,8,1)=HRR(3,8,1)+VRR(3,8,0)
      HRR(3,9,1)=HRR(3,9,1)+VRR(3,9,0)
      HRR(3,10,1)=HRR(3,10,1)+VRR(3,10,0)
      HRR(3,11,1)=HRR(3,11,1)+VRR(3,11,0)
      HRR(3,36,1)=HRR(3,36,1)+FnSpK*VRR(3,11,0)
      HRR(3,12,1)=HRR(3,12,1)+VRR(3,12,0)
      HRR(3,37,1)=HRR(3,37,1)+FnSpK*VRR(3,12,0)
      HRR(3,13,1)=HRR(3,13,1)+VRR(3,13,0)
      HRR(3,38,1)=HRR(3,38,1)+FnSpK*VRR(3,13,0)
      HRR(3,14,1)=HRR(3,14,1)+VRR(3,14,0)
      HRR(3,39,1)=HRR(3,39,1)+FnSpK*VRR(3,14,0)
      HRR(3,15,1)=HRR(3,15,1)+VRR(3,15,0)
      HRR(3,40,1)=HRR(3,40,1)+FnSpK*VRR(3,15,0)
      HRR(3,16,1)=HRR(3,16,1)+VRR(3,16,0)
      HRR(3,41,1)=HRR(3,41,1)+FnSpK*VRR(3,16,0)
      HRR(3,17,1)=HRR(3,17,1)+VRR(3,17,0)
      HRR(3,42,1)=HRR(3,42,1)+FnSpK*VRR(3,17,0)
      HRR(3,18,1)=HRR(3,18,1)+VRR(3,18,0)
      HRR(3,43,1)=HRR(3,43,1)+FnSpK*VRR(3,18,0)
      HRR(3,19,1)=HRR(3,19,1)+VRR(3,19,0)
      HRR(3,44,1)=HRR(3,44,1)+FnSpK*VRR(3,19,0)
      HRR(3,20,1)=HRR(3,20,1)+VRR(3,20,0)
      HRR(3,45,1)=HRR(3,45,1)+FnSpK*VRR(3,20,0)
      HRR(3,21,1)=HRR(3,21,1)+VRR(3,21,0)
      HRR(3,22,1)=HRR(3,22,1)+VRR(3,22,0)
      HRR(3,23,1)=HRR(3,23,1)+VRR(3,23,0)
      HRR(3,24,1)=HRR(3,24,1)+VRR(3,24,0)
      HRR(3,25,1)=HRR(3,25,1)+VRR(3,25,0)
      HRR(3,26,1)=HRR(3,26,1)+VRR(3,26,0)
      HRR(3,27,1)=HRR(3,27,1)+VRR(3,27,0)
      HRR(3,28,1)=HRR(3,28,1)+VRR(3,28,0)
      HRR(3,29,1)=HRR(3,29,1)+VRR(3,29,0)
      HRR(3,30,1)=HRR(3,30,1)+VRR(3,30,0)
      HRR(3,31,1)=HRR(3,31,1)+VRR(3,31,0)
      HRR(3,32,1)=HRR(3,32,1)+VRR(3,32,0)
      HRR(3,33,1)=HRR(3,33,1)+VRR(3,33,0)
      HRR(3,34,1)=HRR(3,34,1)+VRR(3,34,0)
      HRR(3,35,1)=HRR(3,35,1)+VRR(3,35,0)
      HRR(4,1,1)=HRR(4,1,1)+VRR(4,1,0)
      HRR(4,2,1)=HRR(4,2,1)+VRR(4,2,0)
      HRR(4,3,1)=HRR(4,3,1)+VRR(4,3,0)
      HRR(4,4,1)=HRR(4,4,1)+VRR(4,4,0)
      HRR(4,5,1)=HRR(4,5,1)+VRR(4,5,0)
      HRR(4,6,1)=HRR(4,6,1)+VRR(4,6,0)
      HRR(4,7,1)=HRR(4,7,1)+VRR(4,7,0)
      HRR(4,8,1)=HRR(4,8,1)+VRR(4,8,0)
      HRR(4,9,1)=HRR(4,9,1)+VRR(4,9,0)
      HRR(4,10,1)=HRR(4,10,1)+VRR(4,10,0)
      HRR(4,11,1)=HRR(4,11,1)+VRR(4,11,0)
      HRR(4,36,1)=HRR(4,36,1)+FnSpK*VRR(4,11,0)
      HRR(4,12,1)=HRR(4,12,1)+VRR(4,12,0)
      HRR(4,37,1)=HRR(4,37,1)+FnSpK*VRR(4,12,0)
      HRR(4,13,1)=HRR(4,13,1)+VRR(4,13,0)
      HRR(4,38,1)=HRR(4,38,1)+FnSpK*VRR(4,13,0)
      HRR(4,14,1)=HRR(4,14,1)+VRR(4,14,0)
      HRR(4,39,1)=HRR(4,39,1)+FnSpK*VRR(4,14,0)
      HRR(4,15,1)=HRR(4,15,1)+VRR(4,15,0)
      HRR(4,40,1)=HRR(4,40,1)+FnSpK*VRR(4,15,0)
      HRR(4,16,1)=HRR(4,16,1)+VRR(4,16,0)
      HRR(4,41,1)=HRR(4,41,1)+FnSpK*VRR(4,16,0)
      HRR(4,17,1)=HRR(4,17,1)+VRR(4,17,0)
      HRR(4,42,1)=HRR(4,42,1)+FnSpK*VRR(4,17,0)
      HRR(4,18,1)=HRR(4,18,1)+VRR(4,18,0)
      HRR(4,43,1)=HRR(4,43,1)+FnSpK*VRR(4,18,0)
      HRR(4,19,1)=HRR(4,19,1)+VRR(4,19,0)
      HRR(4,44,1)=HRR(4,44,1)+FnSpK*VRR(4,19,0)
      HRR(4,20,1)=HRR(4,20,1)+VRR(4,20,0)
      HRR(4,45,1)=HRR(4,45,1)+FnSpK*VRR(4,20,0)
      HRR(4,21,1)=HRR(4,21,1)+VRR(4,21,0)
      HRR(4,22,1)=HRR(4,22,1)+VRR(4,22,0)
      HRR(4,23,1)=HRR(4,23,1)+VRR(4,23,0)
      HRR(4,24,1)=HRR(4,24,1)+VRR(4,24,0)
      HRR(4,25,1)=HRR(4,25,1)+VRR(4,25,0)
      HRR(4,26,1)=HRR(4,26,1)+VRR(4,26,0)
      HRR(4,27,1)=HRR(4,27,1)+VRR(4,27,0)
      HRR(4,28,1)=HRR(4,28,1)+VRR(4,28,0)
      HRR(4,29,1)=HRR(4,29,1)+VRR(4,29,0)
      HRR(4,30,1)=HRR(4,30,1)+VRR(4,30,0)
      HRR(4,31,1)=HRR(4,31,1)+VRR(4,31,0)
      HRR(4,32,1)=HRR(4,32,1)+VRR(4,32,0)
      HRR(4,33,1)=HRR(4,33,1)+VRR(4,33,0)
      HRR(4,34,1)=HRR(4,34,1)+VRR(4,34,0)
      HRR(4,35,1)=HRR(4,35,1)+VRR(4,35,0)
      HRR(5,1,1)=HRR(5,1,1)+VRR(5,1,0)
      HRR(5,2,1)=HRR(5,2,1)+VRR(5,2,0)
      HRR(5,3,1)=HRR(5,3,1)+VRR(5,3,0)
      HRR(5,4,1)=HRR(5,4,1)+VRR(5,4,0)
      HRR(5,5,1)=HRR(5,5,1)+VRR(5,5,0)
      HRR(5,6,1)=HRR(5,6,1)+VRR(5,6,0)
      HRR(5,7,1)=HRR(5,7,1)+VRR(5,7,0)
      HRR(5,8,1)=HRR(5,8,1)+VRR(5,8,0)
      HRR(5,9,1)=HRR(5,9,1)+VRR(5,9,0)
      HRR(5,10,1)=HRR(5,10,1)+VRR(5,10,0)
      HRR(5,11,1)=HRR(5,11,1)+VRR(5,11,0)
      HRR(5,36,1)=HRR(5,36,1)+FnSpK*VRR(5,11,0)
      HRR(5,12,1)=HRR(5,12,1)+VRR(5,12,0)
      HRR(5,37,1)=HRR(5,37,1)+FnSpK*VRR(5,12,0)
      HRR(5,13,1)=HRR(5,13,1)+VRR(5,13,0)
      HRR(5,38,1)=HRR(5,38,1)+FnSpK*VRR(5,13,0)
      HRR(5,14,1)=HRR(5,14,1)+VRR(5,14,0)
      HRR(5,39,1)=HRR(5,39,1)+FnSpK*VRR(5,14,0)
      HRR(5,15,1)=HRR(5,15,1)+VRR(5,15,0)
      HRR(5,40,1)=HRR(5,40,1)+FnSpK*VRR(5,15,0)
      HRR(5,16,1)=HRR(5,16,1)+VRR(5,16,0)
      HRR(5,41,1)=HRR(5,41,1)+FnSpK*VRR(5,16,0)
      HRR(5,17,1)=HRR(5,17,1)+VRR(5,17,0)
      HRR(5,42,1)=HRR(5,42,1)+FnSpK*VRR(5,17,0)
      HRR(5,18,1)=HRR(5,18,1)+VRR(5,18,0)
      HRR(5,43,1)=HRR(5,43,1)+FnSpK*VRR(5,18,0)
      HRR(5,19,1)=HRR(5,19,1)+VRR(5,19,0)
      HRR(5,44,1)=HRR(5,44,1)+FnSpK*VRR(5,19,0)
      HRR(5,20,1)=HRR(5,20,1)+VRR(5,20,0)
      HRR(5,45,1)=HRR(5,45,1)+FnSpK*VRR(5,20,0)
      HRR(5,21,1)=HRR(5,21,1)+VRR(5,21,0)
      HRR(5,22,1)=HRR(5,22,1)+VRR(5,22,0)
      HRR(5,23,1)=HRR(5,23,1)+VRR(5,23,0)
      HRR(5,24,1)=HRR(5,24,1)+VRR(5,24,0)
      HRR(5,25,1)=HRR(5,25,1)+VRR(5,25,0)
      HRR(5,26,1)=HRR(5,26,1)+VRR(5,26,0)
      HRR(5,27,1)=HRR(5,27,1)+VRR(5,27,0)
      HRR(5,28,1)=HRR(5,28,1)+VRR(5,28,0)
      HRR(5,29,1)=HRR(5,29,1)+VRR(5,29,0)
      HRR(5,30,1)=HRR(5,30,1)+VRR(5,30,0)
      HRR(5,31,1)=HRR(5,31,1)+VRR(5,31,0)
      HRR(5,32,1)=HRR(5,32,1)+VRR(5,32,0)
      HRR(5,33,1)=HRR(5,33,1)+VRR(5,33,0)
      HRR(5,34,1)=HRR(5,34,1)+VRR(5,34,0)
      HRR(5,35,1)=HRR(5,35,1)+VRR(5,35,0)
      HRR(6,1,1)=HRR(6,1,1)+VRR(6,1,0)
      HRR(6,2,1)=HRR(6,2,1)+VRR(6,2,0)
      HRR(6,3,1)=HRR(6,3,1)+VRR(6,3,0)
      HRR(6,4,1)=HRR(6,4,1)+VRR(6,4,0)
      HRR(6,5,1)=HRR(6,5,1)+VRR(6,5,0)
      HRR(6,6,1)=HRR(6,6,1)+VRR(6,6,0)
      HRR(6,7,1)=HRR(6,7,1)+VRR(6,7,0)
      HRR(6,8,1)=HRR(6,8,1)+VRR(6,8,0)
      HRR(6,9,1)=HRR(6,9,1)+VRR(6,9,0)
      HRR(6,10,1)=HRR(6,10,1)+VRR(6,10,0)
      HRR(6,11,1)=HRR(6,11,1)+VRR(6,11,0)
      HRR(6,36,1)=HRR(6,36,1)+FnSpK*VRR(6,11,0)
      HRR(6,12,1)=HRR(6,12,1)+VRR(6,12,0)
      HRR(6,37,1)=HRR(6,37,1)+FnSpK*VRR(6,12,0)
      HRR(6,13,1)=HRR(6,13,1)+VRR(6,13,0)
      HRR(6,38,1)=HRR(6,38,1)+FnSpK*VRR(6,13,0)
      HRR(6,14,1)=HRR(6,14,1)+VRR(6,14,0)
      HRR(6,39,1)=HRR(6,39,1)+FnSpK*VRR(6,14,0)
      HRR(6,15,1)=HRR(6,15,1)+VRR(6,15,0)
      HRR(6,40,1)=HRR(6,40,1)+FnSpK*VRR(6,15,0)
      HRR(6,16,1)=HRR(6,16,1)+VRR(6,16,0)
      HRR(6,41,1)=HRR(6,41,1)+FnSpK*VRR(6,16,0)
      HRR(6,17,1)=HRR(6,17,1)+VRR(6,17,0)
      HRR(6,42,1)=HRR(6,42,1)+FnSpK*VRR(6,17,0)
      HRR(6,18,1)=HRR(6,18,1)+VRR(6,18,0)
      HRR(6,43,1)=HRR(6,43,1)+FnSpK*VRR(6,18,0)
      HRR(6,19,1)=HRR(6,19,1)+VRR(6,19,0)
      HRR(6,44,1)=HRR(6,44,1)+FnSpK*VRR(6,19,0)
      HRR(6,20,1)=HRR(6,20,1)+VRR(6,20,0)
      HRR(6,45,1)=HRR(6,45,1)+FnSpK*VRR(6,20,0)
      HRR(6,21,1)=HRR(6,21,1)+VRR(6,21,0)
      HRR(6,22,1)=HRR(6,22,1)+VRR(6,22,0)
      HRR(6,23,1)=HRR(6,23,1)+VRR(6,23,0)
      HRR(6,24,1)=HRR(6,24,1)+VRR(6,24,0)
      HRR(6,25,1)=HRR(6,25,1)+VRR(6,25,0)
      HRR(6,26,1)=HRR(6,26,1)+VRR(6,26,0)
      HRR(6,27,1)=HRR(6,27,1)+VRR(6,27,0)
      HRR(6,28,1)=HRR(6,28,1)+VRR(6,28,0)
      HRR(6,29,1)=HRR(6,29,1)+VRR(6,29,0)
      HRR(6,30,1)=HRR(6,30,1)+VRR(6,30,0)
      HRR(6,31,1)=HRR(6,31,1)+VRR(6,31,0)
      HRR(6,32,1)=HRR(6,32,1)+VRR(6,32,0)
      HRR(6,33,1)=HRR(6,33,1)+VRR(6,33,0)
      HRR(6,34,1)=HRR(6,34,1)+VRR(6,34,0)
      HRR(6,35,1)=HRR(6,35,1)+VRR(6,35,0)
      HRR(7,1,1)=HRR(7,1,1)+VRR(7,1,0)
      HRR(7,2,1)=HRR(7,2,1)+VRR(7,2,0)
      HRR(7,3,1)=HRR(7,3,1)+VRR(7,3,0)
      HRR(7,4,1)=HRR(7,4,1)+VRR(7,4,0)
      HRR(7,5,1)=HRR(7,5,1)+VRR(7,5,0)
      HRR(7,6,1)=HRR(7,6,1)+VRR(7,6,0)
      HRR(7,7,1)=HRR(7,7,1)+VRR(7,7,0)
      HRR(7,8,1)=HRR(7,8,1)+VRR(7,8,0)
      HRR(7,9,1)=HRR(7,9,1)+VRR(7,9,0)
      HRR(7,10,1)=HRR(7,10,1)+VRR(7,10,0)
      HRR(7,11,1)=HRR(7,11,1)+VRR(7,11,0)
      HRR(7,36,1)=HRR(7,36,1)+FnSpK*VRR(7,11,0)
      HRR(7,12,1)=HRR(7,12,1)+VRR(7,12,0)
      HRR(7,37,1)=HRR(7,37,1)+FnSpK*VRR(7,12,0)
      HRR(7,13,1)=HRR(7,13,1)+VRR(7,13,0)
      HRR(7,38,1)=HRR(7,38,1)+FnSpK*VRR(7,13,0)
      HRR(7,14,1)=HRR(7,14,1)+VRR(7,14,0)
      HRR(7,39,1)=HRR(7,39,1)+FnSpK*VRR(7,14,0)
      HRR(7,15,1)=HRR(7,15,1)+VRR(7,15,0)
      HRR(7,40,1)=HRR(7,40,1)+FnSpK*VRR(7,15,0)
      HRR(7,16,1)=HRR(7,16,1)+VRR(7,16,0)
      HRR(7,41,1)=HRR(7,41,1)+FnSpK*VRR(7,16,0)
      HRR(7,17,1)=HRR(7,17,1)+VRR(7,17,0)
      HRR(7,42,1)=HRR(7,42,1)+FnSpK*VRR(7,17,0)
      HRR(7,18,1)=HRR(7,18,1)+VRR(7,18,0)
      HRR(7,43,1)=HRR(7,43,1)+FnSpK*VRR(7,18,0)
      HRR(7,19,1)=HRR(7,19,1)+VRR(7,19,0)
      HRR(7,44,1)=HRR(7,44,1)+FnSpK*VRR(7,19,0)
      HRR(7,20,1)=HRR(7,20,1)+VRR(7,20,0)
      HRR(7,45,1)=HRR(7,45,1)+FnSpK*VRR(7,20,0)
      HRR(7,21,1)=HRR(7,21,1)+VRR(7,21,0)
      HRR(7,22,1)=HRR(7,22,1)+VRR(7,22,0)
      HRR(7,23,1)=HRR(7,23,1)+VRR(7,23,0)
      HRR(7,24,1)=HRR(7,24,1)+VRR(7,24,0)
      HRR(7,25,1)=HRR(7,25,1)+VRR(7,25,0)
      HRR(7,26,1)=HRR(7,26,1)+VRR(7,26,0)
      HRR(7,27,1)=HRR(7,27,1)+VRR(7,27,0)
      HRR(7,28,1)=HRR(7,28,1)+VRR(7,28,0)
      HRR(7,29,1)=HRR(7,29,1)+VRR(7,29,0)
      HRR(7,30,1)=HRR(7,30,1)+VRR(7,30,0)
      HRR(7,31,1)=HRR(7,31,1)+VRR(7,31,0)
      HRR(7,32,1)=HRR(7,32,1)+VRR(7,32,0)
      HRR(7,33,1)=HRR(7,33,1)+VRR(7,33,0)
      HRR(7,34,1)=HRR(7,34,1)+VRR(7,34,0)
      HRR(7,35,1)=HRR(7,35,1)+VRR(7,35,0)
      HRR(8,1,1)=HRR(8,1,1)+VRR(8,1,0)
      HRR(8,2,1)=HRR(8,2,1)+VRR(8,2,0)
      HRR(8,3,1)=HRR(8,3,1)+VRR(8,3,0)
      HRR(8,4,1)=HRR(8,4,1)+VRR(8,4,0)
      HRR(8,5,1)=HRR(8,5,1)+VRR(8,5,0)
      HRR(8,6,1)=HRR(8,6,1)+VRR(8,6,0)
      HRR(8,7,1)=HRR(8,7,1)+VRR(8,7,0)
      HRR(8,8,1)=HRR(8,8,1)+VRR(8,8,0)
      HRR(8,9,1)=HRR(8,9,1)+VRR(8,9,0)
      HRR(8,10,1)=HRR(8,10,1)+VRR(8,10,0)
      HRR(8,11,1)=HRR(8,11,1)+VRR(8,11,0)
      HRR(8,36,1)=HRR(8,36,1)+FnSpK*VRR(8,11,0)
      HRR(8,12,1)=HRR(8,12,1)+VRR(8,12,0)
      HRR(8,37,1)=HRR(8,37,1)+FnSpK*VRR(8,12,0)
      HRR(8,13,1)=HRR(8,13,1)+VRR(8,13,0)
      HRR(8,38,1)=HRR(8,38,1)+FnSpK*VRR(8,13,0)
      HRR(8,14,1)=HRR(8,14,1)+VRR(8,14,0)
      HRR(8,39,1)=HRR(8,39,1)+FnSpK*VRR(8,14,0)
      HRR(8,15,1)=HRR(8,15,1)+VRR(8,15,0)
      HRR(8,40,1)=HRR(8,40,1)+FnSpK*VRR(8,15,0)
      HRR(8,16,1)=HRR(8,16,1)+VRR(8,16,0)
      HRR(8,41,1)=HRR(8,41,1)+FnSpK*VRR(8,16,0)
      HRR(8,17,1)=HRR(8,17,1)+VRR(8,17,0)
      HRR(8,42,1)=HRR(8,42,1)+FnSpK*VRR(8,17,0)
      HRR(8,18,1)=HRR(8,18,1)+VRR(8,18,0)
      HRR(8,43,1)=HRR(8,43,1)+FnSpK*VRR(8,18,0)
      HRR(8,19,1)=HRR(8,19,1)+VRR(8,19,0)
      HRR(8,44,1)=HRR(8,44,1)+FnSpK*VRR(8,19,0)
      HRR(8,20,1)=HRR(8,20,1)+VRR(8,20,0)
      HRR(8,45,1)=HRR(8,45,1)+FnSpK*VRR(8,20,0)
      HRR(8,21,1)=HRR(8,21,1)+VRR(8,21,0)
      HRR(8,22,1)=HRR(8,22,1)+VRR(8,22,0)
      HRR(8,23,1)=HRR(8,23,1)+VRR(8,23,0)
      HRR(8,24,1)=HRR(8,24,1)+VRR(8,24,0)
      HRR(8,25,1)=HRR(8,25,1)+VRR(8,25,0)
      HRR(8,26,1)=HRR(8,26,1)+VRR(8,26,0)
      HRR(8,27,1)=HRR(8,27,1)+VRR(8,27,0)
      HRR(8,28,1)=HRR(8,28,1)+VRR(8,28,0)
      HRR(8,29,1)=HRR(8,29,1)+VRR(8,29,0)
      HRR(8,30,1)=HRR(8,30,1)+VRR(8,30,0)
      HRR(8,31,1)=HRR(8,31,1)+VRR(8,31,0)
      HRR(8,32,1)=HRR(8,32,1)+VRR(8,32,0)
      HRR(8,33,1)=HRR(8,33,1)+VRR(8,33,0)
      HRR(8,34,1)=HRR(8,34,1)+VRR(8,34,0)
      HRR(8,35,1)=HRR(8,35,1)+VRR(8,35,0)
      HRR(9,1,1)=HRR(9,1,1)+VRR(9,1,0)
      HRR(9,2,1)=HRR(9,2,1)+VRR(9,2,0)
      HRR(9,3,1)=HRR(9,3,1)+VRR(9,3,0)
      HRR(9,4,1)=HRR(9,4,1)+VRR(9,4,0)
      HRR(9,5,1)=HRR(9,5,1)+VRR(9,5,0)
      HRR(9,6,1)=HRR(9,6,1)+VRR(9,6,0)
      HRR(9,7,1)=HRR(9,7,1)+VRR(9,7,0)
      HRR(9,8,1)=HRR(9,8,1)+VRR(9,8,0)
      HRR(9,9,1)=HRR(9,9,1)+VRR(9,9,0)
      HRR(9,10,1)=HRR(9,10,1)+VRR(9,10,0)
      HRR(9,11,1)=HRR(9,11,1)+VRR(9,11,0)
      HRR(9,36,1)=HRR(9,36,1)+FnSpK*VRR(9,11,0)
      HRR(9,12,1)=HRR(9,12,1)+VRR(9,12,0)
      HRR(9,37,1)=HRR(9,37,1)+FnSpK*VRR(9,12,0)
      HRR(9,13,1)=HRR(9,13,1)+VRR(9,13,0)
      HRR(9,38,1)=HRR(9,38,1)+FnSpK*VRR(9,13,0)
      HRR(9,14,1)=HRR(9,14,1)+VRR(9,14,0)
      HRR(9,39,1)=HRR(9,39,1)+FnSpK*VRR(9,14,0)
      HRR(9,15,1)=HRR(9,15,1)+VRR(9,15,0)
      HRR(9,40,1)=HRR(9,40,1)+FnSpK*VRR(9,15,0)
      HRR(9,16,1)=HRR(9,16,1)+VRR(9,16,0)
      HRR(9,41,1)=HRR(9,41,1)+FnSpK*VRR(9,16,0)
      HRR(9,17,1)=HRR(9,17,1)+VRR(9,17,0)
      HRR(9,42,1)=HRR(9,42,1)+FnSpK*VRR(9,17,0)
      HRR(9,18,1)=HRR(9,18,1)+VRR(9,18,0)
      HRR(9,43,1)=HRR(9,43,1)+FnSpK*VRR(9,18,0)
      HRR(9,19,1)=HRR(9,19,1)+VRR(9,19,0)
      HRR(9,44,1)=HRR(9,44,1)+FnSpK*VRR(9,19,0)
      HRR(9,20,1)=HRR(9,20,1)+VRR(9,20,0)
      HRR(9,45,1)=HRR(9,45,1)+FnSpK*VRR(9,20,0)
      HRR(9,21,1)=HRR(9,21,1)+VRR(9,21,0)
      HRR(9,22,1)=HRR(9,22,1)+VRR(9,22,0)
      HRR(9,23,1)=HRR(9,23,1)+VRR(9,23,0)
      HRR(9,24,1)=HRR(9,24,1)+VRR(9,24,0)
      HRR(9,25,1)=HRR(9,25,1)+VRR(9,25,0)
      HRR(9,26,1)=HRR(9,26,1)+VRR(9,26,0)
      HRR(9,27,1)=HRR(9,27,1)+VRR(9,27,0)
      HRR(9,28,1)=HRR(9,28,1)+VRR(9,28,0)
      HRR(9,29,1)=HRR(9,29,1)+VRR(9,29,0)
      HRR(9,30,1)=HRR(9,30,1)+VRR(9,30,0)
      HRR(9,31,1)=HRR(9,31,1)+VRR(9,31,0)
      HRR(9,32,1)=HRR(9,32,1)+VRR(9,32,0)
      HRR(9,33,1)=HRR(9,33,1)+VRR(9,33,0)
      HRR(9,34,1)=HRR(9,34,1)+VRR(9,34,0)
      HRR(9,35,1)=HRR(9,35,1)+VRR(9,35,0)
      HRR(10,1,1)=HRR(10,1,1)+VRR(10,1,0)
      HRR(10,2,1)=HRR(10,2,1)+VRR(10,2,0)
      HRR(10,3,1)=HRR(10,3,1)+VRR(10,3,0)
      HRR(10,4,1)=HRR(10,4,1)+VRR(10,4,0)
      HRR(10,5,1)=HRR(10,5,1)+VRR(10,5,0)
      HRR(10,6,1)=HRR(10,6,1)+VRR(10,6,0)
      HRR(10,7,1)=HRR(10,7,1)+VRR(10,7,0)
      HRR(10,8,1)=HRR(10,8,1)+VRR(10,8,0)
      HRR(10,9,1)=HRR(10,9,1)+VRR(10,9,0)
      HRR(10,10,1)=HRR(10,10,1)+VRR(10,10,0)
      HRR(10,11,1)=HRR(10,11,1)+VRR(10,11,0)
      HRR(10,36,1)=HRR(10,36,1)+FnSpK*VRR(10,11,0)
      HRR(10,12,1)=HRR(10,12,1)+VRR(10,12,0)
      HRR(10,37,1)=HRR(10,37,1)+FnSpK*VRR(10,12,0)
      HRR(10,13,1)=HRR(10,13,1)+VRR(10,13,0)
      HRR(10,38,1)=HRR(10,38,1)+FnSpK*VRR(10,13,0)
      HRR(10,14,1)=HRR(10,14,1)+VRR(10,14,0)
      HRR(10,39,1)=HRR(10,39,1)+FnSpK*VRR(10,14,0)
      HRR(10,15,1)=HRR(10,15,1)+VRR(10,15,0)
      HRR(10,40,1)=HRR(10,40,1)+FnSpK*VRR(10,15,0)
      HRR(10,16,1)=HRR(10,16,1)+VRR(10,16,0)
      HRR(10,41,1)=HRR(10,41,1)+FnSpK*VRR(10,16,0)
      HRR(10,17,1)=HRR(10,17,1)+VRR(10,17,0)
      HRR(10,42,1)=HRR(10,42,1)+FnSpK*VRR(10,17,0)
      HRR(10,18,1)=HRR(10,18,1)+VRR(10,18,0)
      HRR(10,43,1)=HRR(10,43,1)+FnSpK*VRR(10,18,0)
      HRR(10,19,1)=HRR(10,19,1)+VRR(10,19,0)
      HRR(10,44,1)=HRR(10,44,1)+FnSpK*VRR(10,19,0)
      HRR(10,20,1)=HRR(10,20,1)+VRR(10,20,0)
      HRR(10,45,1)=HRR(10,45,1)+FnSpK*VRR(10,20,0)
      HRR(10,21,1)=HRR(10,21,1)+VRR(10,21,0)
      HRR(10,22,1)=HRR(10,22,1)+VRR(10,22,0)
      HRR(10,23,1)=HRR(10,23,1)+VRR(10,23,0)
      HRR(10,24,1)=HRR(10,24,1)+VRR(10,24,0)
      HRR(10,25,1)=HRR(10,25,1)+VRR(10,25,0)
      HRR(10,26,1)=HRR(10,26,1)+VRR(10,26,0)
      HRR(10,27,1)=HRR(10,27,1)+VRR(10,27,0)
      HRR(10,28,1)=HRR(10,28,1)+VRR(10,28,0)
      HRR(10,29,1)=HRR(10,29,1)+VRR(10,29,0)
      HRR(10,30,1)=HRR(10,30,1)+VRR(10,30,0)
      HRR(10,31,1)=HRR(10,31,1)+VRR(10,31,0)
      HRR(10,32,1)=HRR(10,32,1)+VRR(10,32,0)
      HRR(10,33,1)=HRR(10,33,1)+VRR(10,33,0)
      HRR(10,34,1)=HRR(10,34,1)+VRR(10,34,0)
      HRR(10,35,1)=HRR(10,35,1)+VRR(10,35,0)
      HRR(11,1,1)=HRR(11,1,1)+VRR(11,1,0)
      HRR(11,2,1)=HRR(11,2,1)+VRR(11,2,0)
      HRR(11,3,1)=HRR(11,3,1)+VRR(11,3,0)
      HRR(11,4,1)=HRR(11,4,1)+VRR(11,4,0)
      HRR(11,5,1)=HRR(11,5,1)+VRR(11,5,0)
      HRR(11,6,1)=HRR(11,6,1)+VRR(11,6,0)
      HRR(11,7,1)=HRR(11,7,1)+VRR(11,7,0)
      HRR(11,8,1)=HRR(11,8,1)+VRR(11,8,0)
      HRR(11,9,1)=HRR(11,9,1)+VRR(11,9,0)
      HRR(11,10,1)=HRR(11,10,1)+VRR(11,10,0)
      HRR(11,11,1)=HRR(11,11,1)+VRR(11,11,0)
      HRR(11,36,1)=HRR(11,36,1)+FnSpK*VRR(11,11,0)
      HRR(11,12,1)=HRR(11,12,1)+VRR(11,12,0)
      HRR(11,37,1)=HRR(11,37,1)+FnSpK*VRR(11,12,0)
      HRR(11,13,1)=HRR(11,13,1)+VRR(11,13,0)
      HRR(11,38,1)=HRR(11,38,1)+FnSpK*VRR(11,13,0)
      HRR(11,14,1)=HRR(11,14,1)+VRR(11,14,0)
      HRR(11,39,1)=HRR(11,39,1)+FnSpK*VRR(11,14,0)
      HRR(11,15,1)=HRR(11,15,1)+VRR(11,15,0)
      HRR(11,40,1)=HRR(11,40,1)+FnSpK*VRR(11,15,0)
      HRR(11,16,1)=HRR(11,16,1)+VRR(11,16,0)
      HRR(11,41,1)=HRR(11,41,1)+FnSpK*VRR(11,16,0)
      HRR(11,17,1)=HRR(11,17,1)+VRR(11,17,0)
      HRR(11,42,1)=HRR(11,42,1)+FnSpK*VRR(11,17,0)
      HRR(11,18,1)=HRR(11,18,1)+VRR(11,18,0)
      HRR(11,43,1)=HRR(11,43,1)+FnSpK*VRR(11,18,0)
      HRR(11,19,1)=HRR(11,19,1)+VRR(11,19,0)
      HRR(11,44,1)=HRR(11,44,1)+FnSpK*VRR(11,19,0)
      HRR(11,20,1)=HRR(11,20,1)+VRR(11,20,0)
      HRR(11,45,1)=HRR(11,45,1)+FnSpK*VRR(11,20,0)
      HRR(11,21,1)=HRR(11,21,1)+VRR(11,21,0)
      HRR(11,22,1)=HRR(11,22,1)+VRR(11,22,0)
      HRR(11,23,1)=HRR(11,23,1)+VRR(11,23,0)
      HRR(11,24,1)=HRR(11,24,1)+VRR(11,24,0)
      HRR(11,25,1)=HRR(11,25,1)+VRR(11,25,0)
      HRR(11,26,1)=HRR(11,26,1)+VRR(11,26,0)
      HRR(11,27,1)=HRR(11,27,1)+VRR(11,27,0)
      HRR(11,28,1)=HRR(11,28,1)+VRR(11,28,0)
      HRR(11,29,1)=HRR(11,29,1)+VRR(11,29,0)
      HRR(11,30,1)=HRR(11,30,1)+VRR(11,30,0)
      HRR(11,31,1)=HRR(11,31,1)+VRR(11,31,0)
      HRR(11,32,1)=HRR(11,32,1)+VRR(11,32,0)
      HRR(11,33,1)=HRR(11,33,1)+VRR(11,33,0)
      HRR(11,34,1)=HRR(11,34,1)+VRR(11,34,0)
      HRR(11,35,1)=HRR(11,35,1)+VRR(11,35,0)
      HRR(12,1,1)=HRR(12,1,1)+VRR(12,1,0)
      HRR(12,2,1)=HRR(12,2,1)+VRR(12,2,0)
      HRR(12,3,1)=HRR(12,3,1)+VRR(12,3,0)
      HRR(12,4,1)=HRR(12,4,1)+VRR(12,4,0)
      HRR(12,5,1)=HRR(12,5,1)+VRR(12,5,0)
      HRR(12,6,1)=HRR(12,6,1)+VRR(12,6,0)
      HRR(12,7,1)=HRR(12,7,1)+VRR(12,7,0)
      HRR(12,8,1)=HRR(12,8,1)+VRR(12,8,0)
      HRR(12,9,1)=HRR(12,9,1)+VRR(12,9,0)
      HRR(12,10,1)=HRR(12,10,1)+VRR(12,10,0)
      HRR(12,11,1)=HRR(12,11,1)+VRR(12,11,0)
      HRR(12,36,1)=HRR(12,36,1)+FnSpK*VRR(12,11,0)
      HRR(12,12,1)=HRR(12,12,1)+VRR(12,12,0)
      HRR(12,37,1)=HRR(12,37,1)+FnSpK*VRR(12,12,0)
      HRR(12,13,1)=HRR(12,13,1)+VRR(12,13,0)
      HRR(12,38,1)=HRR(12,38,1)+FnSpK*VRR(12,13,0)
      HRR(12,14,1)=HRR(12,14,1)+VRR(12,14,0)
      HRR(12,39,1)=HRR(12,39,1)+FnSpK*VRR(12,14,0)
      HRR(12,15,1)=HRR(12,15,1)+VRR(12,15,0)
      HRR(12,40,1)=HRR(12,40,1)+FnSpK*VRR(12,15,0)
      HRR(12,16,1)=HRR(12,16,1)+VRR(12,16,0)
      HRR(12,41,1)=HRR(12,41,1)+FnSpK*VRR(12,16,0)
      HRR(12,17,1)=HRR(12,17,1)+VRR(12,17,0)
      HRR(12,42,1)=HRR(12,42,1)+FnSpK*VRR(12,17,0)
      HRR(12,18,1)=HRR(12,18,1)+VRR(12,18,0)
      HRR(12,43,1)=HRR(12,43,1)+FnSpK*VRR(12,18,0)
      HRR(12,19,1)=HRR(12,19,1)+VRR(12,19,0)
      HRR(12,44,1)=HRR(12,44,1)+FnSpK*VRR(12,19,0)
      HRR(12,20,1)=HRR(12,20,1)+VRR(12,20,0)
      HRR(12,45,1)=HRR(12,45,1)+FnSpK*VRR(12,20,0)
      HRR(12,21,1)=HRR(12,21,1)+VRR(12,21,0)
      HRR(12,22,1)=HRR(12,22,1)+VRR(12,22,0)
      HRR(12,23,1)=HRR(12,23,1)+VRR(12,23,0)
      HRR(12,24,1)=HRR(12,24,1)+VRR(12,24,0)
      HRR(12,25,1)=HRR(12,25,1)+VRR(12,25,0)
      HRR(12,26,1)=HRR(12,26,1)+VRR(12,26,0)
      HRR(12,27,1)=HRR(12,27,1)+VRR(12,27,0)
      HRR(12,28,1)=HRR(12,28,1)+VRR(12,28,0)
      HRR(12,29,1)=HRR(12,29,1)+VRR(12,29,0)
      HRR(12,30,1)=HRR(12,30,1)+VRR(12,30,0)
      HRR(12,31,1)=HRR(12,31,1)+VRR(12,31,0)
      HRR(12,32,1)=HRR(12,32,1)+VRR(12,32,0)
      HRR(12,33,1)=HRR(12,33,1)+VRR(12,33,0)
      HRR(12,34,1)=HRR(12,34,1)+VRR(12,34,0)
      HRR(12,35,1)=HRR(12,35,1)+VRR(12,35,0)
      HRR(13,1,1)=HRR(13,1,1)+VRR(13,1,0)
      HRR(13,2,1)=HRR(13,2,1)+VRR(13,2,0)
      HRR(13,3,1)=HRR(13,3,1)+VRR(13,3,0)
      HRR(13,4,1)=HRR(13,4,1)+VRR(13,4,0)
      HRR(13,5,1)=HRR(13,5,1)+VRR(13,5,0)
      HRR(13,6,1)=HRR(13,6,1)+VRR(13,6,0)
      HRR(13,7,1)=HRR(13,7,1)+VRR(13,7,0)
      HRR(13,8,1)=HRR(13,8,1)+VRR(13,8,0)
      HRR(13,9,1)=HRR(13,9,1)+VRR(13,9,0)
      HRR(13,10,1)=HRR(13,10,1)+VRR(13,10,0)
      HRR(13,11,1)=HRR(13,11,1)+VRR(13,11,0)
      HRR(13,36,1)=HRR(13,36,1)+FnSpK*VRR(13,11,0)
      HRR(13,12,1)=HRR(13,12,1)+VRR(13,12,0)
      HRR(13,37,1)=HRR(13,37,1)+FnSpK*VRR(13,12,0)
      HRR(13,13,1)=HRR(13,13,1)+VRR(13,13,0)
      HRR(13,38,1)=HRR(13,38,1)+FnSpK*VRR(13,13,0)
      HRR(13,14,1)=HRR(13,14,1)+VRR(13,14,0)
      HRR(13,39,1)=HRR(13,39,1)+FnSpK*VRR(13,14,0)
      HRR(13,15,1)=HRR(13,15,1)+VRR(13,15,0)
      HRR(13,40,1)=HRR(13,40,1)+FnSpK*VRR(13,15,0)
      HRR(13,16,1)=HRR(13,16,1)+VRR(13,16,0)
      HRR(13,41,1)=HRR(13,41,1)+FnSpK*VRR(13,16,0)
      HRR(13,17,1)=HRR(13,17,1)+VRR(13,17,0)
      HRR(13,42,1)=HRR(13,42,1)+FnSpK*VRR(13,17,0)
      HRR(13,18,1)=HRR(13,18,1)+VRR(13,18,0)
      HRR(13,43,1)=HRR(13,43,1)+FnSpK*VRR(13,18,0)
      HRR(13,19,1)=HRR(13,19,1)+VRR(13,19,0)
      HRR(13,44,1)=HRR(13,44,1)+FnSpK*VRR(13,19,0)
      HRR(13,20,1)=HRR(13,20,1)+VRR(13,20,0)
      HRR(13,45,1)=HRR(13,45,1)+FnSpK*VRR(13,20,0)
      HRR(13,21,1)=HRR(13,21,1)+VRR(13,21,0)
      HRR(13,22,1)=HRR(13,22,1)+VRR(13,22,0)
      HRR(13,23,1)=HRR(13,23,1)+VRR(13,23,0)
      HRR(13,24,1)=HRR(13,24,1)+VRR(13,24,0)
      HRR(13,25,1)=HRR(13,25,1)+VRR(13,25,0)
      HRR(13,26,1)=HRR(13,26,1)+VRR(13,26,0)
      HRR(13,27,1)=HRR(13,27,1)+VRR(13,27,0)
      HRR(13,28,1)=HRR(13,28,1)+VRR(13,28,0)
      HRR(13,29,1)=HRR(13,29,1)+VRR(13,29,0)
      HRR(13,30,1)=HRR(13,30,1)+VRR(13,30,0)
      HRR(13,31,1)=HRR(13,31,1)+VRR(13,31,0)
      HRR(13,32,1)=HRR(13,32,1)+VRR(13,32,0)
      HRR(13,33,1)=HRR(13,33,1)+VRR(13,33,0)
      HRR(13,34,1)=HRR(13,34,1)+VRR(13,34,0)
      HRR(13,35,1)=HRR(13,35,1)+VRR(13,35,0)
      HRR(14,1,1)=HRR(14,1,1)+VRR(14,1,0)
      HRR(14,2,1)=HRR(14,2,1)+VRR(14,2,0)
      HRR(14,3,1)=HRR(14,3,1)+VRR(14,3,0)
      HRR(14,4,1)=HRR(14,4,1)+VRR(14,4,0)
      HRR(14,5,1)=HRR(14,5,1)+VRR(14,5,0)
      HRR(14,6,1)=HRR(14,6,1)+VRR(14,6,0)
      HRR(14,7,1)=HRR(14,7,1)+VRR(14,7,0)
      HRR(14,8,1)=HRR(14,8,1)+VRR(14,8,0)
      HRR(14,9,1)=HRR(14,9,1)+VRR(14,9,0)
      HRR(14,10,1)=HRR(14,10,1)+VRR(14,10,0)
      HRR(14,11,1)=HRR(14,11,1)+VRR(14,11,0)
      HRR(14,36,1)=HRR(14,36,1)+FnSpK*VRR(14,11,0)
      HRR(14,12,1)=HRR(14,12,1)+VRR(14,12,0)
      HRR(14,37,1)=HRR(14,37,1)+FnSpK*VRR(14,12,0)
      HRR(14,13,1)=HRR(14,13,1)+VRR(14,13,0)
      HRR(14,38,1)=HRR(14,38,1)+FnSpK*VRR(14,13,0)
      HRR(14,14,1)=HRR(14,14,1)+VRR(14,14,0)
      HRR(14,39,1)=HRR(14,39,1)+FnSpK*VRR(14,14,0)
      HRR(14,15,1)=HRR(14,15,1)+VRR(14,15,0)
      HRR(14,40,1)=HRR(14,40,1)+FnSpK*VRR(14,15,0)
      HRR(14,16,1)=HRR(14,16,1)+VRR(14,16,0)
      HRR(14,41,1)=HRR(14,41,1)+FnSpK*VRR(14,16,0)
      HRR(14,17,1)=HRR(14,17,1)+VRR(14,17,0)
      HRR(14,42,1)=HRR(14,42,1)+FnSpK*VRR(14,17,0)
      HRR(14,18,1)=HRR(14,18,1)+VRR(14,18,0)
      HRR(14,43,1)=HRR(14,43,1)+FnSpK*VRR(14,18,0)
      HRR(14,19,1)=HRR(14,19,1)+VRR(14,19,0)
      HRR(14,44,1)=HRR(14,44,1)+FnSpK*VRR(14,19,0)
      HRR(14,20,1)=HRR(14,20,1)+VRR(14,20,0)
      HRR(14,45,1)=HRR(14,45,1)+FnSpK*VRR(14,20,0)
      HRR(14,21,1)=HRR(14,21,1)+VRR(14,21,0)
      HRR(14,22,1)=HRR(14,22,1)+VRR(14,22,0)
      HRR(14,23,1)=HRR(14,23,1)+VRR(14,23,0)
      HRR(14,24,1)=HRR(14,24,1)+VRR(14,24,0)
      HRR(14,25,1)=HRR(14,25,1)+VRR(14,25,0)
      HRR(14,26,1)=HRR(14,26,1)+VRR(14,26,0)
      HRR(14,27,1)=HRR(14,27,1)+VRR(14,27,0)
      HRR(14,28,1)=HRR(14,28,1)+VRR(14,28,0)
      HRR(14,29,1)=HRR(14,29,1)+VRR(14,29,0)
      HRR(14,30,1)=HRR(14,30,1)+VRR(14,30,0)
      HRR(14,31,1)=HRR(14,31,1)+VRR(14,31,0)
      HRR(14,32,1)=HRR(14,32,1)+VRR(14,32,0)
      HRR(14,33,1)=HRR(14,33,1)+VRR(14,33,0)
      HRR(14,34,1)=HRR(14,34,1)+VRR(14,34,0)
      HRR(14,35,1)=HRR(14,35,1)+VRR(14,35,0)
      HRR(15,1,1)=HRR(15,1,1)+VRR(15,1,0)
      HRR(15,2,1)=HRR(15,2,1)+VRR(15,2,0)
      HRR(15,3,1)=HRR(15,3,1)+VRR(15,3,0)
      HRR(15,4,1)=HRR(15,4,1)+VRR(15,4,0)
      HRR(15,5,1)=HRR(15,5,1)+VRR(15,5,0)
      HRR(15,6,1)=HRR(15,6,1)+VRR(15,6,0)
      HRR(15,7,1)=HRR(15,7,1)+VRR(15,7,0)
      HRR(15,8,1)=HRR(15,8,1)+VRR(15,8,0)
      HRR(15,9,1)=HRR(15,9,1)+VRR(15,9,0)
      HRR(15,10,1)=HRR(15,10,1)+VRR(15,10,0)
      HRR(15,11,1)=HRR(15,11,1)+VRR(15,11,0)
      HRR(15,36,1)=HRR(15,36,1)+FnSpK*VRR(15,11,0)
      HRR(15,12,1)=HRR(15,12,1)+VRR(15,12,0)
      HRR(15,37,1)=HRR(15,37,1)+FnSpK*VRR(15,12,0)
      HRR(15,13,1)=HRR(15,13,1)+VRR(15,13,0)
      HRR(15,38,1)=HRR(15,38,1)+FnSpK*VRR(15,13,0)
      HRR(15,14,1)=HRR(15,14,1)+VRR(15,14,0)
      HRR(15,39,1)=HRR(15,39,1)+FnSpK*VRR(15,14,0)
      HRR(15,15,1)=HRR(15,15,1)+VRR(15,15,0)
      HRR(15,40,1)=HRR(15,40,1)+FnSpK*VRR(15,15,0)
      HRR(15,16,1)=HRR(15,16,1)+VRR(15,16,0)
      HRR(15,41,1)=HRR(15,41,1)+FnSpK*VRR(15,16,0)
      HRR(15,17,1)=HRR(15,17,1)+VRR(15,17,0)
      HRR(15,42,1)=HRR(15,42,1)+FnSpK*VRR(15,17,0)
      HRR(15,18,1)=HRR(15,18,1)+VRR(15,18,0)
      HRR(15,43,1)=HRR(15,43,1)+FnSpK*VRR(15,18,0)
      HRR(15,19,1)=HRR(15,19,1)+VRR(15,19,0)
      HRR(15,44,1)=HRR(15,44,1)+FnSpK*VRR(15,19,0)
      HRR(15,20,1)=HRR(15,20,1)+VRR(15,20,0)
      HRR(15,45,1)=HRR(15,45,1)+FnSpK*VRR(15,20,0)
      HRR(15,21,1)=HRR(15,21,1)+VRR(15,21,0)
      HRR(15,22,1)=HRR(15,22,1)+VRR(15,22,0)
      HRR(15,23,1)=HRR(15,23,1)+VRR(15,23,0)
      HRR(15,24,1)=HRR(15,24,1)+VRR(15,24,0)
      HRR(15,25,1)=HRR(15,25,1)+VRR(15,25,0)
      HRR(15,26,1)=HRR(15,26,1)+VRR(15,26,0)
      HRR(15,27,1)=HRR(15,27,1)+VRR(15,27,0)
      HRR(15,28,1)=HRR(15,28,1)+VRR(15,28,0)
      HRR(15,29,1)=HRR(15,29,1)+VRR(15,29,0)
      HRR(15,30,1)=HRR(15,30,1)+VRR(15,30,0)
      HRR(15,31,1)=HRR(15,31,1)+VRR(15,31,0)
      HRR(15,32,1)=HRR(15,32,1)+VRR(15,32,0)
      HRR(15,33,1)=HRR(15,33,1)+VRR(15,33,0)
      HRR(15,34,1)=HRR(15,34,1)+VRR(15,34,0)
      HRR(15,35,1)=HRR(15,35,1)+VRR(15,35,0)
      HRR(16,1,1)=HRR(16,1,1)+VRR(16,1,0)
      HRR(16,2,1)=HRR(16,2,1)+VRR(16,2,0)
      HRR(16,3,1)=HRR(16,3,1)+VRR(16,3,0)
      HRR(16,4,1)=HRR(16,4,1)+VRR(16,4,0)
      HRR(16,5,1)=HRR(16,5,1)+VRR(16,5,0)
      HRR(16,6,1)=HRR(16,6,1)+VRR(16,6,0)
      HRR(16,7,1)=HRR(16,7,1)+VRR(16,7,0)
      HRR(16,8,1)=HRR(16,8,1)+VRR(16,8,0)
      HRR(16,9,1)=HRR(16,9,1)+VRR(16,9,0)
      HRR(16,10,1)=HRR(16,10,1)+VRR(16,10,0)
      HRR(16,11,1)=HRR(16,11,1)+VRR(16,11,0)
      HRR(16,36,1)=HRR(16,36,1)+FnSpK*VRR(16,11,0)
      HRR(16,12,1)=HRR(16,12,1)+VRR(16,12,0)
      HRR(16,37,1)=HRR(16,37,1)+FnSpK*VRR(16,12,0)
      HRR(16,13,1)=HRR(16,13,1)+VRR(16,13,0)
      HRR(16,38,1)=HRR(16,38,1)+FnSpK*VRR(16,13,0)
      HRR(16,14,1)=HRR(16,14,1)+VRR(16,14,0)
      HRR(16,39,1)=HRR(16,39,1)+FnSpK*VRR(16,14,0)
      HRR(16,15,1)=HRR(16,15,1)+VRR(16,15,0)
      HRR(16,40,1)=HRR(16,40,1)+FnSpK*VRR(16,15,0)
      HRR(16,16,1)=HRR(16,16,1)+VRR(16,16,0)
      HRR(16,41,1)=HRR(16,41,1)+FnSpK*VRR(16,16,0)
      HRR(16,17,1)=HRR(16,17,1)+VRR(16,17,0)
      HRR(16,42,1)=HRR(16,42,1)+FnSpK*VRR(16,17,0)
      HRR(16,18,1)=HRR(16,18,1)+VRR(16,18,0)
      HRR(16,43,1)=HRR(16,43,1)+FnSpK*VRR(16,18,0)
      HRR(16,19,1)=HRR(16,19,1)+VRR(16,19,0)
      HRR(16,44,1)=HRR(16,44,1)+FnSpK*VRR(16,19,0)
      HRR(16,20,1)=HRR(16,20,1)+VRR(16,20,0)
      HRR(16,45,1)=HRR(16,45,1)+FnSpK*VRR(16,20,0)
      HRR(16,21,1)=HRR(16,21,1)+VRR(16,21,0)
      HRR(16,22,1)=HRR(16,22,1)+VRR(16,22,0)
      HRR(16,23,1)=HRR(16,23,1)+VRR(16,23,0)
      HRR(16,24,1)=HRR(16,24,1)+VRR(16,24,0)
      HRR(16,25,1)=HRR(16,25,1)+VRR(16,25,0)
      HRR(16,26,1)=HRR(16,26,1)+VRR(16,26,0)
      HRR(16,27,1)=HRR(16,27,1)+VRR(16,27,0)
      HRR(16,28,1)=HRR(16,28,1)+VRR(16,28,0)
      HRR(16,29,1)=HRR(16,29,1)+VRR(16,29,0)
      HRR(16,30,1)=HRR(16,30,1)+VRR(16,30,0)
      HRR(16,31,1)=HRR(16,31,1)+VRR(16,31,0)
      HRR(16,32,1)=HRR(16,32,1)+VRR(16,32,0)
      HRR(16,33,1)=HRR(16,33,1)+VRR(16,33,0)
      HRR(16,34,1)=HRR(16,34,1)+VRR(16,34,0)
      HRR(16,35,1)=HRR(16,35,1)+VRR(16,35,0)
      HRR(17,1,1)=HRR(17,1,1)+VRR(17,1,0)
      HRR(17,2,1)=HRR(17,2,1)+VRR(17,2,0)
      HRR(17,3,1)=HRR(17,3,1)+VRR(17,3,0)
      HRR(17,4,1)=HRR(17,4,1)+VRR(17,4,0)
      HRR(17,5,1)=HRR(17,5,1)+VRR(17,5,0)
      HRR(17,6,1)=HRR(17,6,1)+VRR(17,6,0)
      HRR(17,7,1)=HRR(17,7,1)+VRR(17,7,0)
      HRR(17,8,1)=HRR(17,8,1)+VRR(17,8,0)
      HRR(17,9,1)=HRR(17,9,1)+VRR(17,9,0)
      HRR(17,10,1)=HRR(17,10,1)+VRR(17,10,0)
      HRR(17,11,1)=HRR(17,11,1)+VRR(17,11,0)
      HRR(17,36,1)=HRR(17,36,1)+FnSpK*VRR(17,11,0)
      HRR(17,12,1)=HRR(17,12,1)+VRR(17,12,0)
      HRR(17,37,1)=HRR(17,37,1)+FnSpK*VRR(17,12,0)
      HRR(17,13,1)=HRR(17,13,1)+VRR(17,13,0)
      HRR(17,38,1)=HRR(17,38,1)+FnSpK*VRR(17,13,0)
      HRR(17,14,1)=HRR(17,14,1)+VRR(17,14,0)
      HRR(17,39,1)=HRR(17,39,1)+FnSpK*VRR(17,14,0)
      HRR(17,15,1)=HRR(17,15,1)+VRR(17,15,0)
      HRR(17,40,1)=HRR(17,40,1)+FnSpK*VRR(17,15,0)
      HRR(17,16,1)=HRR(17,16,1)+VRR(17,16,0)
      HRR(17,41,1)=HRR(17,41,1)+FnSpK*VRR(17,16,0)
      HRR(17,17,1)=HRR(17,17,1)+VRR(17,17,0)
      HRR(17,42,1)=HRR(17,42,1)+FnSpK*VRR(17,17,0)
      HRR(17,18,1)=HRR(17,18,1)+VRR(17,18,0)
      HRR(17,43,1)=HRR(17,43,1)+FnSpK*VRR(17,18,0)
      HRR(17,19,1)=HRR(17,19,1)+VRR(17,19,0)
      HRR(17,44,1)=HRR(17,44,1)+FnSpK*VRR(17,19,0)
      HRR(17,20,1)=HRR(17,20,1)+VRR(17,20,0)
      HRR(17,45,1)=HRR(17,45,1)+FnSpK*VRR(17,20,0)
      HRR(17,21,1)=HRR(17,21,1)+VRR(17,21,0)
      HRR(17,22,1)=HRR(17,22,1)+VRR(17,22,0)
      HRR(17,23,1)=HRR(17,23,1)+VRR(17,23,0)
      HRR(17,24,1)=HRR(17,24,1)+VRR(17,24,0)
      HRR(17,25,1)=HRR(17,25,1)+VRR(17,25,0)
      HRR(17,26,1)=HRR(17,26,1)+VRR(17,26,0)
      HRR(17,27,1)=HRR(17,27,1)+VRR(17,27,0)
      HRR(17,28,1)=HRR(17,28,1)+VRR(17,28,0)
      HRR(17,29,1)=HRR(17,29,1)+VRR(17,29,0)
      HRR(17,30,1)=HRR(17,30,1)+VRR(17,30,0)
      HRR(17,31,1)=HRR(17,31,1)+VRR(17,31,0)
      HRR(17,32,1)=HRR(17,32,1)+VRR(17,32,0)
      HRR(17,33,1)=HRR(17,33,1)+VRR(17,33,0)
      HRR(17,34,1)=HRR(17,34,1)+VRR(17,34,0)
      HRR(17,35,1)=HRR(17,35,1)+VRR(17,35,0)
      HRR(18,1,1)=HRR(18,1,1)+VRR(18,1,0)
      HRR(18,2,1)=HRR(18,2,1)+VRR(18,2,0)
      HRR(18,3,1)=HRR(18,3,1)+VRR(18,3,0)
      HRR(18,4,1)=HRR(18,4,1)+VRR(18,4,0)
      HRR(18,5,1)=HRR(18,5,1)+VRR(18,5,0)
      HRR(18,6,1)=HRR(18,6,1)+VRR(18,6,0)
      HRR(18,7,1)=HRR(18,7,1)+VRR(18,7,0)
      HRR(18,8,1)=HRR(18,8,1)+VRR(18,8,0)
      HRR(18,9,1)=HRR(18,9,1)+VRR(18,9,0)
      HRR(18,10,1)=HRR(18,10,1)+VRR(18,10,0)
      HRR(18,11,1)=HRR(18,11,1)+VRR(18,11,0)
      HRR(18,36,1)=HRR(18,36,1)+FnSpK*VRR(18,11,0)
      HRR(18,12,1)=HRR(18,12,1)+VRR(18,12,0)
      HRR(18,37,1)=HRR(18,37,1)+FnSpK*VRR(18,12,0)
      HRR(18,13,1)=HRR(18,13,1)+VRR(18,13,0)
      HRR(18,38,1)=HRR(18,38,1)+FnSpK*VRR(18,13,0)
      HRR(18,14,1)=HRR(18,14,1)+VRR(18,14,0)
      HRR(18,39,1)=HRR(18,39,1)+FnSpK*VRR(18,14,0)
      HRR(18,15,1)=HRR(18,15,1)+VRR(18,15,0)
      HRR(18,40,1)=HRR(18,40,1)+FnSpK*VRR(18,15,0)
      HRR(18,16,1)=HRR(18,16,1)+VRR(18,16,0)
      HRR(18,41,1)=HRR(18,41,1)+FnSpK*VRR(18,16,0)
      HRR(18,17,1)=HRR(18,17,1)+VRR(18,17,0)
      HRR(18,42,1)=HRR(18,42,1)+FnSpK*VRR(18,17,0)
      HRR(18,18,1)=HRR(18,18,1)+VRR(18,18,0)
      HRR(18,43,1)=HRR(18,43,1)+FnSpK*VRR(18,18,0)
      HRR(18,19,1)=HRR(18,19,1)+VRR(18,19,0)
      HRR(18,44,1)=HRR(18,44,1)+FnSpK*VRR(18,19,0)
      HRR(18,20,1)=HRR(18,20,1)+VRR(18,20,0)
      HRR(18,45,1)=HRR(18,45,1)+FnSpK*VRR(18,20,0)
      HRR(18,21,1)=HRR(18,21,1)+VRR(18,21,0)
      HRR(18,22,1)=HRR(18,22,1)+VRR(18,22,0)
      HRR(18,23,1)=HRR(18,23,1)+VRR(18,23,0)
      HRR(18,24,1)=HRR(18,24,1)+VRR(18,24,0)
      HRR(18,25,1)=HRR(18,25,1)+VRR(18,25,0)
      HRR(18,26,1)=HRR(18,26,1)+VRR(18,26,0)
      HRR(18,27,1)=HRR(18,27,1)+VRR(18,27,0)
      HRR(18,28,1)=HRR(18,28,1)+VRR(18,28,0)
      HRR(18,29,1)=HRR(18,29,1)+VRR(18,29,0)
      HRR(18,30,1)=HRR(18,30,1)+VRR(18,30,0)
      HRR(18,31,1)=HRR(18,31,1)+VRR(18,31,0)
      HRR(18,32,1)=HRR(18,32,1)+VRR(18,32,0)
      HRR(18,33,1)=HRR(18,33,1)+VRR(18,33,0)
      HRR(18,34,1)=HRR(18,34,1)+VRR(18,34,0)
      HRR(18,35,1)=HRR(18,35,1)+VRR(18,35,0)
      HRR(19,1,1)=HRR(19,1,1)+VRR(19,1,0)
      HRR(19,2,1)=HRR(19,2,1)+VRR(19,2,0)
      HRR(19,3,1)=HRR(19,3,1)+VRR(19,3,0)
      HRR(19,4,1)=HRR(19,4,1)+VRR(19,4,0)
      HRR(19,5,1)=HRR(19,5,1)+VRR(19,5,0)
      HRR(19,6,1)=HRR(19,6,1)+VRR(19,6,0)
      HRR(19,7,1)=HRR(19,7,1)+VRR(19,7,0)
      HRR(19,8,1)=HRR(19,8,1)+VRR(19,8,0)
      HRR(19,9,1)=HRR(19,9,1)+VRR(19,9,0)
      HRR(19,10,1)=HRR(19,10,1)+VRR(19,10,0)
      HRR(19,11,1)=HRR(19,11,1)+VRR(19,11,0)
      HRR(19,36,1)=HRR(19,36,1)+FnSpK*VRR(19,11,0)
      HRR(19,12,1)=HRR(19,12,1)+VRR(19,12,0)
      HRR(19,37,1)=HRR(19,37,1)+FnSpK*VRR(19,12,0)
      HRR(19,13,1)=HRR(19,13,1)+VRR(19,13,0)
      HRR(19,38,1)=HRR(19,38,1)+FnSpK*VRR(19,13,0)
      HRR(19,14,1)=HRR(19,14,1)+VRR(19,14,0)
      HRR(19,39,1)=HRR(19,39,1)+FnSpK*VRR(19,14,0)
      HRR(19,15,1)=HRR(19,15,1)+VRR(19,15,0)
      HRR(19,40,1)=HRR(19,40,1)+FnSpK*VRR(19,15,0)
      HRR(19,16,1)=HRR(19,16,1)+VRR(19,16,0)
      HRR(19,41,1)=HRR(19,41,1)+FnSpK*VRR(19,16,0)
      HRR(19,17,1)=HRR(19,17,1)+VRR(19,17,0)
      HRR(19,42,1)=HRR(19,42,1)+FnSpK*VRR(19,17,0)
      HRR(19,18,1)=HRR(19,18,1)+VRR(19,18,0)
      HRR(19,43,1)=HRR(19,43,1)+FnSpK*VRR(19,18,0)
      HRR(19,19,1)=HRR(19,19,1)+VRR(19,19,0)
      HRR(19,44,1)=HRR(19,44,1)+FnSpK*VRR(19,19,0)
      HRR(19,20,1)=HRR(19,20,1)+VRR(19,20,0)
      HRR(19,45,1)=HRR(19,45,1)+FnSpK*VRR(19,20,0)
      HRR(19,21,1)=HRR(19,21,1)+VRR(19,21,0)
      HRR(19,22,1)=HRR(19,22,1)+VRR(19,22,0)
      HRR(19,23,1)=HRR(19,23,1)+VRR(19,23,0)
      HRR(19,24,1)=HRR(19,24,1)+VRR(19,24,0)
      HRR(19,25,1)=HRR(19,25,1)+VRR(19,25,0)
      HRR(19,26,1)=HRR(19,26,1)+VRR(19,26,0)
      HRR(19,27,1)=HRR(19,27,1)+VRR(19,27,0)
      HRR(19,28,1)=HRR(19,28,1)+VRR(19,28,0)
      HRR(19,29,1)=HRR(19,29,1)+VRR(19,29,0)
      HRR(19,30,1)=HRR(19,30,1)+VRR(19,30,0)
      HRR(19,31,1)=HRR(19,31,1)+VRR(19,31,0)
      HRR(19,32,1)=HRR(19,32,1)+VRR(19,32,0)
      HRR(19,33,1)=HRR(19,33,1)+VRR(19,33,0)
      HRR(19,34,1)=HRR(19,34,1)+VRR(19,34,0)
      HRR(19,35,1)=HRR(19,35,1)+VRR(19,35,0)
      HRR(20,1,1)=HRR(20,1,1)+VRR(20,1,0)
      HRR(20,2,1)=HRR(20,2,1)+VRR(20,2,0)
      HRR(20,3,1)=HRR(20,3,1)+VRR(20,3,0)
      HRR(20,4,1)=HRR(20,4,1)+VRR(20,4,0)
      HRR(20,5,1)=HRR(20,5,1)+VRR(20,5,0)
      HRR(20,6,1)=HRR(20,6,1)+VRR(20,6,0)
      HRR(20,7,1)=HRR(20,7,1)+VRR(20,7,0)
      HRR(20,8,1)=HRR(20,8,1)+VRR(20,8,0)
      HRR(20,9,1)=HRR(20,9,1)+VRR(20,9,0)
      HRR(20,10,1)=HRR(20,10,1)+VRR(20,10,0)
      HRR(20,11,1)=HRR(20,11,1)+VRR(20,11,0)
      HRR(20,36,1)=HRR(20,36,1)+FnSpK*VRR(20,11,0)
      HRR(20,12,1)=HRR(20,12,1)+VRR(20,12,0)
      HRR(20,37,1)=HRR(20,37,1)+FnSpK*VRR(20,12,0)
      HRR(20,13,1)=HRR(20,13,1)+VRR(20,13,0)
      HRR(20,38,1)=HRR(20,38,1)+FnSpK*VRR(20,13,0)
      HRR(20,14,1)=HRR(20,14,1)+VRR(20,14,0)
      HRR(20,39,1)=HRR(20,39,1)+FnSpK*VRR(20,14,0)
      HRR(20,15,1)=HRR(20,15,1)+VRR(20,15,0)
      HRR(20,40,1)=HRR(20,40,1)+FnSpK*VRR(20,15,0)
      HRR(20,16,1)=HRR(20,16,1)+VRR(20,16,0)
      HRR(20,41,1)=HRR(20,41,1)+FnSpK*VRR(20,16,0)
      HRR(20,17,1)=HRR(20,17,1)+VRR(20,17,0)
      HRR(20,42,1)=HRR(20,42,1)+FnSpK*VRR(20,17,0)
      HRR(20,18,1)=HRR(20,18,1)+VRR(20,18,0)
      HRR(20,43,1)=HRR(20,43,1)+FnSpK*VRR(20,18,0)
      HRR(20,19,1)=HRR(20,19,1)+VRR(20,19,0)
      HRR(20,44,1)=HRR(20,44,1)+FnSpK*VRR(20,19,0)
      HRR(20,20,1)=HRR(20,20,1)+VRR(20,20,0)
      HRR(20,45,1)=HRR(20,45,1)+FnSpK*VRR(20,20,0)
      HRR(20,21,1)=HRR(20,21,1)+VRR(20,21,0)
      HRR(20,22,1)=HRR(20,22,1)+VRR(20,22,0)
      HRR(20,23,1)=HRR(20,23,1)+VRR(20,23,0)
      HRR(20,24,1)=HRR(20,24,1)+VRR(20,24,0)
      HRR(20,25,1)=HRR(20,25,1)+VRR(20,25,0)
      HRR(20,26,1)=HRR(20,26,1)+VRR(20,26,0)
      HRR(20,27,1)=HRR(20,27,1)+VRR(20,27,0)
      HRR(20,28,1)=HRR(20,28,1)+VRR(20,28,0)
      HRR(20,29,1)=HRR(20,29,1)+VRR(20,29,0)
      HRR(20,30,1)=HRR(20,30,1)+VRR(20,30,0)
      HRR(20,31,1)=HRR(20,31,1)+VRR(20,31,0)
      HRR(20,32,1)=HRR(20,32,1)+VRR(20,32,0)
      HRR(20,33,1)=HRR(20,33,1)+VRR(20,33,0)
      HRR(20,34,1)=HRR(20,34,1)+VRR(20,34,0)
      HRR(20,35,1)=HRR(20,35,1)+VRR(20,35,0)
      HRR(21,1,1)=HRR(21,1,1)+VRR(21,1,0)
      HRR(21,2,1)=HRR(21,2,1)+VRR(21,2,0)
      HRR(21,3,1)=HRR(21,3,1)+VRR(21,3,0)
      HRR(21,4,1)=HRR(21,4,1)+VRR(21,4,0)
      HRR(21,5,1)=HRR(21,5,1)+VRR(21,5,0)
      HRR(21,6,1)=HRR(21,6,1)+VRR(21,6,0)
      HRR(21,7,1)=HRR(21,7,1)+VRR(21,7,0)
      HRR(21,8,1)=HRR(21,8,1)+VRR(21,8,0)
      HRR(21,9,1)=HRR(21,9,1)+VRR(21,9,0)
      HRR(21,10,1)=HRR(21,10,1)+VRR(21,10,0)
      HRR(21,11,1)=HRR(21,11,1)+VRR(21,11,0)
      HRR(21,36,1)=HRR(21,36,1)+FnSpK*VRR(21,11,0)
      HRR(21,12,1)=HRR(21,12,1)+VRR(21,12,0)
      HRR(21,37,1)=HRR(21,37,1)+FnSpK*VRR(21,12,0)
      HRR(21,13,1)=HRR(21,13,1)+VRR(21,13,0)
      HRR(21,38,1)=HRR(21,38,1)+FnSpK*VRR(21,13,0)
      HRR(21,14,1)=HRR(21,14,1)+VRR(21,14,0)
      HRR(21,39,1)=HRR(21,39,1)+FnSpK*VRR(21,14,0)
      HRR(21,15,1)=HRR(21,15,1)+VRR(21,15,0)
      HRR(21,40,1)=HRR(21,40,1)+FnSpK*VRR(21,15,0)
      HRR(21,16,1)=HRR(21,16,1)+VRR(21,16,0)
      HRR(21,41,1)=HRR(21,41,1)+FnSpK*VRR(21,16,0)
      HRR(21,17,1)=HRR(21,17,1)+VRR(21,17,0)
      HRR(21,42,1)=HRR(21,42,1)+FnSpK*VRR(21,17,0)
      HRR(21,18,1)=HRR(21,18,1)+VRR(21,18,0)
      HRR(21,43,1)=HRR(21,43,1)+FnSpK*VRR(21,18,0)
      HRR(21,19,1)=HRR(21,19,1)+VRR(21,19,0)
      HRR(21,44,1)=HRR(21,44,1)+FnSpK*VRR(21,19,0)
      HRR(21,20,1)=HRR(21,20,1)+VRR(21,20,0)
      HRR(21,45,1)=HRR(21,45,1)+FnSpK*VRR(21,20,0)
      HRR(21,21,1)=HRR(21,21,1)+VRR(21,21,0)
      HRR(21,22,1)=HRR(21,22,1)+VRR(21,22,0)
      HRR(21,23,1)=HRR(21,23,1)+VRR(21,23,0)
      HRR(21,24,1)=HRR(21,24,1)+VRR(21,24,0)
      HRR(21,25,1)=HRR(21,25,1)+VRR(21,25,0)
      HRR(21,26,1)=HRR(21,26,1)+VRR(21,26,0)
      HRR(21,27,1)=HRR(21,27,1)+VRR(21,27,0)
      HRR(21,28,1)=HRR(21,28,1)+VRR(21,28,0)
      HRR(21,29,1)=HRR(21,29,1)+VRR(21,29,0)
      HRR(21,30,1)=HRR(21,30,1)+VRR(21,30,0)
      HRR(21,31,1)=HRR(21,31,1)+VRR(21,31,0)
      HRR(21,32,1)=HRR(21,32,1)+VRR(21,32,0)
      HRR(21,33,1)=HRR(21,33,1)+VRR(21,33,0)
      HRR(21,34,1)=HRR(21,34,1)+VRR(21,34,0)
      HRR(21,35,1)=HRR(21,35,1)+VRR(21,35,0)
      HRR(22,1,1)=HRR(22,1,1)+VRR(22,1,0)
      HRR(22,2,1)=HRR(22,2,1)+VRR(22,2,0)
      HRR(22,3,1)=HRR(22,3,1)+VRR(22,3,0)
      HRR(22,4,1)=HRR(22,4,1)+VRR(22,4,0)
      HRR(22,5,1)=HRR(22,5,1)+VRR(22,5,0)
      HRR(22,6,1)=HRR(22,6,1)+VRR(22,6,0)
      HRR(22,7,1)=HRR(22,7,1)+VRR(22,7,0)
      HRR(22,8,1)=HRR(22,8,1)+VRR(22,8,0)
      HRR(22,9,1)=HRR(22,9,1)+VRR(22,9,0)
      HRR(22,10,1)=HRR(22,10,1)+VRR(22,10,0)
      HRR(22,11,1)=HRR(22,11,1)+VRR(22,11,0)
      HRR(22,36,1)=HRR(22,36,1)+FnSpK*VRR(22,11,0)
      HRR(22,12,1)=HRR(22,12,1)+VRR(22,12,0)
      HRR(22,37,1)=HRR(22,37,1)+FnSpK*VRR(22,12,0)
      HRR(22,13,1)=HRR(22,13,1)+VRR(22,13,0)
      HRR(22,38,1)=HRR(22,38,1)+FnSpK*VRR(22,13,0)
      HRR(22,14,1)=HRR(22,14,1)+VRR(22,14,0)
      HRR(22,39,1)=HRR(22,39,1)+FnSpK*VRR(22,14,0)
      HRR(22,15,1)=HRR(22,15,1)+VRR(22,15,0)
      HRR(22,40,1)=HRR(22,40,1)+FnSpK*VRR(22,15,0)
      HRR(22,16,1)=HRR(22,16,1)+VRR(22,16,0)
      HRR(22,41,1)=HRR(22,41,1)+FnSpK*VRR(22,16,0)
      HRR(22,17,1)=HRR(22,17,1)+VRR(22,17,0)
      HRR(22,42,1)=HRR(22,42,1)+FnSpK*VRR(22,17,0)
      HRR(22,18,1)=HRR(22,18,1)+VRR(22,18,0)
      HRR(22,43,1)=HRR(22,43,1)+FnSpK*VRR(22,18,0)
      HRR(22,19,1)=HRR(22,19,1)+VRR(22,19,0)
      HRR(22,44,1)=HRR(22,44,1)+FnSpK*VRR(22,19,0)
      HRR(22,20,1)=HRR(22,20,1)+VRR(22,20,0)
      HRR(22,45,1)=HRR(22,45,1)+FnSpK*VRR(22,20,0)
      HRR(22,21,1)=HRR(22,21,1)+VRR(22,21,0)
      HRR(22,22,1)=HRR(22,22,1)+VRR(22,22,0)
      HRR(22,23,1)=HRR(22,23,1)+VRR(22,23,0)
      HRR(22,24,1)=HRR(22,24,1)+VRR(22,24,0)
      HRR(22,25,1)=HRR(22,25,1)+VRR(22,25,0)
      HRR(22,26,1)=HRR(22,26,1)+VRR(22,26,0)
      HRR(22,27,1)=HRR(22,27,1)+VRR(22,27,0)
      HRR(22,28,1)=HRR(22,28,1)+VRR(22,28,0)
      HRR(22,29,1)=HRR(22,29,1)+VRR(22,29,0)
      HRR(22,30,1)=HRR(22,30,1)+VRR(22,30,0)
      HRR(22,31,1)=HRR(22,31,1)+VRR(22,31,0)
      HRR(22,32,1)=HRR(22,32,1)+VRR(22,32,0)
      HRR(22,33,1)=HRR(22,33,1)+VRR(22,33,0)
      HRR(22,34,1)=HRR(22,34,1)+VRR(22,34,0)
      HRR(22,35,1)=HRR(22,35,1)+VRR(22,35,0)
      HRR(23,1,1)=HRR(23,1,1)+VRR(23,1,0)
      HRR(23,2,1)=HRR(23,2,1)+VRR(23,2,0)
      HRR(23,3,1)=HRR(23,3,1)+VRR(23,3,0)
      HRR(23,4,1)=HRR(23,4,1)+VRR(23,4,0)
      HRR(23,5,1)=HRR(23,5,1)+VRR(23,5,0)
      HRR(23,6,1)=HRR(23,6,1)+VRR(23,6,0)
      HRR(23,7,1)=HRR(23,7,1)+VRR(23,7,0)
      HRR(23,8,1)=HRR(23,8,1)+VRR(23,8,0)
      HRR(23,9,1)=HRR(23,9,1)+VRR(23,9,0)
      HRR(23,10,1)=HRR(23,10,1)+VRR(23,10,0)
      HRR(23,11,1)=HRR(23,11,1)+VRR(23,11,0)
      HRR(23,36,1)=HRR(23,36,1)+FnSpK*VRR(23,11,0)
      HRR(23,12,1)=HRR(23,12,1)+VRR(23,12,0)
      HRR(23,37,1)=HRR(23,37,1)+FnSpK*VRR(23,12,0)
      HRR(23,13,1)=HRR(23,13,1)+VRR(23,13,0)
      HRR(23,38,1)=HRR(23,38,1)+FnSpK*VRR(23,13,0)
      HRR(23,14,1)=HRR(23,14,1)+VRR(23,14,0)
      HRR(23,39,1)=HRR(23,39,1)+FnSpK*VRR(23,14,0)
      HRR(23,15,1)=HRR(23,15,1)+VRR(23,15,0)
      HRR(23,40,1)=HRR(23,40,1)+FnSpK*VRR(23,15,0)
      HRR(23,16,1)=HRR(23,16,1)+VRR(23,16,0)
      HRR(23,41,1)=HRR(23,41,1)+FnSpK*VRR(23,16,0)
      HRR(23,17,1)=HRR(23,17,1)+VRR(23,17,0)
      HRR(23,42,1)=HRR(23,42,1)+FnSpK*VRR(23,17,0)
      HRR(23,18,1)=HRR(23,18,1)+VRR(23,18,0)
      HRR(23,43,1)=HRR(23,43,1)+FnSpK*VRR(23,18,0)
      HRR(23,19,1)=HRR(23,19,1)+VRR(23,19,0)
      HRR(23,44,1)=HRR(23,44,1)+FnSpK*VRR(23,19,0)
      HRR(23,20,1)=HRR(23,20,1)+VRR(23,20,0)
      HRR(23,45,1)=HRR(23,45,1)+FnSpK*VRR(23,20,0)
      HRR(23,21,1)=HRR(23,21,1)+VRR(23,21,0)
      HRR(23,22,1)=HRR(23,22,1)+VRR(23,22,0)
      HRR(23,23,1)=HRR(23,23,1)+VRR(23,23,0)
      HRR(23,24,1)=HRR(23,24,1)+VRR(23,24,0)
      HRR(23,25,1)=HRR(23,25,1)+VRR(23,25,0)
      HRR(23,26,1)=HRR(23,26,1)+VRR(23,26,0)
      HRR(23,27,1)=HRR(23,27,1)+VRR(23,27,0)
      HRR(23,28,1)=HRR(23,28,1)+VRR(23,28,0)
      HRR(23,29,1)=HRR(23,29,1)+VRR(23,29,0)
      HRR(23,30,1)=HRR(23,30,1)+VRR(23,30,0)
      HRR(23,31,1)=HRR(23,31,1)+VRR(23,31,0)
      HRR(23,32,1)=HRR(23,32,1)+VRR(23,32,0)
      HRR(23,33,1)=HRR(23,33,1)+VRR(23,33,0)
      HRR(23,34,1)=HRR(23,34,1)+VRR(23,34,0)
      HRR(23,35,1)=HRR(23,35,1)+VRR(23,35,0)
      HRR(24,1,1)=HRR(24,1,1)+VRR(24,1,0)
      HRR(24,2,1)=HRR(24,2,1)+VRR(24,2,0)
      HRR(24,3,1)=HRR(24,3,1)+VRR(24,3,0)
      HRR(24,4,1)=HRR(24,4,1)+VRR(24,4,0)
      HRR(24,5,1)=HRR(24,5,1)+VRR(24,5,0)
      HRR(24,6,1)=HRR(24,6,1)+VRR(24,6,0)
      HRR(24,7,1)=HRR(24,7,1)+VRR(24,7,0)
      HRR(24,8,1)=HRR(24,8,1)+VRR(24,8,0)
      HRR(24,9,1)=HRR(24,9,1)+VRR(24,9,0)
      HRR(24,10,1)=HRR(24,10,1)+VRR(24,10,0)
      HRR(24,11,1)=HRR(24,11,1)+VRR(24,11,0)
      HRR(24,36,1)=HRR(24,36,1)+FnSpK*VRR(24,11,0)
      HRR(24,12,1)=HRR(24,12,1)+VRR(24,12,0)
      HRR(24,37,1)=HRR(24,37,1)+FnSpK*VRR(24,12,0)
      HRR(24,13,1)=HRR(24,13,1)+VRR(24,13,0)
      HRR(24,38,1)=HRR(24,38,1)+FnSpK*VRR(24,13,0)
      HRR(24,14,1)=HRR(24,14,1)+VRR(24,14,0)
      HRR(24,39,1)=HRR(24,39,1)+FnSpK*VRR(24,14,0)
      HRR(24,15,1)=HRR(24,15,1)+VRR(24,15,0)
      HRR(24,40,1)=HRR(24,40,1)+FnSpK*VRR(24,15,0)
      HRR(24,16,1)=HRR(24,16,1)+VRR(24,16,0)
      HRR(24,41,1)=HRR(24,41,1)+FnSpK*VRR(24,16,0)
      HRR(24,17,1)=HRR(24,17,1)+VRR(24,17,0)
      HRR(24,42,1)=HRR(24,42,1)+FnSpK*VRR(24,17,0)
      HRR(24,18,1)=HRR(24,18,1)+VRR(24,18,0)
      HRR(24,43,1)=HRR(24,43,1)+FnSpK*VRR(24,18,0)
      HRR(24,19,1)=HRR(24,19,1)+VRR(24,19,0)
      HRR(24,44,1)=HRR(24,44,1)+FnSpK*VRR(24,19,0)
      HRR(24,20,1)=HRR(24,20,1)+VRR(24,20,0)
      HRR(24,45,1)=HRR(24,45,1)+FnSpK*VRR(24,20,0)
      HRR(24,21,1)=HRR(24,21,1)+VRR(24,21,0)
      HRR(24,22,1)=HRR(24,22,1)+VRR(24,22,0)
      HRR(24,23,1)=HRR(24,23,1)+VRR(24,23,0)
      HRR(24,24,1)=HRR(24,24,1)+VRR(24,24,0)
      HRR(24,25,1)=HRR(24,25,1)+VRR(24,25,0)
      HRR(24,26,1)=HRR(24,26,1)+VRR(24,26,0)
      HRR(24,27,1)=HRR(24,27,1)+VRR(24,27,0)
      HRR(24,28,1)=HRR(24,28,1)+VRR(24,28,0)
      HRR(24,29,1)=HRR(24,29,1)+VRR(24,29,0)
      HRR(24,30,1)=HRR(24,30,1)+VRR(24,30,0)
      HRR(24,31,1)=HRR(24,31,1)+VRR(24,31,0)
      HRR(24,32,1)=HRR(24,32,1)+VRR(24,32,0)
      HRR(24,33,1)=HRR(24,33,1)+VRR(24,33,0)
      HRR(24,34,1)=HRR(24,34,1)+VRR(24,34,0)
      HRR(24,35,1)=HRR(24,35,1)+VRR(24,35,0)
      HRR(25,1,1)=HRR(25,1,1)+VRR(25,1,0)
      HRR(25,2,1)=HRR(25,2,1)+VRR(25,2,0)
      HRR(25,3,1)=HRR(25,3,1)+VRR(25,3,0)
      HRR(25,4,1)=HRR(25,4,1)+VRR(25,4,0)
      HRR(25,5,1)=HRR(25,5,1)+VRR(25,5,0)
      HRR(25,6,1)=HRR(25,6,1)+VRR(25,6,0)
      HRR(25,7,1)=HRR(25,7,1)+VRR(25,7,0)
      HRR(25,8,1)=HRR(25,8,1)+VRR(25,8,0)
      HRR(25,9,1)=HRR(25,9,1)+VRR(25,9,0)
      HRR(25,10,1)=HRR(25,10,1)+VRR(25,10,0)
      HRR(25,11,1)=HRR(25,11,1)+VRR(25,11,0)
      HRR(25,36,1)=HRR(25,36,1)+FnSpK*VRR(25,11,0)
      HRR(25,12,1)=HRR(25,12,1)+VRR(25,12,0)
      HRR(25,37,1)=HRR(25,37,1)+FnSpK*VRR(25,12,0)
      HRR(25,13,1)=HRR(25,13,1)+VRR(25,13,0)
      HRR(25,38,1)=HRR(25,38,1)+FnSpK*VRR(25,13,0)
      HRR(25,14,1)=HRR(25,14,1)+VRR(25,14,0)
      HRR(25,39,1)=HRR(25,39,1)+FnSpK*VRR(25,14,0)
      HRR(25,15,1)=HRR(25,15,1)+VRR(25,15,0)
      HRR(25,40,1)=HRR(25,40,1)+FnSpK*VRR(25,15,0)
      HRR(25,16,1)=HRR(25,16,1)+VRR(25,16,0)
      HRR(25,41,1)=HRR(25,41,1)+FnSpK*VRR(25,16,0)
      HRR(25,17,1)=HRR(25,17,1)+VRR(25,17,0)
      HRR(25,42,1)=HRR(25,42,1)+FnSpK*VRR(25,17,0)
      HRR(25,18,1)=HRR(25,18,1)+VRR(25,18,0)
      HRR(25,43,1)=HRR(25,43,1)+FnSpK*VRR(25,18,0)
      HRR(25,19,1)=HRR(25,19,1)+VRR(25,19,0)
      HRR(25,44,1)=HRR(25,44,1)+FnSpK*VRR(25,19,0)
      HRR(25,20,1)=HRR(25,20,1)+VRR(25,20,0)
      HRR(25,45,1)=HRR(25,45,1)+FnSpK*VRR(25,20,0)
      HRR(25,21,1)=HRR(25,21,1)+VRR(25,21,0)
      HRR(25,22,1)=HRR(25,22,1)+VRR(25,22,0)
      HRR(25,23,1)=HRR(25,23,1)+VRR(25,23,0)
      HRR(25,24,1)=HRR(25,24,1)+VRR(25,24,0)
      HRR(25,25,1)=HRR(25,25,1)+VRR(25,25,0)
      HRR(25,26,1)=HRR(25,26,1)+VRR(25,26,0)
      HRR(25,27,1)=HRR(25,27,1)+VRR(25,27,0)
      HRR(25,28,1)=HRR(25,28,1)+VRR(25,28,0)
      HRR(25,29,1)=HRR(25,29,1)+VRR(25,29,0)
      HRR(25,30,1)=HRR(25,30,1)+VRR(25,30,0)
      HRR(25,31,1)=HRR(25,31,1)+VRR(25,31,0)
      HRR(25,32,1)=HRR(25,32,1)+VRR(25,32,0)
      HRR(25,33,1)=HRR(25,33,1)+VRR(25,33,0)
      HRR(25,34,1)=HRR(25,34,1)+VRR(25,34,0)
      HRR(25,35,1)=HRR(25,35,1)+VRR(25,35,0)
      HRR(26,1,1)=HRR(26,1,1)+VRR(26,1,0)
      HRR(26,2,1)=HRR(26,2,1)+VRR(26,2,0)
      HRR(26,3,1)=HRR(26,3,1)+VRR(26,3,0)
      HRR(26,4,1)=HRR(26,4,1)+VRR(26,4,0)
      HRR(26,5,1)=HRR(26,5,1)+VRR(26,5,0)
      HRR(26,6,1)=HRR(26,6,1)+VRR(26,6,0)
      HRR(26,7,1)=HRR(26,7,1)+VRR(26,7,0)
      HRR(26,8,1)=HRR(26,8,1)+VRR(26,8,0)
      HRR(26,9,1)=HRR(26,9,1)+VRR(26,9,0)
      HRR(26,10,1)=HRR(26,10,1)+VRR(26,10,0)
      HRR(26,11,1)=HRR(26,11,1)+VRR(26,11,0)
      HRR(26,36,1)=HRR(26,36,1)+FnSpK*VRR(26,11,0)
      HRR(26,12,1)=HRR(26,12,1)+VRR(26,12,0)
      HRR(26,37,1)=HRR(26,37,1)+FnSpK*VRR(26,12,0)
      HRR(26,13,1)=HRR(26,13,1)+VRR(26,13,0)
      HRR(26,38,1)=HRR(26,38,1)+FnSpK*VRR(26,13,0)
      HRR(26,14,1)=HRR(26,14,1)+VRR(26,14,0)
      HRR(26,39,1)=HRR(26,39,1)+FnSpK*VRR(26,14,0)
      HRR(26,15,1)=HRR(26,15,1)+VRR(26,15,0)
      HRR(26,40,1)=HRR(26,40,1)+FnSpK*VRR(26,15,0)
      HRR(26,16,1)=HRR(26,16,1)+VRR(26,16,0)
      HRR(26,41,1)=HRR(26,41,1)+FnSpK*VRR(26,16,0)
      HRR(26,17,1)=HRR(26,17,1)+VRR(26,17,0)
      HRR(26,42,1)=HRR(26,42,1)+FnSpK*VRR(26,17,0)
      HRR(26,18,1)=HRR(26,18,1)+VRR(26,18,0)
      HRR(26,43,1)=HRR(26,43,1)+FnSpK*VRR(26,18,0)
      HRR(26,19,1)=HRR(26,19,1)+VRR(26,19,0)
      HRR(26,44,1)=HRR(26,44,1)+FnSpK*VRR(26,19,0)
      HRR(26,20,1)=HRR(26,20,1)+VRR(26,20,0)
      HRR(26,45,1)=HRR(26,45,1)+FnSpK*VRR(26,20,0)
      HRR(26,21,1)=HRR(26,21,1)+VRR(26,21,0)
      HRR(26,22,1)=HRR(26,22,1)+VRR(26,22,0)
      HRR(26,23,1)=HRR(26,23,1)+VRR(26,23,0)
      HRR(26,24,1)=HRR(26,24,1)+VRR(26,24,0)
      HRR(26,25,1)=HRR(26,25,1)+VRR(26,25,0)
      HRR(26,26,1)=HRR(26,26,1)+VRR(26,26,0)
      HRR(26,27,1)=HRR(26,27,1)+VRR(26,27,0)
      HRR(26,28,1)=HRR(26,28,1)+VRR(26,28,0)
      HRR(26,29,1)=HRR(26,29,1)+VRR(26,29,0)
      HRR(26,30,1)=HRR(26,30,1)+VRR(26,30,0)
      HRR(26,31,1)=HRR(26,31,1)+VRR(26,31,0)
      HRR(26,32,1)=HRR(26,32,1)+VRR(26,32,0)
      HRR(26,33,1)=HRR(26,33,1)+VRR(26,33,0)
      HRR(26,34,1)=HRR(26,34,1)+VRR(26,34,0)
      HRR(26,35,1)=HRR(26,35,1)+VRR(26,35,0)
      HRR(27,1,1)=HRR(27,1,1)+VRR(27,1,0)
      HRR(27,2,1)=HRR(27,2,1)+VRR(27,2,0)
      HRR(27,3,1)=HRR(27,3,1)+VRR(27,3,0)
      HRR(27,4,1)=HRR(27,4,1)+VRR(27,4,0)
      HRR(27,5,1)=HRR(27,5,1)+VRR(27,5,0)
      HRR(27,6,1)=HRR(27,6,1)+VRR(27,6,0)
      HRR(27,7,1)=HRR(27,7,1)+VRR(27,7,0)
      HRR(27,8,1)=HRR(27,8,1)+VRR(27,8,0)
      HRR(27,9,1)=HRR(27,9,1)+VRR(27,9,0)
      HRR(27,10,1)=HRR(27,10,1)+VRR(27,10,0)
      HRR(27,11,1)=HRR(27,11,1)+VRR(27,11,0)
      HRR(27,36,1)=HRR(27,36,1)+FnSpK*VRR(27,11,0)
      HRR(27,12,1)=HRR(27,12,1)+VRR(27,12,0)
      HRR(27,37,1)=HRR(27,37,1)+FnSpK*VRR(27,12,0)
      HRR(27,13,1)=HRR(27,13,1)+VRR(27,13,0)
      HRR(27,38,1)=HRR(27,38,1)+FnSpK*VRR(27,13,0)
      HRR(27,14,1)=HRR(27,14,1)+VRR(27,14,0)
      HRR(27,39,1)=HRR(27,39,1)+FnSpK*VRR(27,14,0)
      HRR(27,15,1)=HRR(27,15,1)+VRR(27,15,0)
      HRR(27,40,1)=HRR(27,40,1)+FnSpK*VRR(27,15,0)
      HRR(27,16,1)=HRR(27,16,1)+VRR(27,16,0)
      HRR(27,41,1)=HRR(27,41,1)+FnSpK*VRR(27,16,0)
      HRR(27,17,1)=HRR(27,17,1)+VRR(27,17,0)
      HRR(27,42,1)=HRR(27,42,1)+FnSpK*VRR(27,17,0)
      HRR(27,18,1)=HRR(27,18,1)+VRR(27,18,0)
      HRR(27,43,1)=HRR(27,43,1)+FnSpK*VRR(27,18,0)
      HRR(27,19,1)=HRR(27,19,1)+VRR(27,19,0)
      HRR(27,44,1)=HRR(27,44,1)+FnSpK*VRR(27,19,0)
      HRR(27,20,1)=HRR(27,20,1)+VRR(27,20,0)
      HRR(27,45,1)=HRR(27,45,1)+FnSpK*VRR(27,20,0)
      HRR(27,21,1)=HRR(27,21,1)+VRR(27,21,0)
      HRR(27,22,1)=HRR(27,22,1)+VRR(27,22,0)
      HRR(27,23,1)=HRR(27,23,1)+VRR(27,23,0)
      HRR(27,24,1)=HRR(27,24,1)+VRR(27,24,0)
      HRR(27,25,1)=HRR(27,25,1)+VRR(27,25,0)
      HRR(27,26,1)=HRR(27,26,1)+VRR(27,26,0)
      HRR(27,27,1)=HRR(27,27,1)+VRR(27,27,0)
      HRR(27,28,1)=HRR(27,28,1)+VRR(27,28,0)
      HRR(27,29,1)=HRR(27,29,1)+VRR(27,29,0)
      HRR(27,30,1)=HRR(27,30,1)+VRR(27,30,0)
      HRR(27,31,1)=HRR(27,31,1)+VRR(27,31,0)
      HRR(27,32,1)=HRR(27,32,1)+VRR(27,32,0)
      HRR(27,33,1)=HRR(27,33,1)+VRR(27,33,0)
      HRR(27,34,1)=HRR(27,34,1)+VRR(27,34,0)
      HRR(27,35,1)=HRR(27,35,1)+VRR(27,35,0)
      HRR(28,1,1)=HRR(28,1,1)+VRR(28,1,0)
      HRR(28,2,1)=HRR(28,2,1)+VRR(28,2,0)
      HRR(28,3,1)=HRR(28,3,1)+VRR(28,3,0)
      HRR(28,4,1)=HRR(28,4,1)+VRR(28,4,0)
      HRR(28,5,1)=HRR(28,5,1)+VRR(28,5,0)
      HRR(28,6,1)=HRR(28,6,1)+VRR(28,6,0)
      HRR(28,7,1)=HRR(28,7,1)+VRR(28,7,0)
      HRR(28,8,1)=HRR(28,8,1)+VRR(28,8,0)
      HRR(28,9,1)=HRR(28,9,1)+VRR(28,9,0)
      HRR(28,10,1)=HRR(28,10,1)+VRR(28,10,0)
      HRR(28,11,1)=HRR(28,11,1)+VRR(28,11,0)
      HRR(28,36,1)=HRR(28,36,1)+FnSpK*VRR(28,11,0)
      HRR(28,12,1)=HRR(28,12,1)+VRR(28,12,0)
      HRR(28,37,1)=HRR(28,37,1)+FnSpK*VRR(28,12,0)
      HRR(28,13,1)=HRR(28,13,1)+VRR(28,13,0)
      HRR(28,38,1)=HRR(28,38,1)+FnSpK*VRR(28,13,0)
      HRR(28,14,1)=HRR(28,14,1)+VRR(28,14,0)
      HRR(28,39,1)=HRR(28,39,1)+FnSpK*VRR(28,14,0)
      HRR(28,15,1)=HRR(28,15,1)+VRR(28,15,0)
      HRR(28,40,1)=HRR(28,40,1)+FnSpK*VRR(28,15,0)
      HRR(28,16,1)=HRR(28,16,1)+VRR(28,16,0)
      HRR(28,41,1)=HRR(28,41,1)+FnSpK*VRR(28,16,0)
      HRR(28,17,1)=HRR(28,17,1)+VRR(28,17,0)
      HRR(28,42,1)=HRR(28,42,1)+FnSpK*VRR(28,17,0)
      HRR(28,18,1)=HRR(28,18,1)+VRR(28,18,0)
      HRR(28,43,1)=HRR(28,43,1)+FnSpK*VRR(28,18,0)
      HRR(28,19,1)=HRR(28,19,1)+VRR(28,19,0)
      HRR(28,44,1)=HRR(28,44,1)+FnSpK*VRR(28,19,0)
      HRR(28,20,1)=HRR(28,20,1)+VRR(28,20,0)
      HRR(28,45,1)=HRR(28,45,1)+FnSpK*VRR(28,20,0)
      HRR(28,21,1)=HRR(28,21,1)+VRR(28,21,0)
      HRR(28,22,1)=HRR(28,22,1)+VRR(28,22,0)
      HRR(28,23,1)=HRR(28,23,1)+VRR(28,23,0)
      HRR(28,24,1)=HRR(28,24,1)+VRR(28,24,0)
      HRR(28,25,1)=HRR(28,25,1)+VRR(28,25,0)
      HRR(28,26,1)=HRR(28,26,1)+VRR(28,26,0)
      HRR(28,27,1)=HRR(28,27,1)+VRR(28,27,0)
      HRR(28,28,1)=HRR(28,28,1)+VRR(28,28,0)
      HRR(28,29,1)=HRR(28,29,1)+VRR(28,29,0)
      HRR(28,30,1)=HRR(28,30,1)+VRR(28,30,0)
      HRR(28,31,1)=HRR(28,31,1)+VRR(28,31,0)
      HRR(28,32,1)=HRR(28,32,1)+VRR(28,32,0)
      HRR(28,33,1)=HRR(28,33,1)+VRR(28,33,0)
      HRR(28,34,1)=HRR(28,34,1)+VRR(28,34,0)
      HRR(28,35,1)=HRR(28,35,1)+VRR(28,35,0)
      HRR(29,1,1)=HRR(29,1,1)+VRR(29,1,0)
      HRR(29,2,1)=HRR(29,2,1)+VRR(29,2,0)
      HRR(29,3,1)=HRR(29,3,1)+VRR(29,3,0)
      HRR(29,4,1)=HRR(29,4,1)+VRR(29,4,0)
      HRR(29,5,1)=HRR(29,5,1)+VRR(29,5,0)
      HRR(29,6,1)=HRR(29,6,1)+VRR(29,6,0)
      HRR(29,7,1)=HRR(29,7,1)+VRR(29,7,0)
      HRR(29,8,1)=HRR(29,8,1)+VRR(29,8,0)
      HRR(29,9,1)=HRR(29,9,1)+VRR(29,9,0)
      HRR(29,10,1)=HRR(29,10,1)+VRR(29,10,0)
      HRR(29,11,1)=HRR(29,11,1)+VRR(29,11,0)
      HRR(29,36,1)=HRR(29,36,1)+FnSpK*VRR(29,11,0)
      HRR(29,12,1)=HRR(29,12,1)+VRR(29,12,0)
      HRR(29,37,1)=HRR(29,37,1)+FnSpK*VRR(29,12,0)
      HRR(29,13,1)=HRR(29,13,1)+VRR(29,13,0)
      HRR(29,38,1)=HRR(29,38,1)+FnSpK*VRR(29,13,0)
      HRR(29,14,1)=HRR(29,14,1)+VRR(29,14,0)
      HRR(29,39,1)=HRR(29,39,1)+FnSpK*VRR(29,14,0)
      HRR(29,15,1)=HRR(29,15,1)+VRR(29,15,0)
      HRR(29,40,1)=HRR(29,40,1)+FnSpK*VRR(29,15,0)
      HRR(29,16,1)=HRR(29,16,1)+VRR(29,16,0)
      HRR(29,41,1)=HRR(29,41,1)+FnSpK*VRR(29,16,0)
      HRR(29,17,1)=HRR(29,17,1)+VRR(29,17,0)
      HRR(29,42,1)=HRR(29,42,1)+FnSpK*VRR(29,17,0)
      HRR(29,18,1)=HRR(29,18,1)+VRR(29,18,0)
      HRR(29,43,1)=HRR(29,43,1)+FnSpK*VRR(29,18,0)
      HRR(29,19,1)=HRR(29,19,1)+VRR(29,19,0)
      HRR(29,44,1)=HRR(29,44,1)+FnSpK*VRR(29,19,0)
      HRR(29,20,1)=HRR(29,20,1)+VRR(29,20,0)
      HRR(29,45,1)=HRR(29,45,1)+FnSpK*VRR(29,20,0)
      HRR(29,21,1)=HRR(29,21,1)+VRR(29,21,0)
      HRR(29,22,1)=HRR(29,22,1)+VRR(29,22,0)
      HRR(29,23,1)=HRR(29,23,1)+VRR(29,23,0)
      HRR(29,24,1)=HRR(29,24,1)+VRR(29,24,0)
      HRR(29,25,1)=HRR(29,25,1)+VRR(29,25,0)
      HRR(29,26,1)=HRR(29,26,1)+VRR(29,26,0)
      HRR(29,27,1)=HRR(29,27,1)+VRR(29,27,0)
      HRR(29,28,1)=HRR(29,28,1)+VRR(29,28,0)
      HRR(29,29,1)=HRR(29,29,1)+VRR(29,29,0)
      HRR(29,30,1)=HRR(29,30,1)+VRR(29,30,0)
      HRR(29,31,1)=HRR(29,31,1)+VRR(29,31,0)
      HRR(29,32,1)=HRR(29,32,1)+VRR(29,32,0)
      HRR(29,33,1)=HRR(29,33,1)+VRR(29,33,0)
      HRR(29,34,1)=HRR(29,34,1)+VRR(29,34,0)
      HRR(29,35,1)=HRR(29,35,1)+VRR(29,35,0)
      HRR(30,1,1)=HRR(30,1,1)+VRR(30,1,0)
      HRR(30,2,1)=HRR(30,2,1)+VRR(30,2,0)
      HRR(30,3,1)=HRR(30,3,1)+VRR(30,3,0)
      HRR(30,4,1)=HRR(30,4,1)+VRR(30,4,0)
      HRR(30,5,1)=HRR(30,5,1)+VRR(30,5,0)
      HRR(30,6,1)=HRR(30,6,1)+VRR(30,6,0)
      HRR(30,7,1)=HRR(30,7,1)+VRR(30,7,0)
      HRR(30,8,1)=HRR(30,8,1)+VRR(30,8,0)
      HRR(30,9,1)=HRR(30,9,1)+VRR(30,9,0)
      HRR(30,10,1)=HRR(30,10,1)+VRR(30,10,0)
      HRR(30,11,1)=HRR(30,11,1)+VRR(30,11,0)
      HRR(30,36,1)=HRR(30,36,1)+FnSpK*VRR(30,11,0)
      HRR(30,12,1)=HRR(30,12,1)+VRR(30,12,0)
      HRR(30,37,1)=HRR(30,37,1)+FnSpK*VRR(30,12,0)
      HRR(30,13,1)=HRR(30,13,1)+VRR(30,13,0)
      HRR(30,38,1)=HRR(30,38,1)+FnSpK*VRR(30,13,0)
      HRR(30,14,1)=HRR(30,14,1)+VRR(30,14,0)
      HRR(30,39,1)=HRR(30,39,1)+FnSpK*VRR(30,14,0)
      HRR(30,15,1)=HRR(30,15,1)+VRR(30,15,0)
      HRR(30,40,1)=HRR(30,40,1)+FnSpK*VRR(30,15,0)
      HRR(30,16,1)=HRR(30,16,1)+VRR(30,16,0)
      HRR(30,41,1)=HRR(30,41,1)+FnSpK*VRR(30,16,0)
      HRR(30,17,1)=HRR(30,17,1)+VRR(30,17,0)
      HRR(30,42,1)=HRR(30,42,1)+FnSpK*VRR(30,17,0)
      HRR(30,18,1)=HRR(30,18,1)+VRR(30,18,0)
      HRR(30,43,1)=HRR(30,43,1)+FnSpK*VRR(30,18,0)
      HRR(30,19,1)=HRR(30,19,1)+VRR(30,19,0)
      HRR(30,44,1)=HRR(30,44,1)+FnSpK*VRR(30,19,0)
      HRR(30,20,1)=HRR(30,20,1)+VRR(30,20,0)
      HRR(30,45,1)=HRR(30,45,1)+FnSpK*VRR(30,20,0)
      HRR(30,21,1)=HRR(30,21,1)+VRR(30,21,0)
      HRR(30,22,1)=HRR(30,22,1)+VRR(30,22,0)
      HRR(30,23,1)=HRR(30,23,1)+VRR(30,23,0)
      HRR(30,24,1)=HRR(30,24,1)+VRR(30,24,0)
      HRR(30,25,1)=HRR(30,25,1)+VRR(30,25,0)
      HRR(30,26,1)=HRR(30,26,1)+VRR(30,26,0)
      HRR(30,27,1)=HRR(30,27,1)+VRR(30,27,0)
      HRR(30,28,1)=HRR(30,28,1)+VRR(30,28,0)
      HRR(30,29,1)=HRR(30,29,1)+VRR(30,29,0)
      HRR(30,30,1)=HRR(30,30,1)+VRR(30,30,0)
      HRR(30,31,1)=HRR(30,31,1)+VRR(30,31,0)
      HRR(30,32,1)=HRR(30,32,1)+VRR(30,32,0)
      HRR(30,33,1)=HRR(30,33,1)+VRR(30,33,0)
      HRR(30,34,1)=HRR(30,34,1)+VRR(30,34,0)
      HRR(30,35,1)=HRR(30,35,1)+VRR(30,35,0)
      HRR(31,1,1)=HRR(31,1,1)+VRR(31,1,0)
      HRR(31,2,1)=HRR(31,2,1)+VRR(31,2,0)
      HRR(31,3,1)=HRR(31,3,1)+VRR(31,3,0)
      HRR(31,4,1)=HRR(31,4,1)+VRR(31,4,0)
      HRR(31,5,1)=HRR(31,5,1)+VRR(31,5,0)
      HRR(31,6,1)=HRR(31,6,1)+VRR(31,6,0)
      HRR(31,7,1)=HRR(31,7,1)+VRR(31,7,0)
      HRR(31,8,1)=HRR(31,8,1)+VRR(31,8,0)
      HRR(31,9,1)=HRR(31,9,1)+VRR(31,9,0)
      HRR(31,10,1)=HRR(31,10,1)+VRR(31,10,0)
      HRR(31,11,1)=HRR(31,11,1)+VRR(31,11,0)
      HRR(31,36,1)=HRR(31,36,1)+FnSpK*VRR(31,11,0)
      HRR(31,12,1)=HRR(31,12,1)+VRR(31,12,0)
      HRR(31,37,1)=HRR(31,37,1)+FnSpK*VRR(31,12,0)
      HRR(31,13,1)=HRR(31,13,1)+VRR(31,13,0)
      HRR(31,38,1)=HRR(31,38,1)+FnSpK*VRR(31,13,0)
      HRR(31,14,1)=HRR(31,14,1)+VRR(31,14,0)
      HRR(31,39,1)=HRR(31,39,1)+FnSpK*VRR(31,14,0)
      HRR(31,15,1)=HRR(31,15,1)+VRR(31,15,0)
      HRR(31,40,1)=HRR(31,40,1)+FnSpK*VRR(31,15,0)
      HRR(31,16,1)=HRR(31,16,1)+VRR(31,16,0)
      HRR(31,41,1)=HRR(31,41,1)+FnSpK*VRR(31,16,0)
      HRR(31,17,1)=HRR(31,17,1)+VRR(31,17,0)
      HRR(31,42,1)=HRR(31,42,1)+FnSpK*VRR(31,17,0)
      HRR(31,18,1)=HRR(31,18,1)+VRR(31,18,0)
      HRR(31,43,1)=HRR(31,43,1)+FnSpK*VRR(31,18,0)
      HRR(31,19,1)=HRR(31,19,1)+VRR(31,19,0)
      HRR(31,44,1)=HRR(31,44,1)+FnSpK*VRR(31,19,0)
      HRR(31,20,1)=HRR(31,20,1)+VRR(31,20,0)
      HRR(31,45,1)=HRR(31,45,1)+FnSpK*VRR(31,20,0)
      HRR(31,21,1)=HRR(31,21,1)+VRR(31,21,0)
      HRR(31,22,1)=HRR(31,22,1)+VRR(31,22,0)
      HRR(31,23,1)=HRR(31,23,1)+VRR(31,23,0)
      HRR(31,24,1)=HRR(31,24,1)+VRR(31,24,0)
      HRR(31,25,1)=HRR(31,25,1)+VRR(31,25,0)
      HRR(31,26,1)=HRR(31,26,1)+VRR(31,26,0)
      HRR(31,27,1)=HRR(31,27,1)+VRR(31,27,0)
      HRR(31,28,1)=HRR(31,28,1)+VRR(31,28,0)
      HRR(31,29,1)=HRR(31,29,1)+VRR(31,29,0)
      HRR(31,30,1)=HRR(31,30,1)+VRR(31,30,0)
      HRR(31,31,1)=HRR(31,31,1)+VRR(31,31,0)
      HRR(31,32,1)=HRR(31,32,1)+VRR(31,32,0)
      HRR(31,33,1)=HRR(31,33,1)+VRR(31,33,0)
      HRR(31,34,1)=HRR(31,34,1)+VRR(31,34,0)
      HRR(31,35,1)=HRR(31,35,1)+VRR(31,35,0)
      HRR(32,1,1)=HRR(32,1,1)+VRR(32,1,0)
      HRR(32,2,1)=HRR(32,2,1)+VRR(32,2,0)
      HRR(32,3,1)=HRR(32,3,1)+VRR(32,3,0)
      HRR(32,4,1)=HRR(32,4,1)+VRR(32,4,0)
      HRR(32,5,1)=HRR(32,5,1)+VRR(32,5,0)
      HRR(32,6,1)=HRR(32,6,1)+VRR(32,6,0)
      HRR(32,7,1)=HRR(32,7,1)+VRR(32,7,0)
      HRR(32,8,1)=HRR(32,8,1)+VRR(32,8,0)
      HRR(32,9,1)=HRR(32,9,1)+VRR(32,9,0)
      HRR(32,10,1)=HRR(32,10,1)+VRR(32,10,0)
      HRR(32,11,1)=HRR(32,11,1)+VRR(32,11,0)
      HRR(32,36,1)=HRR(32,36,1)+FnSpK*VRR(32,11,0)
      HRR(32,12,1)=HRR(32,12,1)+VRR(32,12,0)
      HRR(32,37,1)=HRR(32,37,1)+FnSpK*VRR(32,12,0)
      HRR(32,13,1)=HRR(32,13,1)+VRR(32,13,0)
      HRR(32,38,1)=HRR(32,38,1)+FnSpK*VRR(32,13,0)
      HRR(32,14,1)=HRR(32,14,1)+VRR(32,14,0)
      HRR(32,39,1)=HRR(32,39,1)+FnSpK*VRR(32,14,0)
      HRR(32,15,1)=HRR(32,15,1)+VRR(32,15,0)
      HRR(32,40,1)=HRR(32,40,1)+FnSpK*VRR(32,15,0)
      HRR(32,16,1)=HRR(32,16,1)+VRR(32,16,0)
      HRR(32,41,1)=HRR(32,41,1)+FnSpK*VRR(32,16,0)
      HRR(32,17,1)=HRR(32,17,1)+VRR(32,17,0)
      HRR(32,42,1)=HRR(32,42,1)+FnSpK*VRR(32,17,0)
      HRR(32,18,1)=HRR(32,18,1)+VRR(32,18,0)
      HRR(32,43,1)=HRR(32,43,1)+FnSpK*VRR(32,18,0)
      HRR(32,19,1)=HRR(32,19,1)+VRR(32,19,0)
      HRR(32,44,1)=HRR(32,44,1)+FnSpK*VRR(32,19,0)
      HRR(32,20,1)=HRR(32,20,1)+VRR(32,20,0)
      HRR(32,45,1)=HRR(32,45,1)+FnSpK*VRR(32,20,0)
      HRR(32,21,1)=HRR(32,21,1)+VRR(32,21,0)
      HRR(32,22,1)=HRR(32,22,1)+VRR(32,22,0)
      HRR(32,23,1)=HRR(32,23,1)+VRR(32,23,0)
      HRR(32,24,1)=HRR(32,24,1)+VRR(32,24,0)
      HRR(32,25,1)=HRR(32,25,1)+VRR(32,25,0)
      HRR(32,26,1)=HRR(32,26,1)+VRR(32,26,0)
      HRR(32,27,1)=HRR(32,27,1)+VRR(32,27,0)
      HRR(32,28,1)=HRR(32,28,1)+VRR(32,28,0)
      HRR(32,29,1)=HRR(32,29,1)+VRR(32,29,0)
      HRR(32,30,1)=HRR(32,30,1)+VRR(32,30,0)
      HRR(32,31,1)=HRR(32,31,1)+VRR(32,31,0)
      HRR(32,32,1)=HRR(32,32,1)+VRR(32,32,0)
      HRR(32,33,1)=HRR(32,33,1)+VRR(32,33,0)
      HRR(32,34,1)=HRR(32,34,1)+VRR(32,34,0)
      HRR(32,35,1)=HRR(32,35,1)+VRR(32,35,0)
      HRR(33,1,1)=HRR(33,1,1)+VRR(33,1,0)
      HRR(33,2,1)=HRR(33,2,1)+VRR(33,2,0)
      HRR(33,3,1)=HRR(33,3,1)+VRR(33,3,0)
      HRR(33,4,1)=HRR(33,4,1)+VRR(33,4,0)
      HRR(33,5,1)=HRR(33,5,1)+VRR(33,5,0)
      HRR(33,6,1)=HRR(33,6,1)+VRR(33,6,0)
      HRR(33,7,1)=HRR(33,7,1)+VRR(33,7,0)
      HRR(33,8,1)=HRR(33,8,1)+VRR(33,8,0)
      HRR(33,9,1)=HRR(33,9,1)+VRR(33,9,0)
      HRR(33,10,1)=HRR(33,10,1)+VRR(33,10,0)
      HRR(33,11,1)=HRR(33,11,1)+VRR(33,11,0)
      HRR(33,36,1)=HRR(33,36,1)+FnSpK*VRR(33,11,0)
      HRR(33,12,1)=HRR(33,12,1)+VRR(33,12,0)
      HRR(33,37,1)=HRR(33,37,1)+FnSpK*VRR(33,12,0)
      HRR(33,13,1)=HRR(33,13,1)+VRR(33,13,0)
      HRR(33,38,1)=HRR(33,38,1)+FnSpK*VRR(33,13,0)
      HRR(33,14,1)=HRR(33,14,1)+VRR(33,14,0)
      HRR(33,39,1)=HRR(33,39,1)+FnSpK*VRR(33,14,0)
      HRR(33,15,1)=HRR(33,15,1)+VRR(33,15,0)
      HRR(33,40,1)=HRR(33,40,1)+FnSpK*VRR(33,15,0)
      HRR(33,16,1)=HRR(33,16,1)+VRR(33,16,0)
      HRR(33,41,1)=HRR(33,41,1)+FnSpK*VRR(33,16,0)
      HRR(33,17,1)=HRR(33,17,1)+VRR(33,17,0)
      HRR(33,42,1)=HRR(33,42,1)+FnSpK*VRR(33,17,0)
      HRR(33,18,1)=HRR(33,18,1)+VRR(33,18,0)
      HRR(33,43,1)=HRR(33,43,1)+FnSpK*VRR(33,18,0)
      HRR(33,19,1)=HRR(33,19,1)+VRR(33,19,0)
      HRR(33,44,1)=HRR(33,44,1)+FnSpK*VRR(33,19,0)
      HRR(33,20,1)=HRR(33,20,1)+VRR(33,20,0)
      HRR(33,45,1)=HRR(33,45,1)+FnSpK*VRR(33,20,0)
      HRR(33,21,1)=HRR(33,21,1)+VRR(33,21,0)
      HRR(33,22,1)=HRR(33,22,1)+VRR(33,22,0)
      HRR(33,23,1)=HRR(33,23,1)+VRR(33,23,0)
      HRR(33,24,1)=HRR(33,24,1)+VRR(33,24,0)
      HRR(33,25,1)=HRR(33,25,1)+VRR(33,25,0)
      HRR(33,26,1)=HRR(33,26,1)+VRR(33,26,0)
      HRR(33,27,1)=HRR(33,27,1)+VRR(33,27,0)
      HRR(33,28,1)=HRR(33,28,1)+VRR(33,28,0)
      HRR(33,29,1)=HRR(33,29,1)+VRR(33,29,0)
      HRR(33,30,1)=HRR(33,30,1)+VRR(33,30,0)
      HRR(33,31,1)=HRR(33,31,1)+VRR(33,31,0)
      HRR(33,32,1)=HRR(33,32,1)+VRR(33,32,0)
      HRR(33,33,1)=HRR(33,33,1)+VRR(33,33,0)
      HRR(33,34,1)=HRR(33,34,1)+VRR(33,34,0)
      HRR(33,35,1)=HRR(33,35,1)+VRR(33,35,0)
      HRR(34,1,1)=HRR(34,1,1)+VRR(34,1,0)
      HRR(34,2,1)=HRR(34,2,1)+VRR(34,2,0)
      HRR(34,3,1)=HRR(34,3,1)+VRR(34,3,0)
      HRR(34,4,1)=HRR(34,4,1)+VRR(34,4,0)
      HRR(34,5,1)=HRR(34,5,1)+VRR(34,5,0)
      HRR(34,6,1)=HRR(34,6,1)+VRR(34,6,0)
      HRR(34,7,1)=HRR(34,7,1)+VRR(34,7,0)
      HRR(34,8,1)=HRR(34,8,1)+VRR(34,8,0)
      HRR(34,9,1)=HRR(34,9,1)+VRR(34,9,0)
      HRR(34,10,1)=HRR(34,10,1)+VRR(34,10,0)
      HRR(34,11,1)=HRR(34,11,1)+VRR(34,11,0)
      HRR(34,36,1)=HRR(34,36,1)+FnSpK*VRR(34,11,0)
      HRR(34,12,1)=HRR(34,12,1)+VRR(34,12,0)
      HRR(34,37,1)=HRR(34,37,1)+FnSpK*VRR(34,12,0)
      HRR(34,13,1)=HRR(34,13,1)+VRR(34,13,0)
      HRR(34,38,1)=HRR(34,38,1)+FnSpK*VRR(34,13,0)
      HRR(34,14,1)=HRR(34,14,1)+VRR(34,14,0)
      HRR(34,39,1)=HRR(34,39,1)+FnSpK*VRR(34,14,0)
      HRR(34,15,1)=HRR(34,15,1)+VRR(34,15,0)
      HRR(34,40,1)=HRR(34,40,1)+FnSpK*VRR(34,15,0)
      HRR(34,16,1)=HRR(34,16,1)+VRR(34,16,0)
      HRR(34,41,1)=HRR(34,41,1)+FnSpK*VRR(34,16,0)
      HRR(34,17,1)=HRR(34,17,1)+VRR(34,17,0)
      HRR(34,42,1)=HRR(34,42,1)+FnSpK*VRR(34,17,0)
      HRR(34,18,1)=HRR(34,18,1)+VRR(34,18,0)
      HRR(34,43,1)=HRR(34,43,1)+FnSpK*VRR(34,18,0)
      HRR(34,19,1)=HRR(34,19,1)+VRR(34,19,0)
      HRR(34,44,1)=HRR(34,44,1)+FnSpK*VRR(34,19,0)
      HRR(34,20,1)=HRR(34,20,1)+VRR(34,20,0)
      HRR(34,45,1)=HRR(34,45,1)+FnSpK*VRR(34,20,0)
      HRR(34,21,1)=HRR(34,21,1)+VRR(34,21,0)
      HRR(34,22,1)=HRR(34,22,1)+VRR(34,22,0)
      HRR(34,23,1)=HRR(34,23,1)+VRR(34,23,0)
      HRR(34,24,1)=HRR(34,24,1)+VRR(34,24,0)
      HRR(34,25,1)=HRR(34,25,1)+VRR(34,25,0)
      HRR(34,26,1)=HRR(34,26,1)+VRR(34,26,0)
      HRR(34,27,1)=HRR(34,27,1)+VRR(34,27,0)
      HRR(34,28,1)=HRR(34,28,1)+VRR(34,28,0)
      HRR(34,29,1)=HRR(34,29,1)+VRR(34,29,0)
      HRR(34,30,1)=HRR(34,30,1)+VRR(34,30,0)
      HRR(34,31,1)=HRR(34,31,1)+VRR(34,31,0)
      HRR(34,32,1)=HRR(34,32,1)+VRR(34,32,0)
      HRR(34,33,1)=HRR(34,33,1)+VRR(34,33,0)
      HRR(34,34,1)=HRR(34,34,1)+VRR(34,34,0)
      HRR(34,35,1)=HRR(34,35,1)+VRR(34,35,0)
      HRR(35,1,1)=HRR(35,1,1)+VRR(35,1,0)
      HRR(35,2,1)=HRR(35,2,1)+VRR(35,2,0)
      HRR(35,3,1)=HRR(35,3,1)+VRR(35,3,0)
      HRR(35,4,1)=HRR(35,4,1)+VRR(35,4,0)
      HRR(35,5,1)=HRR(35,5,1)+VRR(35,5,0)
      HRR(35,6,1)=HRR(35,6,1)+VRR(35,6,0)
      HRR(35,7,1)=HRR(35,7,1)+VRR(35,7,0)
      HRR(35,8,1)=HRR(35,8,1)+VRR(35,8,0)
      HRR(35,9,1)=HRR(35,9,1)+VRR(35,9,0)
      HRR(35,10,1)=HRR(35,10,1)+VRR(35,10,0)
      HRR(35,11,1)=HRR(35,11,1)+VRR(35,11,0)
      HRR(35,36,1)=HRR(35,36,1)+FnSpK*VRR(35,11,0)
      HRR(35,12,1)=HRR(35,12,1)+VRR(35,12,0)
      HRR(35,37,1)=HRR(35,37,1)+FnSpK*VRR(35,12,0)
      HRR(35,13,1)=HRR(35,13,1)+VRR(35,13,0)
      HRR(35,38,1)=HRR(35,38,1)+FnSpK*VRR(35,13,0)
      HRR(35,14,1)=HRR(35,14,1)+VRR(35,14,0)
      HRR(35,39,1)=HRR(35,39,1)+FnSpK*VRR(35,14,0)
      HRR(35,15,1)=HRR(35,15,1)+VRR(35,15,0)
      HRR(35,40,1)=HRR(35,40,1)+FnSpK*VRR(35,15,0)
      HRR(35,16,1)=HRR(35,16,1)+VRR(35,16,0)
      HRR(35,41,1)=HRR(35,41,1)+FnSpK*VRR(35,16,0)
      HRR(35,17,1)=HRR(35,17,1)+VRR(35,17,0)
      HRR(35,42,1)=HRR(35,42,1)+FnSpK*VRR(35,17,0)
      HRR(35,18,1)=HRR(35,18,1)+VRR(35,18,0)
      HRR(35,43,1)=HRR(35,43,1)+FnSpK*VRR(35,18,0)
      HRR(35,19,1)=HRR(35,19,1)+VRR(35,19,0)
      HRR(35,44,1)=HRR(35,44,1)+FnSpK*VRR(35,19,0)
      HRR(35,20,1)=HRR(35,20,1)+VRR(35,20,0)
      HRR(35,45,1)=HRR(35,45,1)+FnSpK*VRR(35,20,0)
      HRR(35,21,1)=HRR(35,21,1)+VRR(35,21,0)
      HRR(35,22,1)=HRR(35,22,1)+VRR(35,22,0)
      HRR(35,23,1)=HRR(35,23,1)+VRR(35,23,0)
      HRR(35,24,1)=HRR(35,24,1)+VRR(35,24,0)
      HRR(35,25,1)=HRR(35,25,1)+VRR(35,25,0)
      HRR(35,26,1)=HRR(35,26,1)+VRR(35,26,0)
      HRR(35,27,1)=HRR(35,27,1)+VRR(35,27,0)
      HRR(35,28,1)=HRR(35,28,1)+VRR(35,28,0)
      HRR(35,29,1)=HRR(35,29,1)+VRR(35,29,0)
      HRR(35,30,1)=HRR(35,30,1)+VRR(35,30,0)
      HRR(35,31,1)=HRR(35,31,1)+VRR(35,31,0)
      HRR(35,32,1)=HRR(35,32,1)+VRR(35,32,0)
      HRR(35,33,1)=HRR(35,33,1)+VRR(35,33,0)
      HRR(35,34,1)=HRR(35,34,1)+VRR(35,34,0)
      HRR(35,35,1)=HRR(35,35,1)+VRR(35,35,0)
      HRR(36,1,1)=HRR(36,1,1)+VRR(36,1,0)
      HRR(36,2,1)=HRR(36,2,1)+VRR(36,2,0)
      HRR(36,3,1)=HRR(36,3,1)+VRR(36,3,0)
      HRR(36,4,1)=HRR(36,4,1)+VRR(36,4,0)
      HRR(36,5,1)=HRR(36,5,1)+VRR(36,5,0)
      HRR(36,6,1)=HRR(36,6,1)+VRR(36,6,0)
      HRR(36,7,1)=HRR(36,7,1)+VRR(36,7,0)
      HRR(36,8,1)=HRR(36,8,1)+VRR(36,8,0)
      HRR(36,9,1)=HRR(36,9,1)+VRR(36,9,0)
      HRR(36,10,1)=HRR(36,10,1)+VRR(36,10,0)
      HRR(36,11,1)=HRR(36,11,1)+VRR(36,11,0)
      HRR(36,36,1)=HRR(36,36,1)+FnSpK*VRR(36,11,0)
      HRR(36,12,1)=HRR(36,12,1)+VRR(36,12,0)
      HRR(36,37,1)=HRR(36,37,1)+FnSpK*VRR(36,12,0)
      HRR(36,13,1)=HRR(36,13,1)+VRR(36,13,0)
      HRR(36,38,1)=HRR(36,38,1)+FnSpK*VRR(36,13,0)
      HRR(36,14,1)=HRR(36,14,1)+VRR(36,14,0)
      HRR(36,39,1)=HRR(36,39,1)+FnSpK*VRR(36,14,0)
      HRR(36,15,1)=HRR(36,15,1)+VRR(36,15,0)
      HRR(36,40,1)=HRR(36,40,1)+FnSpK*VRR(36,15,0)
      HRR(36,16,1)=HRR(36,16,1)+VRR(36,16,0)
      HRR(36,41,1)=HRR(36,41,1)+FnSpK*VRR(36,16,0)
      HRR(36,17,1)=HRR(36,17,1)+VRR(36,17,0)
      HRR(36,42,1)=HRR(36,42,1)+FnSpK*VRR(36,17,0)
      HRR(36,18,1)=HRR(36,18,1)+VRR(36,18,0)
      HRR(36,43,1)=HRR(36,43,1)+FnSpK*VRR(36,18,0)
      HRR(36,19,1)=HRR(36,19,1)+VRR(36,19,0)
      HRR(36,44,1)=HRR(36,44,1)+FnSpK*VRR(36,19,0)
      HRR(36,20,1)=HRR(36,20,1)+VRR(36,20,0)
      HRR(36,45,1)=HRR(36,45,1)+FnSpK*VRR(36,20,0)
      HRR(36,21,1)=HRR(36,21,1)+VRR(36,21,0)
      HRR(36,22,1)=HRR(36,22,1)+VRR(36,22,0)
      HRR(36,23,1)=HRR(36,23,1)+VRR(36,23,0)
      HRR(36,24,1)=HRR(36,24,1)+VRR(36,24,0)
      HRR(36,25,1)=HRR(36,25,1)+VRR(36,25,0)
      HRR(36,26,1)=HRR(36,26,1)+VRR(36,26,0)
      HRR(36,27,1)=HRR(36,27,1)+VRR(36,27,0)
      HRR(36,28,1)=HRR(36,28,1)+VRR(36,28,0)
      HRR(36,29,1)=HRR(36,29,1)+VRR(36,29,0)
      HRR(36,30,1)=HRR(36,30,1)+VRR(36,30,0)
      HRR(36,31,1)=HRR(36,31,1)+VRR(36,31,0)
      HRR(36,32,1)=HRR(36,32,1)+VRR(36,32,0)
      HRR(36,33,1)=HRR(36,33,1)+VRR(36,33,0)
      HRR(36,34,1)=HRR(36,34,1)+VRR(36,34,0)
      HRR(36,35,1)=HRR(36,35,1)+VRR(36,35,0)
      HRR(37,1,1)=HRR(37,1,1)+VRR(37,1,0)
      HRR(37,2,1)=HRR(37,2,1)+VRR(37,2,0)
      HRR(37,3,1)=HRR(37,3,1)+VRR(37,3,0)
      HRR(37,4,1)=HRR(37,4,1)+VRR(37,4,0)
      HRR(37,5,1)=HRR(37,5,1)+VRR(37,5,0)
      HRR(37,6,1)=HRR(37,6,1)+VRR(37,6,0)
      HRR(37,7,1)=HRR(37,7,1)+VRR(37,7,0)
      HRR(37,8,1)=HRR(37,8,1)+VRR(37,8,0)
      HRR(37,9,1)=HRR(37,9,1)+VRR(37,9,0)
      HRR(37,10,1)=HRR(37,10,1)+VRR(37,10,0)
      HRR(37,11,1)=HRR(37,11,1)+VRR(37,11,0)
      HRR(37,36,1)=HRR(37,36,1)+FnSpK*VRR(37,11,0)
      HRR(37,12,1)=HRR(37,12,1)+VRR(37,12,0)
      HRR(37,37,1)=HRR(37,37,1)+FnSpK*VRR(37,12,0)
      HRR(37,13,1)=HRR(37,13,1)+VRR(37,13,0)
      HRR(37,38,1)=HRR(37,38,1)+FnSpK*VRR(37,13,0)
      HRR(37,14,1)=HRR(37,14,1)+VRR(37,14,0)
      HRR(37,39,1)=HRR(37,39,1)+FnSpK*VRR(37,14,0)
      HRR(37,15,1)=HRR(37,15,1)+VRR(37,15,0)
      HRR(37,40,1)=HRR(37,40,1)+FnSpK*VRR(37,15,0)
      HRR(37,16,1)=HRR(37,16,1)+VRR(37,16,0)
      HRR(37,41,1)=HRR(37,41,1)+FnSpK*VRR(37,16,0)
      HRR(37,17,1)=HRR(37,17,1)+VRR(37,17,0)
      HRR(37,42,1)=HRR(37,42,1)+FnSpK*VRR(37,17,0)
      HRR(37,18,1)=HRR(37,18,1)+VRR(37,18,0)
      HRR(37,43,1)=HRR(37,43,1)+FnSpK*VRR(37,18,0)
      HRR(37,19,1)=HRR(37,19,1)+VRR(37,19,0)
      HRR(37,44,1)=HRR(37,44,1)+FnSpK*VRR(37,19,0)
      HRR(37,20,1)=HRR(37,20,1)+VRR(37,20,0)
      HRR(37,45,1)=HRR(37,45,1)+FnSpK*VRR(37,20,0)
      HRR(37,21,1)=HRR(37,21,1)+VRR(37,21,0)
      HRR(37,22,1)=HRR(37,22,1)+VRR(37,22,0)
      HRR(37,23,1)=HRR(37,23,1)+VRR(37,23,0)
      HRR(37,24,1)=HRR(37,24,1)+VRR(37,24,0)
      HRR(37,25,1)=HRR(37,25,1)+VRR(37,25,0)
      HRR(37,26,1)=HRR(37,26,1)+VRR(37,26,0)
      HRR(37,27,1)=HRR(37,27,1)+VRR(37,27,0)
      HRR(37,28,1)=HRR(37,28,1)+VRR(37,28,0)
      HRR(37,29,1)=HRR(37,29,1)+VRR(37,29,0)
      HRR(37,30,1)=HRR(37,30,1)+VRR(37,30,0)
      HRR(37,31,1)=HRR(37,31,1)+VRR(37,31,0)
      HRR(37,32,1)=HRR(37,32,1)+VRR(37,32,0)
      HRR(37,33,1)=HRR(37,33,1)+VRR(37,33,0)
      HRR(37,34,1)=HRR(37,34,1)+VRR(37,34,0)
      HRR(37,35,1)=HRR(37,35,1)+VRR(37,35,0)
      HRR(38,1,1)=HRR(38,1,1)+VRR(38,1,0)
      HRR(38,2,1)=HRR(38,2,1)+VRR(38,2,0)
      HRR(38,3,1)=HRR(38,3,1)+VRR(38,3,0)
      HRR(38,4,1)=HRR(38,4,1)+VRR(38,4,0)
      HRR(38,5,1)=HRR(38,5,1)+VRR(38,5,0)
      HRR(38,6,1)=HRR(38,6,1)+VRR(38,6,0)
      HRR(38,7,1)=HRR(38,7,1)+VRR(38,7,0)
      HRR(38,8,1)=HRR(38,8,1)+VRR(38,8,0)
      HRR(38,9,1)=HRR(38,9,1)+VRR(38,9,0)
      HRR(38,10,1)=HRR(38,10,1)+VRR(38,10,0)
      HRR(38,11,1)=HRR(38,11,1)+VRR(38,11,0)
      HRR(38,36,1)=HRR(38,36,1)+FnSpK*VRR(38,11,0)
      HRR(38,12,1)=HRR(38,12,1)+VRR(38,12,0)
      HRR(38,37,1)=HRR(38,37,1)+FnSpK*VRR(38,12,0)
      HRR(38,13,1)=HRR(38,13,1)+VRR(38,13,0)
      HRR(38,38,1)=HRR(38,38,1)+FnSpK*VRR(38,13,0)
      HRR(38,14,1)=HRR(38,14,1)+VRR(38,14,0)
      HRR(38,39,1)=HRR(38,39,1)+FnSpK*VRR(38,14,0)
      HRR(38,15,1)=HRR(38,15,1)+VRR(38,15,0)
      HRR(38,40,1)=HRR(38,40,1)+FnSpK*VRR(38,15,0)
      HRR(38,16,1)=HRR(38,16,1)+VRR(38,16,0)
      HRR(38,41,1)=HRR(38,41,1)+FnSpK*VRR(38,16,0)
      HRR(38,17,1)=HRR(38,17,1)+VRR(38,17,0)
      HRR(38,42,1)=HRR(38,42,1)+FnSpK*VRR(38,17,0)
      HRR(38,18,1)=HRR(38,18,1)+VRR(38,18,0)
      HRR(38,43,1)=HRR(38,43,1)+FnSpK*VRR(38,18,0)
      HRR(38,19,1)=HRR(38,19,1)+VRR(38,19,0)
      HRR(38,44,1)=HRR(38,44,1)+FnSpK*VRR(38,19,0)
      HRR(38,20,1)=HRR(38,20,1)+VRR(38,20,0)
      HRR(38,45,1)=HRR(38,45,1)+FnSpK*VRR(38,20,0)
      HRR(38,21,1)=HRR(38,21,1)+VRR(38,21,0)
      HRR(38,22,1)=HRR(38,22,1)+VRR(38,22,0)
      HRR(38,23,1)=HRR(38,23,1)+VRR(38,23,0)
      HRR(38,24,1)=HRR(38,24,1)+VRR(38,24,0)
      HRR(38,25,1)=HRR(38,25,1)+VRR(38,25,0)
      HRR(38,26,1)=HRR(38,26,1)+VRR(38,26,0)
      HRR(38,27,1)=HRR(38,27,1)+VRR(38,27,0)
      HRR(38,28,1)=HRR(38,28,1)+VRR(38,28,0)
      HRR(38,29,1)=HRR(38,29,1)+VRR(38,29,0)
      HRR(38,30,1)=HRR(38,30,1)+VRR(38,30,0)
      HRR(38,31,1)=HRR(38,31,1)+VRR(38,31,0)
      HRR(38,32,1)=HRR(38,32,1)+VRR(38,32,0)
      HRR(38,33,1)=HRR(38,33,1)+VRR(38,33,0)
      HRR(38,34,1)=HRR(38,34,1)+VRR(38,34,0)
      HRR(38,35,1)=HRR(38,35,1)+VRR(38,35,0)
      HRR(39,1,1)=HRR(39,1,1)+VRR(39,1,0)
      HRR(39,2,1)=HRR(39,2,1)+VRR(39,2,0)
      HRR(39,3,1)=HRR(39,3,1)+VRR(39,3,0)
      HRR(39,4,1)=HRR(39,4,1)+VRR(39,4,0)
      HRR(39,5,1)=HRR(39,5,1)+VRR(39,5,0)
      HRR(39,6,1)=HRR(39,6,1)+VRR(39,6,0)
      HRR(39,7,1)=HRR(39,7,1)+VRR(39,7,0)
      HRR(39,8,1)=HRR(39,8,1)+VRR(39,8,0)
      HRR(39,9,1)=HRR(39,9,1)+VRR(39,9,0)
      HRR(39,10,1)=HRR(39,10,1)+VRR(39,10,0)
      HRR(39,11,1)=HRR(39,11,1)+VRR(39,11,0)
      HRR(39,36,1)=HRR(39,36,1)+FnSpK*VRR(39,11,0)
      HRR(39,12,1)=HRR(39,12,1)+VRR(39,12,0)
      HRR(39,37,1)=HRR(39,37,1)+FnSpK*VRR(39,12,0)
      HRR(39,13,1)=HRR(39,13,1)+VRR(39,13,0)
      HRR(39,38,1)=HRR(39,38,1)+FnSpK*VRR(39,13,0)
      HRR(39,14,1)=HRR(39,14,1)+VRR(39,14,0)
      HRR(39,39,1)=HRR(39,39,1)+FnSpK*VRR(39,14,0)
      HRR(39,15,1)=HRR(39,15,1)+VRR(39,15,0)
      HRR(39,40,1)=HRR(39,40,1)+FnSpK*VRR(39,15,0)
      HRR(39,16,1)=HRR(39,16,1)+VRR(39,16,0)
      HRR(39,41,1)=HRR(39,41,1)+FnSpK*VRR(39,16,0)
      HRR(39,17,1)=HRR(39,17,1)+VRR(39,17,0)
      HRR(39,42,1)=HRR(39,42,1)+FnSpK*VRR(39,17,0)
      HRR(39,18,1)=HRR(39,18,1)+VRR(39,18,0)
      HRR(39,43,1)=HRR(39,43,1)+FnSpK*VRR(39,18,0)
      HRR(39,19,1)=HRR(39,19,1)+VRR(39,19,0)
      HRR(39,44,1)=HRR(39,44,1)+FnSpK*VRR(39,19,0)
      HRR(39,20,1)=HRR(39,20,1)+VRR(39,20,0)
      HRR(39,45,1)=HRR(39,45,1)+FnSpK*VRR(39,20,0)
      HRR(39,21,1)=HRR(39,21,1)+VRR(39,21,0)
      HRR(39,22,1)=HRR(39,22,1)+VRR(39,22,0)
      HRR(39,23,1)=HRR(39,23,1)+VRR(39,23,0)
      HRR(39,24,1)=HRR(39,24,1)+VRR(39,24,0)
      HRR(39,25,1)=HRR(39,25,1)+VRR(39,25,0)
      HRR(39,26,1)=HRR(39,26,1)+VRR(39,26,0)
      HRR(39,27,1)=HRR(39,27,1)+VRR(39,27,0)
      HRR(39,28,1)=HRR(39,28,1)+VRR(39,28,0)
      HRR(39,29,1)=HRR(39,29,1)+VRR(39,29,0)
      HRR(39,30,1)=HRR(39,30,1)+VRR(39,30,0)
      HRR(39,31,1)=HRR(39,31,1)+VRR(39,31,0)
      HRR(39,32,1)=HRR(39,32,1)+VRR(39,32,0)
      HRR(39,33,1)=HRR(39,33,1)+VRR(39,33,0)
      HRR(39,34,1)=HRR(39,34,1)+VRR(39,34,0)
      HRR(39,35,1)=HRR(39,35,1)+VRR(39,35,0)
      HRR(40,1,1)=HRR(40,1,1)+VRR(40,1,0)
      HRR(40,2,1)=HRR(40,2,1)+VRR(40,2,0)
      HRR(40,3,1)=HRR(40,3,1)+VRR(40,3,0)
      HRR(40,4,1)=HRR(40,4,1)+VRR(40,4,0)
      HRR(40,5,1)=HRR(40,5,1)+VRR(40,5,0)
      HRR(40,6,1)=HRR(40,6,1)+VRR(40,6,0)
      HRR(40,7,1)=HRR(40,7,1)+VRR(40,7,0)
      HRR(40,8,1)=HRR(40,8,1)+VRR(40,8,0)
      HRR(40,9,1)=HRR(40,9,1)+VRR(40,9,0)
      HRR(40,10,1)=HRR(40,10,1)+VRR(40,10,0)
      HRR(40,11,1)=HRR(40,11,1)+VRR(40,11,0)
      HRR(40,36,1)=HRR(40,36,1)+FnSpK*VRR(40,11,0)
      HRR(40,12,1)=HRR(40,12,1)+VRR(40,12,0)
      HRR(40,37,1)=HRR(40,37,1)+FnSpK*VRR(40,12,0)
      HRR(40,13,1)=HRR(40,13,1)+VRR(40,13,0)
      HRR(40,38,1)=HRR(40,38,1)+FnSpK*VRR(40,13,0)
      HRR(40,14,1)=HRR(40,14,1)+VRR(40,14,0)
      HRR(40,39,1)=HRR(40,39,1)+FnSpK*VRR(40,14,0)
      HRR(40,15,1)=HRR(40,15,1)+VRR(40,15,0)
      HRR(40,40,1)=HRR(40,40,1)+FnSpK*VRR(40,15,0)
      HRR(40,16,1)=HRR(40,16,1)+VRR(40,16,0)
      HRR(40,41,1)=HRR(40,41,1)+FnSpK*VRR(40,16,0)
      HRR(40,17,1)=HRR(40,17,1)+VRR(40,17,0)
      HRR(40,42,1)=HRR(40,42,1)+FnSpK*VRR(40,17,0)
      HRR(40,18,1)=HRR(40,18,1)+VRR(40,18,0)
      HRR(40,43,1)=HRR(40,43,1)+FnSpK*VRR(40,18,0)
      HRR(40,19,1)=HRR(40,19,1)+VRR(40,19,0)
      HRR(40,44,1)=HRR(40,44,1)+FnSpK*VRR(40,19,0)
      HRR(40,20,1)=HRR(40,20,1)+VRR(40,20,0)
      HRR(40,45,1)=HRR(40,45,1)+FnSpK*VRR(40,20,0)
      HRR(40,21,1)=HRR(40,21,1)+VRR(40,21,0)
      HRR(40,22,1)=HRR(40,22,1)+VRR(40,22,0)
      HRR(40,23,1)=HRR(40,23,1)+VRR(40,23,0)
      HRR(40,24,1)=HRR(40,24,1)+VRR(40,24,0)
      HRR(40,25,1)=HRR(40,25,1)+VRR(40,25,0)
      HRR(40,26,1)=HRR(40,26,1)+VRR(40,26,0)
      HRR(40,27,1)=HRR(40,27,1)+VRR(40,27,0)
      HRR(40,28,1)=HRR(40,28,1)+VRR(40,28,0)
      HRR(40,29,1)=HRR(40,29,1)+VRR(40,29,0)
      HRR(40,30,1)=HRR(40,30,1)+VRR(40,30,0)
      HRR(40,31,1)=HRR(40,31,1)+VRR(40,31,0)
      HRR(40,32,1)=HRR(40,32,1)+VRR(40,32,0)
      HRR(40,33,1)=HRR(40,33,1)+VRR(40,33,0)
      HRR(40,34,1)=HRR(40,34,1)+VRR(40,34,0)
      HRR(40,35,1)=HRR(40,35,1)+VRR(40,35,0)
      HRR(41,1,1)=HRR(41,1,1)+VRR(41,1,0)
      HRR(41,2,1)=HRR(41,2,1)+VRR(41,2,0)
      HRR(41,3,1)=HRR(41,3,1)+VRR(41,3,0)
      HRR(41,4,1)=HRR(41,4,1)+VRR(41,4,0)
      HRR(41,5,1)=HRR(41,5,1)+VRR(41,5,0)
      HRR(41,6,1)=HRR(41,6,1)+VRR(41,6,0)
      HRR(41,7,1)=HRR(41,7,1)+VRR(41,7,0)
      HRR(41,8,1)=HRR(41,8,1)+VRR(41,8,0)
      HRR(41,9,1)=HRR(41,9,1)+VRR(41,9,0)
      HRR(41,10,1)=HRR(41,10,1)+VRR(41,10,0)
      HRR(41,11,1)=HRR(41,11,1)+VRR(41,11,0)
      HRR(41,36,1)=HRR(41,36,1)+FnSpK*VRR(41,11,0)
      HRR(41,12,1)=HRR(41,12,1)+VRR(41,12,0)
      HRR(41,37,1)=HRR(41,37,1)+FnSpK*VRR(41,12,0)
      HRR(41,13,1)=HRR(41,13,1)+VRR(41,13,0)
      HRR(41,38,1)=HRR(41,38,1)+FnSpK*VRR(41,13,0)
      HRR(41,14,1)=HRR(41,14,1)+VRR(41,14,0)
      HRR(41,39,1)=HRR(41,39,1)+FnSpK*VRR(41,14,0)
      HRR(41,15,1)=HRR(41,15,1)+VRR(41,15,0)
      HRR(41,40,1)=HRR(41,40,1)+FnSpK*VRR(41,15,0)
      HRR(41,16,1)=HRR(41,16,1)+VRR(41,16,0)
      HRR(41,41,1)=HRR(41,41,1)+FnSpK*VRR(41,16,0)
      HRR(41,17,1)=HRR(41,17,1)+VRR(41,17,0)
      HRR(41,42,1)=HRR(41,42,1)+FnSpK*VRR(41,17,0)
      HRR(41,18,1)=HRR(41,18,1)+VRR(41,18,0)
      HRR(41,43,1)=HRR(41,43,1)+FnSpK*VRR(41,18,0)
      HRR(41,19,1)=HRR(41,19,1)+VRR(41,19,0)
      HRR(41,44,1)=HRR(41,44,1)+FnSpK*VRR(41,19,0)
      HRR(41,20,1)=HRR(41,20,1)+VRR(41,20,0)
      HRR(41,45,1)=HRR(41,45,1)+FnSpK*VRR(41,20,0)
      HRR(41,21,1)=HRR(41,21,1)+VRR(41,21,0)
      HRR(41,22,1)=HRR(41,22,1)+VRR(41,22,0)
      HRR(41,23,1)=HRR(41,23,1)+VRR(41,23,0)
      HRR(41,24,1)=HRR(41,24,1)+VRR(41,24,0)
      HRR(41,25,1)=HRR(41,25,1)+VRR(41,25,0)
      HRR(41,26,1)=HRR(41,26,1)+VRR(41,26,0)
      HRR(41,27,1)=HRR(41,27,1)+VRR(41,27,0)
      HRR(41,28,1)=HRR(41,28,1)+VRR(41,28,0)
      HRR(41,29,1)=HRR(41,29,1)+VRR(41,29,0)
      HRR(41,30,1)=HRR(41,30,1)+VRR(41,30,0)
      HRR(41,31,1)=HRR(41,31,1)+VRR(41,31,0)
      HRR(41,32,1)=HRR(41,32,1)+VRR(41,32,0)
      HRR(41,33,1)=HRR(41,33,1)+VRR(41,33,0)
      HRR(41,34,1)=HRR(41,34,1)+VRR(41,34,0)
      HRR(41,35,1)=HRR(41,35,1)+VRR(41,35,0)
      HRR(42,1,1)=HRR(42,1,1)+VRR(42,1,0)
      HRR(42,2,1)=HRR(42,2,1)+VRR(42,2,0)
      HRR(42,3,1)=HRR(42,3,1)+VRR(42,3,0)
      HRR(42,4,1)=HRR(42,4,1)+VRR(42,4,0)
      HRR(42,5,1)=HRR(42,5,1)+VRR(42,5,0)
      HRR(42,6,1)=HRR(42,6,1)+VRR(42,6,0)
      HRR(42,7,1)=HRR(42,7,1)+VRR(42,7,0)
      HRR(42,8,1)=HRR(42,8,1)+VRR(42,8,0)
      HRR(42,9,1)=HRR(42,9,1)+VRR(42,9,0)
      HRR(42,10,1)=HRR(42,10,1)+VRR(42,10,0)
      HRR(42,11,1)=HRR(42,11,1)+VRR(42,11,0)
      HRR(42,36,1)=HRR(42,36,1)+FnSpK*VRR(42,11,0)
      HRR(42,12,1)=HRR(42,12,1)+VRR(42,12,0)
      HRR(42,37,1)=HRR(42,37,1)+FnSpK*VRR(42,12,0)
      HRR(42,13,1)=HRR(42,13,1)+VRR(42,13,0)
      HRR(42,38,1)=HRR(42,38,1)+FnSpK*VRR(42,13,0)
      HRR(42,14,1)=HRR(42,14,1)+VRR(42,14,0)
      HRR(42,39,1)=HRR(42,39,1)+FnSpK*VRR(42,14,0)
      HRR(42,15,1)=HRR(42,15,1)+VRR(42,15,0)
      HRR(42,40,1)=HRR(42,40,1)+FnSpK*VRR(42,15,0)
      HRR(42,16,1)=HRR(42,16,1)+VRR(42,16,0)
      HRR(42,41,1)=HRR(42,41,1)+FnSpK*VRR(42,16,0)
      HRR(42,17,1)=HRR(42,17,1)+VRR(42,17,0)
      HRR(42,42,1)=HRR(42,42,1)+FnSpK*VRR(42,17,0)
      HRR(42,18,1)=HRR(42,18,1)+VRR(42,18,0)
      HRR(42,43,1)=HRR(42,43,1)+FnSpK*VRR(42,18,0)
      HRR(42,19,1)=HRR(42,19,1)+VRR(42,19,0)
      HRR(42,44,1)=HRR(42,44,1)+FnSpK*VRR(42,19,0)
      HRR(42,20,1)=HRR(42,20,1)+VRR(42,20,0)
      HRR(42,45,1)=HRR(42,45,1)+FnSpK*VRR(42,20,0)
      HRR(42,21,1)=HRR(42,21,1)+VRR(42,21,0)
      HRR(42,22,1)=HRR(42,22,1)+VRR(42,22,0)
      HRR(42,23,1)=HRR(42,23,1)+VRR(42,23,0)
      HRR(42,24,1)=HRR(42,24,1)+VRR(42,24,0)
      HRR(42,25,1)=HRR(42,25,1)+VRR(42,25,0)
      HRR(42,26,1)=HRR(42,26,1)+VRR(42,26,0)
      HRR(42,27,1)=HRR(42,27,1)+VRR(42,27,0)
      HRR(42,28,1)=HRR(42,28,1)+VRR(42,28,0)
      HRR(42,29,1)=HRR(42,29,1)+VRR(42,29,0)
      HRR(42,30,1)=HRR(42,30,1)+VRR(42,30,0)
      HRR(42,31,1)=HRR(42,31,1)+VRR(42,31,0)
      HRR(42,32,1)=HRR(42,32,1)+VRR(42,32,0)
      HRR(42,33,1)=HRR(42,33,1)+VRR(42,33,0)
      HRR(42,34,1)=HRR(42,34,1)+VRR(42,34,0)
      HRR(42,35,1)=HRR(42,35,1)+VRR(42,35,0)
      HRR(43,1,1)=HRR(43,1,1)+VRR(43,1,0)
      HRR(43,2,1)=HRR(43,2,1)+VRR(43,2,0)
      HRR(43,3,1)=HRR(43,3,1)+VRR(43,3,0)
      HRR(43,4,1)=HRR(43,4,1)+VRR(43,4,0)
      HRR(43,5,1)=HRR(43,5,1)+VRR(43,5,0)
      HRR(43,6,1)=HRR(43,6,1)+VRR(43,6,0)
      HRR(43,7,1)=HRR(43,7,1)+VRR(43,7,0)
      HRR(43,8,1)=HRR(43,8,1)+VRR(43,8,0)
      HRR(43,9,1)=HRR(43,9,1)+VRR(43,9,0)
      HRR(43,10,1)=HRR(43,10,1)+VRR(43,10,0)
      HRR(43,11,1)=HRR(43,11,1)+VRR(43,11,0)
      HRR(43,36,1)=HRR(43,36,1)+FnSpK*VRR(43,11,0)
      HRR(43,12,1)=HRR(43,12,1)+VRR(43,12,0)
      HRR(43,37,1)=HRR(43,37,1)+FnSpK*VRR(43,12,0)
      HRR(43,13,1)=HRR(43,13,1)+VRR(43,13,0)
      HRR(43,38,1)=HRR(43,38,1)+FnSpK*VRR(43,13,0)
      HRR(43,14,1)=HRR(43,14,1)+VRR(43,14,0)
      HRR(43,39,1)=HRR(43,39,1)+FnSpK*VRR(43,14,0)
      HRR(43,15,1)=HRR(43,15,1)+VRR(43,15,0)
      HRR(43,40,1)=HRR(43,40,1)+FnSpK*VRR(43,15,0)
      HRR(43,16,1)=HRR(43,16,1)+VRR(43,16,0)
      HRR(43,41,1)=HRR(43,41,1)+FnSpK*VRR(43,16,0)
      HRR(43,17,1)=HRR(43,17,1)+VRR(43,17,0)
      HRR(43,42,1)=HRR(43,42,1)+FnSpK*VRR(43,17,0)
      HRR(43,18,1)=HRR(43,18,1)+VRR(43,18,0)
      HRR(43,43,1)=HRR(43,43,1)+FnSpK*VRR(43,18,0)
      HRR(43,19,1)=HRR(43,19,1)+VRR(43,19,0)
      HRR(43,44,1)=HRR(43,44,1)+FnSpK*VRR(43,19,0)
      HRR(43,20,1)=HRR(43,20,1)+VRR(43,20,0)
      HRR(43,45,1)=HRR(43,45,1)+FnSpK*VRR(43,20,0)
      HRR(43,21,1)=HRR(43,21,1)+VRR(43,21,0)
      HRR(43,22,1)=HRR(43,22,1)+VRR(43,22,0)
      HRR(43,23,1)=HRR(43,23,1)+VRR(43,23,0)
      HRR(43,24,1)=HRR(43,24,1)+VRR(43,24,0)
      HRR(43,25,1)=HRR(43,25,1)+VRR(43,25,0)
      HRR(43,26,1)=HRR(43,26,1)+VRR(43,26,0)
      HRR(43,27,1)=HRR(43,27,1)+VRR(43,27,0)
      HRR(43,28,1)=HRR(43,28,1)+VRR(43,28,0)
      HRR(43,29,1)=HRR(43,29,1)+VRR(43,29,0)
      HRR(43,30,1)=HRR(43,30,1)+VRR(43,30,0)
      HRR(43,31,1)=HRR(43,31,1)+VRR(43,31,0)
      HRR(43,32,1)=HRR(43,32,1)+VRR(43,32,0)
      HRR(43,33,1)=HRR(43,33,1)+VRR(43,33,0)
      HRR(43,34,1)=HRR(43,34,1)+VRR(43,34,0)
      HRR(43,35,1)=HRR(43,35,1)+VRR(43,35,0)
      HRR(44,1,1)=HRR(44,1,1)+VRR(44,1,0)
      HRR(44,2,1)=HRR(44,2,1)+VRR(44,2,0)
      HRR(44,3,1)=HRR(44,3,1)+VRR(44,3,0)
      HRR(44,4,1)=HRR(44,4,1)+VRR(44,4,0)
      HRR(44,5,1)=HRR(44,5,1)+VRR(44,5,0)
      HRR(44,6,1)=HRR(44,6,1)+VRR(44,6,0)
      HRR(44,7,1)=HRR(44,7,1)+VRR(44,7,0)
      HRR(44,8,1)=HRR(44,8,1)+VRR(44,8,0)
      HRR(44,9,1)=HRR(44,9,1)+VRR(44,9,0)
      HRR(44,10,1)=HRR(44,10,1)+VRR(44,10,0)
      HRR(44,11,1)=HRR(44,11,1)+VRR(44,11,0)
      HRR(44,36,1)=HRR(44,36,1)+FnSpK*VRR(44,11,0)
      HRR(44,12,1)=HRR(44,12,1)+VRR(44,12,0)
      HRR(44,37,1)=HRR(44,37,1)+FnSpK*VRR(44,12,0)
      HRR(44,13,1)=HRR(44,13,1)+VRR(44,13,0)
      HRR(44,38,1)=HRR(44,38,1)+FnSpK*VRR(44,13,0)
      HRR(44,14,1)=HRR(44,14,1)+VRR(44,14,0)
      HRR(44,39,1)=HRR(44,39,1)+FnSpK*VRR(44,14,0)
      HRR(44,15,1)=HRR(44,15,1)+VRR(44,15,0)
      HRR(44,40,1)=HRR(44,40,1)+FnSpK*VRR(44,15,0)
      HRR(44,16,1)=HRR(44,16,1)+VRR(44,16,0)
      HRR(44,41,1)=HRR(44,41,1)+FnSpK*VRR(44,16,0)
      HRR(44,17,1)=HRR(44,17,1)+VRR(44,17,0)
      HRR(44,42,1)=HRR(44,42,1)+FnSpK*VRR(44,17,0)
      HRR(44,18,1)=HRR(44,18,1)+VRR(44,18,0)
      HRR(44,43,1)=HRR(44,43,1)+FnSpK*VRR(44,18,0)
      HRR(44,19,1)=HRR(44,19,1)+VRR(44,19,0)
      HRR(44,44,1)=HRR(44,44,1)+FnSpK*VRR(44,19,0)
      HRR(44,20,1)=HRR(44,20,1)+VRR(44,20,0)
      HRR(44,45,1)=HRR(44,45,1)+FnSpK*VRR(44,20,0)
      HRR(44,21,1)=HRR(44,21,1)+VRR(44,21,0)
      HRR(44,22,1)=HRR(44,22,1)+VRR(44,22,0)
      HRR(44,23,1)=HRR(44,23,1)+VRR(44,23,0)
      HRR(44,24,1)=HRR(44,24,1)+VRR(44,24,0)
      HRR(44,25,1)=HRR(44,25,1)+VRR(44,25,0)
      HRR(44,26,1)=HRR(44,26,1)+VRR(44,26,0)
      HRR(44,27,1)=HRR(44,27,1)+VRR(44,27,0)
      HRR(44,28,1)=HRR(44,28,1)+VRR(44,28,0)
      HRR(44,29,1)=HRR(44,29,1)+VRR(44,29,0)
      HRR(44,30,1)=HRR(44,30,1)+VRR(44,30,0)
      HRR(44,31,1)=HRR(44,31,1)+VRR(44,31,0)
      HRR(44,32,1)=HRR(44,32,1)+VRR(44,32,0)
      HRR(44,33,1)=HRR(44,33,1)+VRR(44,33,0)
      HRR(44,34,1)=HRR(44,34,1)+VRR(44,34,0)
      HRR(44,35,1)=HRR(44,35,1)+VRR(44,35,0)
      HRR(45,1,1)=HRR(45,1,1)+VRR(45,1,0)
      HRR(45,2,1)=HRR(45,2,1)+VRR(45,2,0)
      HRR(45,3,1)=HRR(45,3,1)+VRR(45,3,0)
      HRR(45,4,1)=HRR(45,4,1)+VRR(45,4,0)
      HRR(45,5,1)=HRR(45,5,1)+VRR(45,5,0)
      HRR(45,6,1)=HRR(45,6,1)+VRR(45,6,0)
      HRR(45,7,1)=HRR(45,7,1)+VRR(45,7,0)
      HRR(45,8,1)=HRR(45,8,1)+VRR(45,8,0)
      HRR(45,9,1)=HRR(45,9,1)+VRR(45,9,0)
      HRR(45,10,1)=HRR(45,10,1)+VRR(45,10,0)
      HRR(45,11,1)=HRR(45,11,1)+VRR(45,11,0)
      HRR(45,36,1)=HRR(45,36,1)+FnSpK*VRR(45,11,0)
      HRR(45,12,1)=HRR(45,12,1)+VRR(45,12,0)
      HRR(45,37,1)=HRR(45,37,1)+FnSpK*VRR(45,12,0)
      HRR(45,13,1)=HRR(45,13,1)+VRR(45,13,0)
      HRR(45,38,1)=HRR(45,38,1)+FnSpK*VRR(45,13,0)
      HRR(45,14,1)=HRR(45,14,1)+VRR(45,14,0)
      HRR(45,39,1)=HRR(45,39,1)+FnSpK*VRR(45,14,0)
      HRR(45,15,1)=HRR(45,15,1)+VRR(45,15,0)
      HRR(45,40,1)=HRR(45,40,1)+FnSpK*VRR(45,15,0)
      HRR(45,16,1)=HRR(45,16,1)+VRR(45,16,0)
      HRR(45,41,1)=HRR(45,41,1)+FnSpK*VRR(45,16,0)
      HRR(45,17,1)=HRR(45,17,1)+VRR(45,17,0)
      HRR(45,42,1)=HRR(45,42,1)+FnSpK*VRR(45,17,0)
      HRR(45,18,1)=HRR(45,18,1)+VRR(45,18,0)
      HRR(45,43,1)=HRR(45,43,1)+FnSpK*VRR(45,18,0)
      HRR(45,19,1)=HRR(45,19,1)+VRR(45,19,0)
      HRR(45,44,1)=HRR(45,44,1)+FnSpK*VRR(45,19,0)
      HRR(45,20,1)=HRR(45,20,1)+VRR(45,20,0)
      HRR(45,45,1)=HRR(45,45,1)+FnSpK*VRR(45,20,0)
      HRR(45,21,1)=HRR(45,21,1)+VRR(45,21,0)
      HRR(45,22,1)=HRR(45,22,1)+VRR(45,22,0)
      HRR(45,23,1)=HRR(45,23,1)+VRR(45,23,0)
      HRR(45,24,1)=HRR(45,24,1)+VRR(45,24,0)
      HRR(45,25,1)=HRR(45,25,1)+VRR(45,25,0)
      HRR(45,26,1)=HRR(45,26,1)+VRR(45,26,0)
      HRR(45,27,1)=HRR(45,27,1)+VRR(45,27,0)
      HRR(45,28,1)=HRR(45,28,1)+VRR(45,28,0)
      HRR(45,29,1)=HRR(45,29,1)+VRR(45,29,0)
      HRR(45,30,1)=HRR(45,30,1)+VRR(45,30,0)
      HRR(45,31,1)=HRR(45,31,1)+VRR(45,31,0)
      HRR(45,32,1)=HRR(45,32,1)+VRR(45,32,0)
      HRR(45,33,1)=HRR(45,33,1)+VRR(45,33,0)
      HRR(45,34,1)=HRR(45,34,1)+VRR(45,34,0)
      HRR(45,35,1)=HRR(45,35,1)+VRR(45,35,0)
      HRR(46,1,1)=HRR(46,1,1)+VRR(46,1,0)
      HRR(46,2,1)=HRR(46,2,1)+VRR(46,2,0)
      HRR(46,3,1)=HRR(46,3,1)+VRR(46,3,0)
      HRR(46,4,1)=HRR(46,4,1)+VRR(46,4,0)
      HRR(46,5,1)=HRR(46,5,1)+VRR(46,5,0)
      HRR(46,6,1)=HRR(46,6,1)+VRR(46,6,0)
      HRR(46,7,1)=HRR(46,7,1)+VRR(46,7,0)
      HRR(46,8,1)=HRR(46,8,1)+VRR(46,8,0)
      HRR(46,9,1)=HRR(46,9,1)+VRR(46,9,0)
      HRR(46,10,1)=HRR(46,10,1)+VRR(46,10,0)
      HRR(46,11,1)=HRR(46,11,1)+VRR(46,11,0)
      HRR(46,36,1)=HRR(46,36,1)+FnSpK*VRR(46,11,0)
      HRR(46,12,1)=HRR(46,12,1)+VRR(46,12,0)
      HRR(46,37,1)=HRR(46,37,1)+FnSpK*VRR(46,12,0)
      HRR(46,13,1)=HRR(46,13,1)+VRR(46,13,0)
      HRR(46,38,1)=HRR(46,38,1)+FnSpK*VRR(46,13,0)
      HRR(46,14,1)=HRR(46,14,1)+VRR(46,14,0)
      HRR(46,39,1)=HRR(46,39,1)+FnSpK*VRR(46,14,0)
      HRR(46,15,1)=HRR(46,15,1)+VRR(46,15,0)
      HRR(46,40,1)=HRR(46,40,1)+FnSpK*VRR(46,15,0)
      HRR(46,16,1)=HRR(46,16,1)+VRR(46,16,0)
      HRR(46,41,1)=HRR(46,41,1)+FnSpK*VRR(46,16,0)
      HRR(46,17,1)=HRR(46,17,1)+VRR(46,17,0)
      HRR(46,42,1)=HRR(46,42,1)+FnSpK*VRR(46,17,0)
      HRR(46,18,1)=HRR(46,18,1)+VRR(46,18,0)
      HRR(46,43,1)=HRR(46,43,1)+FnSpK*VRR(46,18,0)
      HRR(46,19,1)=HRR(46,19,1)+VRR(46,19,0)
      HRR(46,44,1)=HRR(46,44,1)+FnSpK*VRR(46,19,0)
      HRR(46,20,1)=HRR(46,20,1)+VRR(46,20,0)
      HRR(46,45,1)=HRR(46,45,1)+FnSpK*VRR(46,20,0)
      HRR(46,21,1)=HRR(46,21,1)+VRR(46,21,0)
      HRR(46,22,1)=HRR(46,22,1)+VRR(46,22,0)
      HRR(46,23,1)=HRR(46,23,1)+VRR(46,23,0)
      HRR(46,24,1)=HRR(46,24,1)+VRR(46,24,0)
      HRR(46,25,1)=HRR(46,25,1)+VRR(46,25,0)
      HRR(46,26,1)=HRR(46,26,1)+VRR(46,26,0)
      HRR(46,27,1)=HRR(46,27,1)+VRR(46,27,0)
      HRR(46,28,1)=HRR(46,28,1)+VRR(46,28,0)
      HRR(46,29,1)=HRR(46,29,1)+VRR(46,29,0)
      HRR(46,30,1)=HRR(46,30,1)+VRR(46,30,0)
      HRR(46,31,1)=HRR(46,31,1)+VRR(46,31,0)
      HRR(46,32,1)=HRR(46,32,1)+VRR(46,32,0)
      HRR(46,33,1)=HRR(46,33,1)+VRR(46,33,0)
      HRR(46,34,1)=HRR(46,34,1)+VRR(46,34,0)
      HRR(46,35,1)=HRR(46,35,1)+VRR(46,35,0)
      HRR(47,1,1)=HRR(47,1,1)+VRR(47,1,0)
      HRR(47,2,1)=HRR(47,2,1)+VRR(47,2,0)
      HRR(47,3,1)=HRR(47,3,1)+VRR(47,3,0)
      HRR(47,4,1)=HRR(47,4,1)+VRR(47,4,0)
      HRR(47,5,1)=HRR(47,5,1)+VRR(47,5,0)
      HRR(47,6,1)=HRR(47,6,1)+VRR(47,6,0)
      HRR(47,7,1)=HRR(47,7,1)+VRR(47,7,0)
      HRR(47,8,1)=HRR(47,8,1)+VRR(47,8,0)
      HRR(47,9,1)=HRR(47,9,1)+VRR(47,9,0)
      HRR(47,10,1)=HRR(47,10,1)+VRR(47,10,0)
      HRR(47,11,1)=HRR(47,11,1)+VRR(47,11,0)
      HRR(47,36,1)=HRR(47,36,1)+FnSpK*VRR(47,11,0)
      HRR(47,12,1)=HRR(47,12,1)+VRR(47,12,0)
      HRR(47,37,1)=HRR(47,37,1)+FnSpK*VRR(47,12,0)
      HRR(47,13,1)=HRR(47,13,1)+VRR(47,13,0)
      HRR(47,38,1)=HRR(47,38,1)+FnSpK*VRR(47,13,0)
      HRR(47,14,1)=HRR(47,14,1)+VRR(47,14,0)
      HRR(47,39,1)=HRR(47,39,1)+FnSpK*VRR(47,14,0)
      HRR(47,15,1)=HRR(47,15,1)+VRR(47,15,0)
      HRR(47,40,1)=HRR(47,40,1)+FnSpK*VRR(47,15,0)
      HRR(47,16,1)=HRR(47,16,1)+VRR(47,16,0)
      HRR(47,41,1)=HRR(47,41,1)+FnSpK*VRR(47,16,0)
      HRR(47,17,1)=HRR(47,17,1)+VRR(47,17,0)
      HRR(47,42,1)=HRR(47,42,1)+FnSpK*VRR(47,17,0)
      HRR(47,18,1)=HRR(47,18,1)+VRR(47,18,0)
      HRR(47,43,1)=HRR(47,43,1)+FnSpK*VRR(47,18,0)
      HRR(47,19,1)=HRR(47,19,1)+VRR(47,19,0)
      HRR(47,44,1)=HRR(47,44,1)+FnSpK*VRR(47,19,0)
      HRR(47,20,1)=HRR(47,20,1)+VRR(47,20,0)
      HRR(47,45,1)=HRR(47,45,1)+FnSpK*VRR(47,20,0)
      HRR(47,21,1)=HRR(47,21,1)+VRR(47,21,0)
      HRR(47,22,1)=HRR(47,22,1)+VRR(47,22,0)
      HRR(47,23,1)=HRR(47,23,1)+VRR(47,23,0)
      HRR(47,24,1)=HRR(47,24,1)+VRR(47,24,0)
      HRR(47,25,1)=HRR(47,25,1)+VRR(47,25,0)
      HRR(47,26,1)=HRR(47,26,1)+VRR(47,26,0)
      HRR(47,27,1)=HRR(47,27,1)+VRR(47,27,0)
      HRR(47,28,1)=HRR(47,28,1)+VRR(47,28,0)
      HRR(47,29,1)=HRR(47,29,1)+VRR(47,29,0)
      HRR(47,30,1)=HRR(47,30,1)+VRR(47,30,0)
      HRR(47,31,1)=HRR(47,31,1)+VRR(47,31,0)
      HRR(47,32,1)=HRR(47,32,1)+VRR(47,32,0)
      HRR(47,33,1)=HRR(47,33,1)+VRR(47,33,0)
      HRR(47,34,1)=HRR(47,34,1)+VRR(47,34,0)
      HRR(47,35,1)=HRR(47,35,1)+VRR(47,35,0)
      HRR(48,1,1)=HRR(48,1,1)+VRR(48,1,0)
      HRR(48,2,1)=HRR(48,2,1)+VRR(48,2,0)
      HRR(48,3,1)=HRR(48,3,1)+VRR(48,3,0)
      HRR(48,4,1)=HRR(48,4,1)+VRR(48,4,0)
      HRR(48,5,1)=HRR(48,5,1)+VRR(48,5,0)
      HRR(48,6,1)=HRR(48,6,1)+VRR(48,6,0)
      HRR(48,7,1)=HRR(48,7,1)+VRR(48,7,0)
      HRR(48,8,1)=HRR(48,8,1)+VRR(48,8,0)
      HRR(48,9,1)=HRR(48,9,1)+VRR(48,9,0)
      HRR(48,10,1)=HRR(48,10,1)+VRR(48,10,0)
      HRR(48,11,1)=HRR(48,11,1)+VRR(48,11,0)
      HRR(48,36,1)=HRR(48,36,1)+FnSpK*VRR(48,11,0)
      HRR(48,12,1)=HRR(48,12,1)+VRR(48,12,0)
      HRR(48,37,1)=HRR(48,37,1)+FnSpK*VRR(48,12,0)
      HRR(48,13,1)=HRR(48,13,1)+VRR(48,13,0)
      HRR(48,38,1)=HRR(48,38,1)+FnSpK*VRR(48,13,0)
      HRR(48,14,1)=HRR(48,14,1)+VRR(48,14,0)
      HRR(48,39,1)=HRR(48,39,1)+FnSpK*VRR(48,14,0)
      HRR(48,15,1)=HRR(48,15,1)+VRR(48,15,0)
      HRR(48,40,1)=HRR(48,40,1)+FnSpK*VRR(48,15,0)
      HRR(48,16,1)=HRR(48,16,1)+VRR(48,16,0)
      HRR(48,41,1)=HRR(48,41,1)+FnSpK*VRR(48,16,0)
      HRR(48,17,1)=HRR(48,17,1)+VRR(48,17,0)
      HRR(48,42,1)=HRR(48,42,1)+FnSpK*VRR(48,17,0)
      HRR(48,18,1)=HRR(48,18,1)+VRR(48,18,0)
      HRR(48,43,1)=HRR(48,43,1)+FnSpK*VRR(48,18,0)
      HRR(48,19,1)=HRR(48,19,1)+VRR(48,19,0)
      HRR(48,44,1)=HRR(48,44,1)+FnSpK*VRR(48,19,0)
      HRR(48,20,1)=HRR(48,20,1)+VRR(48,20,0)
      HRR(48,45,1)=HRR(48,45,1)+FnSpK*VRR(48,20,0)
      HRR(48,21,1)=HRR(48,21,1)+VRR(48,21,0)
      HRR(48,22,1)=HRR(48,22,1)+VRR(48,22,0)
      HRR(48,23,1)=HRR(48,23,1)+VRR(48,23,0)
      HRR(48,24,1)=HRR(48,24,1)+VRR(48,24,0)
      HRR(48,25,1)=HRR(48,25,1)+VRR(48,25,0)
      HRR(48,26,1)=HRR(48,26,1)+VRR(48,26,0)
      HRR(48,27,1)=HRR(48,27,1)+VRR(48,27,0)
      HRR(48,28,1)=HRR(48,28,1)+VRR(48,28,0)
      HRR(48,29,1)=HRR(48,29,1)+VRR(48,29,0)
      HRR(48,30,1)=HRR(48,30,1)+VRR(48,30,0)
      HRR(48,31,1)=HRR(48,31,1)+VRR(48,31,0)
      HRR(48,32,1)=HRR(48,32,1)+VRR(48,32,0)
      HRR(48,33,1)=HRR(48,33,1)+VRR(48,33,0)
      HRR(48,34,1)=HRR(48,34,1)+VRR(48,34,0)
      HRR(48,35,1)=HRR(48,35,1)+VRR(48,35,0)
      HRR(49,1,1)=HRR(49,1,1)+VRR(49,1,0)
      HRR(49,2,1)=HRR(49,2,1)+VRR(49,2,0)
      HRR(49,3,1)=HRR(49,3,1)+VRR(49,3,0)
      HRR(49,4,1)=HRR(49,4,1)+VRR(49,4,0)
      HRR(49,5,1)=HRR(49,5,1)+VRR(49,5,0)
      HRR(49,6,1)=HRR(49,6,1)+VRR(49,6,0)
      HRR(49,7,1)=HRR(49,7,1)+VRR(49,7,0)
      HRR(49,8,1)=HRR(49,8,1)+VRR(49,8,0)
      HRR(49,9,1)=HRR(49,9,1)+VRR(49,9,0)
      HRR(49,10,1)=HRR(49,10,1)+VRR(49,10,0)
      HRR(49,11,1)=HRR(49,11,1)+VRR(49,11,0)
      HRR(49,36,1)=HRR(49,36,1)+FnSpK*VRR(49,11,0)
      HRR(49,12,1)=HRR(49,12,1)+VRR(49,12,0)
      HRR(49,37,1)=HRR(49,37,1)+FnSpK*VRR(49,12,0)
      HRR(49,13,1)=HRR(49,13,1)+VRR(49,13,0)
      HRR(49,38,1)=HRR(49,38,1)+FnSpK*VRR(49,13,0)
      HRR(49,14,1)=HRR(49,14,1)+VRR(49,14,0)
      HRR(49,39,1)=HRR(49,39,1)+FnSpK*VRR(49,14,0)
      HRR(49,15,1)=HRR(49,15,1)+VRR(49,15,0)
      HRR(49,40,1)=HRR(49,40,1)+FnSpK*VRR(49,15,0)
      HRR(49,16,1)=HRR(49,16,1)+VRR(49,16,0)
      HRR(49,41,1)=HRR(49,41,1)+FnSpK*VRR(49,16,0)
      HRR(49,17,1)=HRR(49,17,1)+VRR(49,17,0)
      HRR(49,42,1)=HRR(49,42,1)+FnSpK*VRR(49,17,0)
      HRR(49,18,1)=HRR(49,18,1)+VRR(49,18,0)
      HRR(49,43,1)=HRR(49,43,1)+FnSpK*VRR(49,18,0)
      HRR(49,19,1)=HRR(49,19,1)+VRR(49,19,0)
      HRR(49,44,1)=HRR(49,44,1)+FnSpK*VRR(49,19,0)
      HRR(49,20,1)=HRR(49,20,1)+VRR(49,20,0)
      HRR(49,45,1)=HRR(49,45,1)+FnSpK*VRR(49,20,0)
      HRR(49,21,1)=HRR(49,21,1)+VRR(49,21,0)
      HRR(49,22,1)=HRR(49,22,1)+VRR(49,22,0)
      HRR(49,23,1)=HRR(49,23,1)+VRR(49,23,0)
      HRR(49,24,1)=HRR(49,24,1)+VRR(49,24,0)
      HRR(49,25,1)=HRR(49,25,1)+VRR(49,25,0)
      HRR(49,26,1)=HRR(49,26,1)+VRR(49,26,0)
      HRR(49,27,1)=HRR(49,27,1)+VRR(49,27,0)
      HRR(49,28,1)=HRR(49,28,1)+VRR(49,28,0)
      HRR(49,29,1)=HRR(49,29,1)+VRR(49,29,0)
      HRR(49,30,1)=HRR(49,30,1)+VRR(49,30,0)
      HRR(49,31,1)=HRR(49,31,1)+VRR(49,31,0)
      HRR(49,32,1)=HRR(49,32,1)+VRR(49,32,0)
      HRR(49,33,1)=HRR(49,33,1)+VRR(49,33,0)
      HRR(49,34,1)=HRR(49,34,1)+VRR(49,34,0)
      HRR(49,35,1)=HRR(49,35,1)+VRR(49,35,0)
      HRR(50,1,1)=HRR(50,1,1)+VRR(50,1,0)
      HRR(50,2,1)=HRR(50,2,1)+VRR(50,2,0)
      HRR(50,3,1)=HRR(50,3,1)+VRR(50,3,0)
      HRR(50,4,1)=HRR(50,4,1)+VRR(50,4,0)
      HRR(50,5,1)=HRR(50,5,1)+VRR(50,5,0)
      HRR(50,6,1)=HRR(50,6,1)+VRR(50,6,0)
      HRR(50,7,1)=HRR(50,7,1)+VRR(50,7,0)
      HRR(50,8,1)=HRR(50,8,1)+VRR(50,8,0)
      HRR(50,9,1)=HRR(50,9,1)+VRR(50,9,0)
      HRR(50,10,1)=HRR(50,10,1)+VRR(50,10,0)
      HRR(50,11,1)=HRR(50,11,1)+VRR(50,11,0)
      HRR(50,36,1)=HRR(50,36,1)+FnSpK*VRR(50,11,0)
      HRR(50,12,1)=HRR(50,12,1)+VRR(50,12,0)
      HRR(50,37,1)=HRR(50,37,1)+FnSpK*VRR(50,12,0)
      HRR(50,13,1)=HRR(50,13,1)+VRR(50,13,0)
      HRR(50,38,1)=HRR(50,38,1)+FnSpK*VRR(50,13,0)
      HRR(50,14,1)=HRR(50,14,1)+VRR(50,14,0)
      HRR(50,39,1)=HRR(50,39,1)+FnSpK*VRR(50,14,0)
      HRR(50,15,1)=HRR(50,15,1)+VRR(50,15,0)
      HRR(50,40,1)=HRR(50,40,1)+FnSpK*VRR(50,15,0)
      HRR(50,16,1)=HRR(50,16,1)+VRR(50,16,0)
      HRR(50,41,1)=HRR(50,41,1)+FnSpK*VRR(50,16,0)
      HRR(50,17,1)=HRR(50,17,1)+VRR(50,17,0)
      HRR(50,42,1)=HRR(50,42,1)+FnSpK*VRR(50,17,0)
      HRR(50,18,1)=HRR(50,18,1)+VRR(50,18,0)
      HRR(50,43,1)=HRR(50,43,1)+FnSpK*VRR(50,18,0)
      HRR(50,19,1)=HRR(50,19,1)+VRR(50,19,0)
      HRR(50,44,1)=HRR(50,44,1)+FnSpK*VRR(50,19,0)
      HRR(50,20,1)=HRR(50,20,1)+VRR(50,20,0)
      HRR(50,45,1)=HRR(50,45,1)+FnSpK*VRR(50,20,0)
      HRR(50,21,1)=HRR(50,21,1)+VRR(50,21,0)
      HRR(50,22,1)=HRR(50,22,1)+VRR(50,22,0)
      HRR(50,23,1)=HRR(50,23,1)+VRR(50,23,0)
      HRR(50,24,1)=HRR(50,24,1)+VRR(50,24,0)
      HRR(50,25,1)=HRR(50,25,1)+VRR(50,25,0)
      HRR(50,26,1)=HRR(50,26,1)+VRR(50,26,0)
      HRR(50,27,1)=HRR(50,27,1)+VRR(50,27,0)
      HRR(50,28,1)=HRR(50,28,1)+VRR(50,28,0)
      HRR(50,29,1)=HRR(50,29,1)+VRR(50,29,0)
      HRR(50,30,1)=HRR(50,30,1)+VRR(50,30,0)
      HRR(50,31,1)=HRR(50,31,1)+VRR(50,31,0)
      HRR(50,32,1)=HRR(50,32,1)+VRR(50,32,0)
      HRR(50,33,1)=HRR(50,33,1)+VRR(50,33,0)
      HRR(50,34,1)=HRR(50,34,1)+VRR(50,34,0)
      HRR(50,35,1)=HRR(50,35,1)+VRR(50,35,0)
      HRR(51,1,1)=HRR(51,1,1)+VRR(51,1,0)
      HRR(51,2,1)=HRR(51,2,1)+VRR(51,2,0)
      HRR(51,3,1)=HRR(51,3,1)+VRR(51,3,0)
      HRR(51,4,1)=HRR(51,4,1)+VRR(51,4,0)
      HRR(51,5,1)=HRR(51,5,1)+VRR(51,5,0)
      HRR(51,6,1)=HRR(51,6,1)+VRR(51,6,0)
      HRR(51,7,1)=HRR(51,7,1)+VRR(51,7,0)
      HRR(51,8,1)=HRR(51,8,1)+VRR(51,8,0)
      HRR(51,9,1)=HRR(51,9,1)+VRR(51,9,0)
      HRR(51,10,1)=HRR(51,10,1)+VRR(51,10,0)
      HRR(51,11,1)=HRR(51,11,1)+VRR(51,11,0)
      HRR(51,36,1)=HRR(51,36,1)+FnSpK*VRR(51,11,0)
      HRR(51,12,1)=HRR(51,12,1)+VRR(51,12,0)
      HRR(51,37,1)=HRR(51,37,1)+FnSpK*VRR(51,12,0)
      HRR(51,13,1)=HRR(51,13,1)+VRR(51,13,0)
      HRR(51,38,1)=HRR(51,38,1)+FnSpK*VRR(51,13,0)
      HRR(51,14,1)=HRR(51,14,1)+VRR(51,14,0)
      HRR(51,39,1)=HRR(51,39,1)+FnSpK*VRR(51,14,0)
      HRR(51,15,1)=HRR(51,15,1)+VRR(51,15,0)
      HRR(51,40,1)=HRR(51,40,1)+FnSpK*VRR(51,15,0)
      HRR(51,16,1)=HRR(51,16,1)+VRR(51,16,0)
      HRR(51,41,1)=HRR(51,41,1)+FnSpK*VRR(51,16,0)
      HRR(51,17,1)=HRR(51,17,1)+VRR(51,17,0)
      HRR(51,42,1)=HRR(51,42,1)+FnSpK*VRR(51,17,0)
      HRR(51,18,1)=HRR(51,18,1)+VRR(51,18,0)
      HRR(51,43,1)=HRR(51,43,1)+FnSpK*VRR(51,18,0)
      HRR(51,19,1)=HRR(51,19,1)+VRR(51,19,0)
      HRR(51,44,1)=HRR(51,44,1)+FnSpK*VRR(51,19,0)
      HRR(51,20,1)=HRR(51,20,1)+VRR(51,20,0)
      HRR(51,45,1)=HRR(51,45,1)+FnSpK*VRR(51,20,0)
      HRR(51,21,1)=HRR(51,21,1)+VRR(51,21,0)
      HRR(51,22,1)=HRR(51,22,1)+VRR(51,22,0)
      HRR(51,23,1)=HRR(51,23,1)+VRR(51,23,0)
      HRR(51,24,1)=HRR(51,24,1)+VRR(51,24,0)
      HRR(51,25,1)=HRR(51,25,1)+VRR(51,25,0)
      HRR(51,26,1)=HRR(51,26,1)+VRR(51,26,0)
      HRR(51,27,1)=HRR(51,27,1)+VRR(51,27,0)
      HRR(51,28,1)=HRR(51,28,1)+VRR(51,28,0)
      HRR(51,29,1)=HRR(51,29,1)+VRR(51,29,0)
      HRR(51,30,1)=HRR(51,30,1)+VRR(51,30,0)
      HRR(51,31,1)=HRR(51,31,1)+VRR(51,31,0)
      HRR(51,32,1)=HRR(51,32,1)+VRR(51,32,0)
      HRR(51,33,1)=HRR(51,33,1)+VRR(51,33,0)
      HRR(51,34,1)=HRR(51,34,1)+VRR(51,34,0)
      HRR(51,35,1)=HRR(51,35,1)+VRR(51,35,0)
      HRR(52,1,1)=HRR(52,1,1)+VRR(52,1,0)
      HRR(52,2,1)=HRR(52,2,1)+VRR(52,2,0)
      HRR(52,3,1)=HRR(52,3,1)+VRR(52,3,0)
      HRR(52,4,1)=HRR(52,4,1)+VRR(52,4,0)
      HRR(52,5,1)=HRR(52,5,1)+VRR(52,5,0)
      HRR(52,6,1)=HRR(52,6,1)+VRR(52,6,0)
      HRR(52,7,1)=HRR(52,7,1)+VRR(52,7,0)
      HRR(52,8,1)=HRR(52,8,1)+VRR(52,8,0)
      HRR(52,9,1)=HRR(52,9,1)+VRR(52,9,0)
      HRR(52,10,1)=HRR(52,10,1)+VRR(52,10,0)
      HRR(52,11,1)=HRR(52,11,1)+VRR(52,11,0)
      HRR(52,36,1)=HRR(52,36,1)+FnSpK*VRR(52,11,0)
      HRR(52,12,1)=HRR(52,12,1)+VRR(52,12,0)
      HRR(52,37,1)=HRR(52,37,1)+FnSpK*VRR(52,12,0)
      HRR(52,13,1)=HRR(52,13,1)+VRR(52,13,0)
      HRR(52,38,1)=HRR(52,38,1)+FnSpK*VRR(52,13,0)
      HRR(52,14,1)=HRR(52,14,1)+VRR(52,14,0)
      HRR(52,39,1)=HRR(52,39,1)+FnSpK*VRR(52,14,0)
      HRR(52,15,1)=HRR(52,15,1)+VRR(52,15,0)
      HRR(52,40,1)=HRR(52,40,1)+FnSpK*VRR(52,15,0)
      HRR(52,16,1)=HRR(52,16,1)+VRR(52,16,0)
      HRR(52,41,1)=HRR(52,41,1)+FnSpK*VRR(52,16,0)
      HRR(52,17,1)=HRR(52,17,1)+VRR(52,17,0)
      HRR(52,42,1)=HRR(52,42,1)+FnSpK*VRR(52,17,0)
      HRR(52,18,1)=HRR(52,18,1)+VRR(52,18,0)
      HRR(52,43,1)=HRR(52,43,1)+FnSpK*VRR(52,18,0)
      HRR(52,19,1)=HRR(52,19,1)+VRR(52,19,0)
      HRR(52,44,1)=HRR(52,44,1)+FnSpK*VRR(52,19,0)
      HRR(52,20,1)=HRR(52,20,1)+VRR(52,20,0)
      HRR(52,45,1)=HRR(52,45,1)+FnSpK*VRR(52,20,0)
      HRR(52,21,1)=HRR(52,21,1)+VRR(52,21,0)
      HRR(52,22,1)=HRR(52,22,1)+VRR(52,22,0)
      HRR(52,23,1)=HRR(52,23,1)+VRR(52,23,0)
      HRR(52,24,1)=HRR(52,24,1)+VRR(52,24,0)
      HRR(52,25,1)=HRR(52,25,1)+VRR(52,25,0)
      HRR(52,26,1)=HRR(52,26,1)+VRR(52,26,0)
      HRR(52,27,1)=HRR(52,27,1)+VRR(52,27,0)
      HRR(52,28,1)=HRR(52,28,1)+VRR(52,28,0)
      HRR(52,29,1)=HRR(52,29,1)+VRR(52,29,0)
      HRR(52,30,1)=HRR(52,30,1)+VRR(52,30,0)
      HRR(52,31,1)=HRR(52,31,1)+VRR(52,31,0)
      HRR(52,32,1)=HRR(52,32,1)+VRR(52,32,0)
      HRR(52,33,1)=HRR(52,33,1)+VRR(52,33,0)
      HRR(52,34,1)=HRR(52,34,1)+VRR(52,34,0)
      HRR(52,35,1)=HRR(52,35,1)+VRR(52,35,0)
      HRR(53,1,1)=HRR(53,1,1)+VRR(53,1,0)
      HRR(53,2,1)=HRR(53,2,1)+VRR(53,2,0)
      HRR(53,3,1)=HRR(53,3,1)+VRR(53,3,0)
      HRR(53,4,1)=HRR(53,4,1)+VRR(53,4,0)
      HRR(53,5,1)=HRR(53,5,1)+VRR(53,5,0)
      HRR(53,6,1)=HRR(53,6,1)+VRR(53,6,0)
      HRR(53,7,1)=HRR(53,7,1)+VRR(53,7,0)
      HRR(53,8,1)=HRR(53,8,1)+VRR(53,8,0)
      HRR(53,9,1)=HRR(53,9,1)+VRR(53,9,0)
      HRR(53,10,1)=HRR(53,10,1)+VRR(53,10,0)
      HRR(53,11,1)=HRR(53,11,1)+VRR(53,11,0)
      HRR(53,36,1)=HRR(53,36,1)+FnSpK*VRR(53,11,0)
      HRR(53,12,1)=HRR(53,12,1)+VRR(53,12,0)
      HRR(53,37,1)=HRR(53,37,1)+FnSpK*VRR(53,12,0)
      HRR(53,13,1)=HRR(53,13,1)+VRR(53,13,0)
      HRR(53,38,1)=HRR(53,38,1)+FnSpK*VRR(53,13,0)
      HRR(53,14,1)=HRR(53,14,1)+VRR(53,14,0)
      HRR(53,39,1)=HRR(53,39,1)+FnSpK*VRR(53,14,0)
      HRR(53,15,1)=HRR(53,15,1)+VRR(53,15,0)
      HRR(53,40,1)=HRR(53,40,1)+FnSpK*VRR(53,15,0)
      HRR(53,16,1)=HRR(53,16,1)+VRR(53,16,0)
      HRR(53,41,1)=HRR(53,41,1)+FnSpK*VRR(53,16,0)
      HRR(53,17,1)=HRR(53,17,1)+VRR(53,17,0)
      HRR(53,42,1)=HRR(53,42,1)+FnSpK*VRR(53,17,0)
      HRR(53,18,1)=HRR(53,18,1)+VRR(53,18,0)
      HRR(53,43,1)=HRR(53,43,1)+FnSpK*VRR(53,18,0)
      HRR(53,19,1)=HRR(53,19,1)+VRR(53,19,0)
      HRR(53,44,1)=HRR(53,44,1)+FnSpK*VRR(53,19,0)
      HRR(53,20,1)=HRR(53,20,1)+VRR(53,20,0)
      HRR(53,45,1)=HRR(53,45,1)+FnSpK*VRR(53,20,0)
      HRR(53,21,1)=HRR(53,21,1)+VRR(53,21,0)
      HRR(53,22,1)=HRR(53,22,1)+VRR(53,22,0)
      HRR(53,23,1)=HRR(53,23,1)+VRR(53,23,0)
      HRR(53,24,1)=HRR(53,24,1)+VRR(53,24,0)
      HRR(53,25,1)=HRR(53,25,1)+VRR(53,25,0)
      HRR(53,26,1)=HRR(53,26,1)+VRR(53,26,0)
      HRR(53,27,1)=HRR(53,27,1)+VRR(53,27,0)
      HRR(53,28,1)=HRR(53,28,1)+VRR(53,28,0)
      HRR(53,29,1)=HRR(53,29,1)+VRR(53,29,0)
      HRR(53,30,1)=HRR(53,30,1)+VRR(53,30,0)
      HRR(53,31,1)=HRR(53,31,1)+VRR(53,31,0)
      HRR(53,32,1)=HRR(53,32,1)+VRR(53,32,0)
      HRR(53,33,1)=HRR(53,33,1)+VRR(53,33,0)
      HRR(53,34,1)=HRR(53,34,1)+VRR(53,34,0)
      HRR(53,35,1)=HRR(53,35,1)+VRR(53,35,0)
      HRR(54,1,1)=HRR(54,1,1)+VRR(54,1,0)
      HRR(54,2,1)=HRR(54,2,1)+VRR(54,2,0)
      HRR(54,3,1)=HRR(54,3,1)+VRR(54,3,0)
      HRR(54,4,1)=HRR(54,4,1)+VRR(54,4,0)
      HRR(54,5,1)=HRR(54,5,1)+VRR(54,5,0)
      HRR(54,6,1)=HRR(54,6,1)+VRR(54,6,0)
      HRR(54,7,1)=HRR(54,7,1)+VRR(54,7,0)
      HRR(54,8,1)=HRR(54,8,1)+VRR(54,8,0)
      HRR(54,9,1)=HRR(54,9,1)+VRR(54,9,0)
      HRR(54,10,1)=HRR(54,10,1)+VRR(54,10,0)
      HRR(54,11,1)=HRR(54,11,1)+VRR(54,11,0)
      HRR(54,36,1)=HRR(54,36,1)+FnSpK*VRR(54,11,0)
      HRR(54,12,1)=HRR(54,12,1)+VRR(54,12,0)
      HRR(54,37,1)=HRR(54,37,1)+FnSpK*VRR(54,12,0)
      HRR(54,13,1)=HRR(54,13,1)+VRR(54,13,0)
      HRR(54,38,1)=HRR(54,38,1)+FnSpK*VRR(54,13,0)
      HRR(54,14,1)=HRR(54,14,1)+VRR(54,14,0)
      HRR(54,39,1)=HRR(54,39,1)+FnSpK*VRR(54,14,0)
      HRR(54,15,1)=HRR(54,15,1)+VRR(54,15,0)
      HRR(54,40,1)=HRR(54,40,1)+FnSpK*VRR(54,15,0)
      HRR(54,16,1)=HRR(54,16,1)+VRR(54,16,0)
      HRR(54,41,1)=HRR(54,41,1)+FnSpK*VRR(54,16,0)
      HRR(54,17,1)=HRR(54,17,1)+VRR(54,17,0)
      HRR(54,42,1)=HRR(54,42,1)+FnSpK*VRR(54,17,0)
      HRR(54,18,1)=HRR(54,18,1)+VRR(54,18,0)
      HRR(54,43,1)=HRR(54,43,1)+FnSpK*VRR(54,18,0)
      HRR(54,19,1)=HRR(54,19,1)+VRR(54,19,0)
      HRR(54,44,1)=HRR(54,44,1)+FnSpK*VRR(54,19,0)
      HRR(54,20,1)=HRR(54,20,1)+VRR(54,20,0)
      HRR(54,45,1)=HRR(54,45,1)+FnSpK*VRR(54,20,0)
      HRR(54,21,1)=HRR(54,21,1)+VRR(54,21,0)
      HRR(54,22,1)=HRR(54,22,1)+VRR(54,22,0)
      HRR(54,23,1)=HRR(54,23,1)+VRR(54,23,0)
      HRR(54,24,1)=HRR(54,24,1)+VRR(54,24,0)
      HRR(54,25,1)=HRR(54,25,1)+VRR(54,25,0)
      HRR(54,26,1)=HRR(54,26,1)+VRR(54,26,0)
      HRR(54,27,1)=HRR(54,27,1)+VRR(54,27,0)
      HRR(54,28,1)=HRR(54,28,1)+VRR(54,28,0)
      HRR(54,29,1)=HRR(54,29,1)+VRR(54,29,0)
      HRR(54,30,1)=HRR(54,30,1)+VRR(54,30,0)
      HRR(54,31,1)=HRR(54,31,1)+VRR(54,31,0)
      HRR(54,32,1)=HRR(54,32,1)+VRR(54,32,0)
      HRR(54,33,1)=HRR(54,33,1)+VRR(54,33,0)
      HRR(54,34,1)=HRR(54,34,1)+VRR(54,34,0)
      HRR(54,35,1)=HRR(54,35,1)+VRR(54,35,0)
      HRR(55,1,1)=HRR(55,1,1)+VRR(55,1,0)
      HRR(55,2,1)=HRR(55,2,1)+VRR(55,2,0)
      HRR(55,3,1)=HRR(55,3,1)+VRR(55,3,0)
      HRR(55,4,1)=HRR(55,4,1)+VRR(55,4,0)
      HRR(55,5,1)=HRR(55,5,1)+VRR(55,5,0)
      HRR(55,6,1)=HRR(55,6,1)+VRR(55,6,0)
      HRR(55,7,1)=HRR(55,7,1)+VRR(55,7,0)
      HRR(55,8,1)=HRR(55,8,1)+VRR(55,8,0)
      HRR(55,9,1)=HRR(55,9,1)+VRR(55,9,0)
      HRR(55,10,1)=HRR(55,10,1)+VRR(55,10,0)
      HRR(55,11,1)=HRR(55,11,1)+VRR(55,11,0)
      HRR(55,36,1)=HRR(55,36,1)+FnSpK*VRR(55,11,0)
      HRR(55,12,1)=HRR(55,12,1)+VRR(55,12,0)
      HRR(55,37,1)=HRR(55,37,1)+FnSpK*VRR(55,12,0)
      HRR(55,13,1)=HRR(55,13,1)+VRR(55,13,0)
      HRR(55,38,1)=HRR(55,38,1)+FnSpK*VRR(55,13,0)
      HRR(55,14,1)=HRR(55,14,1)+VRR(55,14,0)
      HRR(55,39,1)=HRR(55,39,1)+FnSpK*VRR(55,14,0)
      HRR(55,15,1)=HRR(55,15,1)+VRR(55,15,0)
      HRR(55,40,1)=HRR(55,40,1)+FnSpK*VRR(55,15,0)
      HRR(55,16,1)=HRR(55,16,1)+VRR(55,16,0)
      HRR(55,41,1)=HRR(55,41,1)+FnSpK*VRR(55,16,0)
      HRR(55,17,1)=HRR(55,17,1)+VRR(55,17,0)
      HRR(55,42,1)=HRR(55,42,1)+FnSpK*VRR(55,17,0)
      HRR(55,18,1)=HRR(55,18,1)+VRR(55,18,0)
      HRR(55,43,1)=HRR(55,43,1)+FnSpK*VRR(55,18,0)
      HRR(55,19,1)=HRR(55,19,1)+VRR(55,19,0)
      HRR(55,44,1)=HRR(55,44,1)+FnSpK*VRR(55,19,0)
      HRR(55,20,1)=HRR(55,20,1)+VRR(55,20,0)
      HRR(55,45,1)=HRR(55,45,1)+FnSpK*VRR(55,20,0)
      HRR(55,21,1)=HRR(55,21,1)+VRR(55,21,0)
      HRR(55,22,1)=HRR(55,22,1)+VRR(55,22,0)
      HRR(55,23,1)=HRR(55,23,1)+VRR(55,23,0)
      HRR(55,24,1)=HRR(55,24,1)+VRR(55,24,0)
      HRR(55,25,1)=HRR(55,25,1)+VRR(55,25,0)
      HRR(55,26,1)=HRR(55,26,1)+VRR(55,26,0)
      HRR(55,27,1)=HRR(55,27,1)+VRR(55,27,0)
      HRR(55,28,1)=HRR(55,28,1)+VRR(55,28,0)
      HRR(55,29,1)=HRR(55,29,1)+VRR(55,29,0)
      HRR(55,30,1)=HRR(55,30,1)+VRR(55,30,0)
      HRR(55,31,1)=HRR(55,31,1)+VRR(55,31,0)
      HRR(55,32,1)=HRR(55,32,1)+VRR(55,32,0)
      HRR(55,33,1)=HRR(55,33,1)+VRR(55,33,0)
      HRR(55,34,1)=HRR(55,34,1)+VRR(55,34,0)
      HRR(55,35,1)=HRR(55,35,1)+VRR(55,35,0)
      HRR(56,1,1)=HRR(56,1,1)+VRR(56,1,0)
      HRR(56,2,1)=HRR(56,2,1)+VRR(56,2,0)
      HRR(56,3,1)=HRR(56,3,1)+VRR(56,3,0)
      HRR(56,4,1)=HRR(56,4,1)+VRR(56,4,0)
      HRR(56,5,1)=HRR(56,5,1)+VRR(56,5,0)
      HRR(56,6,1)=HRR(56,6,1)+VRR(56,6,0)
      HRR(56,7,1)=HRR(56,7,1)+VRR(56,7,0)
      HRR(56,8,1)=HRR(56,8,1)+VRR(56,8,0)
      HRR(56,9,1)=HRR(56,9,1)+VRR(56,9,0)
      HRR(56,10,1)=HRR(56,10,1)+VRR(56,10,0)
      HRR(56,11,1)=HRR(56,11,1)+VRR(56,11,0)
      HRR(56,36,1)=HRR(56,36,1)+FnSpK*VRR(56,11,0)
      HRR(56,12,1)=HRR(56,12,1)+VRR(56,12,0)
      HRR(56,37,1)=HRR(56,37,1)+FnSpK*VRR(56,12,0)
      HRR(56,13,1)=HRR(56,13,1)+VRR(56,13,0)
      HRR(56,38,1)=HRR(56,38,1)+FnSpK*VRR(56,13,0)
      HRR(56,14,1)=HRR(56,14,1)+VRR(56,14,0)
      HRR(56,39,1)=HRR(56,39,1)+FnSpK*VRR(56,14,0)
      HRR(56,15,1)=HRR(56,15,1)+VRR(56,15,0)
      HRR(56,40,1)=HRR(56,40,1)+FnSpK*VRR(56,15,0)
      HRR(56,16,1)=HRR(56,16,1)+VRR(56,16,0)
      HRR(56,41,1)=HRR(56,41,1)+FnSpK*VRR(56,16,0)
      HRR(56,17,1)=HRR(56,17,1)+VRR(56,17,0)
      HRR(56,42,1)=HRR(56,42,1)+FnSpK*VRR(56,17,0)
      HRR(56,18,1)=HRR(56,18,1)+VRR(56,18,0)
      HRR(56,43,1)=HRR(56,43,1)+FnSpK*VRR(56,18,0)
      HRR(56,19,1)=HRR(56,19,1)+VRR(56,19,0)
      HRR(56,44,1)=HRR(56,44,1)+FnSpK*VRR(56,19,0)
      HRR(56,20,1)=HRR(56,20,1)+VRR(56,20,0)
      HRR(56,45,1)=HRR(56,45,1)+FnSpK*VRR(56,20,0)
      HRR(56,21,1)=HRR(56,21,1)+VRR(56,21,0)
      HRR(56,22,1)=HRR(56,22,1)+VRR(56,22,0)
      HRR(56,23,1)=HRR(56,23,1)+VRR(56,23,0)
      HRR(56,24,1)=HRR(56,24,1)+VRR(56,24,0)
      HRR(56,25,1)=HRR(56,25,1)+VRR(56,25,0)
      HRR(56,26,1)=HRR(56,26,1)+VRR(56,26,0)
      HRR(56,27,1)=HRR(56,27,1)+VRR(56,27,0)
      HRR(56,28,1)=HRR(56,28,1)+VRR(56,28,0)
      HRR(56,29,1)=HRR(56,29,1)+VRR(56,29,0)
      HRR(56,30,1)=HRR(56,30,1)+VRR(56,30,0)
      HRR(56,31,1)=HRR(56,31,1)+VRR(56,31,0)
      HRR(56,32,1)=HRR(56,32,1)+VRR(56,32,0)
      HRR(56,33,1)=HRR(56,33,1)+VRR(56,33,0)
      HRR(56,34,1)=HRR(56,34,1)+VRR(56,34,0)
      HRR(56,35,1)=HRR(56,35,1)+VRR(56,35,0)
      HRR(57,1,1)=HRR(57,1,1)+VRR(57,1,0)
      HRR(57,2,1)=HRR(57,2,1)+VRR(57,2,0)
      HRR(57,3,1)=HRR(57,3,1)+VRR(57,3,0)
      HRR(57,4,1)=HRR(57,4,1)+VRR(57,4,0)
      HRR(57,5,1)=HRR(57,5,1)+VRR(57,5,0)
      HRR(57,6,1)=HRR(57,6,1)+VRR(57,6,0)
      HRR(57,7,1)=HRR(57,7,1)+VRR(57,7,0)
      HRR(57,8,1)=HRR(57,8,1)+VRR(57,8,0)
      HRR(57,9,1)=HRR(57,9,1)+VRR(57,9,0)
      HRR(57,10,1)=HRR(57,10,1)+VRR(57,10,0)
      HRR(57,11,1)=HRR(57,11,1)+VRR(57,11,0)
      HRR(57,36,1)=HRR(57,36,1)+FnSpK*VRR(57,11,0)
      HRR(57,12,1)=HRR(57,12,1)+VRR(57,12,0)
      HRR(57,37,1)=HRR(57,37,1)+FnSpK*VRR(57,12,0)
      HRR(57,13,1)=HRR(57,13,1)+VRR(57,13,0)
      HRR(57,38,1)=HRR(57,38,1)+FnSpK*VRR(57,13,0)
      HRR(57,14,1)=HRR(57,14,1)+VRR(57,14,0)
      HRR(57,39,1)=HRR(57,39,1)+FnSpK*VRR(57,14,0)
      HRR(57,15,1)=HRR(57,15,1)+VRR(57,15,0)
      HRR(57,40,1)=HRR(57,40,1)+FnSpK*VRR(57,15,0)
      HRR(57,16,1)=HRR(57,16,1)+VRR(57,16,0)
      HRR(57,41,1)=HRR(57,41,1)+FnSpK*VRR(57,16,0)
      HRR(57,17,1)=HRR(57,17,1)+VRR(57,17,0)
      HRR(57,42,1)=HRR(57,42,1)+FnSpK*VRR(57,17,0)
      HRR(57,18,1)=HRR(57,18,1)+VRR(57,18,0)
      HRR(57,43,1)=HRR(57,43,1)+FnSpK*VRR(57,18,0)
      HRR(57,19,1)=HRR(57,19,1)+VRR(57,19,0)
      HRR(57,44,1)=HRR(57,44,1)+FnSpK*VRR(57,19,0)
      HRR(57,20,1)=HRR(57,20,1)+VRR(57,20,0)
      HRR(57,45,1)=HRR(57,45,1)+FnSpK*VRR(57,20,0)
      HRR(57,21,1)=HRR(57,21,1)+VRR(57,21,0)
      HRR(57,22,1)=HRR(57,22,1)+VRR(57,22,0)
      HRR(57,23,1)=HRR(57,23,1)+VRR(57,23,0)
      HRR(57,24,1)=HRR(57,24,1)+VRR(57,24,0)
      HRR(57,25,1)=HRR(57,25,1)+VRR(57,25,0)
      HRR(57,26,1)=HRR(57,26,1)+VRR(57,26,0)
      HRR(57,27,1)=HRR(57,27,1)+VRR(57,27,0)
      HRR(57,28,1)=HRR(57,28,1)+VRR(57,28,0)
      HRR(57,29,1)=HRR(57,29,1)+VRR(57,29,0)
      HRR(57,30,1)=HRR(57,30,1)+VRR(57,30,0)
      HRR(57,31,1)=HRR(57,31,1)+VRR(57,31,0)
      HRR(57,32,1)=HRR(57,32,1)+VRR(57,32,0)
      HRR(57,33,1)=HRR(57,33,1)+VRR(57,33,0)
      HRR(57,34,1)=HRR(57,34,1)+VRR(57,34,0)
      HRR(57,35,1)=HRR(57,35,1)+VRR(57,35,0)
      HRR(58,1,1)=HRR(58,1,1)+VRR(58,1,0)
      HRR(58,2,1)=HRR(58,2,1)+VRR(58,2,0)
      HRR(58,3,1)=HRR(58,3,1)+VRR(58,3,0)
      HRR(58,4,1)=HRR(58,4,1)+VRR(58,4,0)
      HRR(58,5,1)=HRR(58,5,1)+VRR(58,5,0)
      HRR(58,6,1)=HRR(58,6,1)+VRR(58,6,0)
      HRR(58,7,1)=HRR(58,7,1)+VRR(58,7,0)
      HRR(58,8,1)=HRR(58,8,1)+VRR(58,8,0)
      HRR(58,9,1)=HRR(58,9,1)+VRR(58,9,0)
      HRR(58,10,1)=HRR(58,10,1)+VRR(58,10,0)
      HRR(58,11,1)=HRR(58,11,1)+VRR(58,11,0)
      HRR(58,36,1)=HRR(58,36,1)+FnSpK*VRR(58,11,0)
      HRR(58,12,1)=HRR(58,12,1)+VRR(58,12,0)
      HRR(58,37,1)=HRR(58,37,1)+FnSpK*VRR(58,12,0)
      HRR(58,13,1)=HRR(58,13,1)+VRR(58,13,0)
      HRR(58,38,1)=HRR(58,38,1)+FnSpK*VRR(58,13,0)
      HRR(58,14,1)=HRR(58,14,1)+VRR(58,14,0)
      HRR(58,39,1)=HRR(58,39,1)+FnSpK*VRR(58,14,0)
      HRR(58,15,1)=HRR(58,15,1)+VRR(58,15,0)
      HRR(58,40,1)=HRR(58,40,1)+FnSpK*VRR(58,15,0)
      HRR(58,16,1)=HRR(58,16,1)+VRR(58,16,0)
      HRR(58,41,1)=HRR(58,41,1)+FnSpK*VRR(58,16,0)
      HRR(58,17,1)=HRR(58,17,1)+VRR(58,17,0)
      HRR(58,42,1)=HRR(58,42,1)+FnSpK*VRR(58,17,0)
      HRR(58,18,1)=HRR(58,18,1)+VRR(58,18,0)
      HRR(58,43,1)=HRR(58,43,1)+FnSpK*VRR(58,18,0)
      HRR(58,19,1)=HRR(58,19,1)+VRR(58,19,0)
      HRR(58,44,1)=HRR(58,44,1)+FnSpK*VRR(58,19,0)
      HRR(58,20,1)=HRR(58,20,1)+VRR(58,20,0)
      HRR(58,45,1)=HRR(58,45,1)+FnSpK*VRR(58,20,0)
      HRR(58,21,1)=HRR(58,21,1)+VRR(58,21,0)
      HRR(58,22,1)=HRR(58,22,1)+VRR(58,22,0)
      HRR(58,23,1)=HRR(58,23,1)+VRR(58,23,0)
      HRR(58,24,1)=HRR(58,24,1)+VRR(58,24,0)
      HRR(58,25,1)=HRR(58,25,1)+VRR(58,25,0)
      HRR(58,26,1)=HRR(58,26,1)+VRR(58,26,0)
      HRR(58,27,1)=HRR(58,27,1)+VRR(58,27,0)
      HRR(58,28,1)=HRR(58,28,1)+VRR(58,28,0)
      HRR(58,29,1)=HRR(58,29,1)+VRR(58,29,0)
      HRR(58,30,1)=HRR(58,30,1)+VRR(58,30,0)
      HRR(58,31,1)=HRR(58,31,1)+VRR(58,31,0)
      HRR(58,32,1)=HRR(58,32,1)+VRR(58,32,0)
      HRR(58,33,1)=HRR(58,33,1)+VRR(58,33,0)
      HRR(58,34,1)=HRR(58,34,1)+VRR(58,34,0)
      HRR(58,35,1)=HRR(58,35,1)+VRR(58,35,0)
      HRR(59,1,1)=HRR(59,1,1)+VRR(59,1,0)
      HRR(59,2,1)=HRR(59,2,1)+VRR(59,2,0)
      HRR(59,3,1)=HRR(59,3,1)+VRR(59,3,0)
      HRR(59,4,1)=HRR(59,4,1)+VRR(59,4,0)
      HRR(59,5,1)=HRR(59,5,1)+VRR(59,5,0)
      HRR(59,6,1)=HRR(59,6,1)+VRR(59,6,0)
      HRR(59,7,1)=HRR(59,7,1)+VRR(59,7,0)
      HRR(59,8,1)=HRR(59,8,1)+VRR(59,8,0)
      HRR(59,9,1)=HRR(59,9,1)+VRR(59,9,0)
      HRR(59,10,1)=HRR(59,10,1)+VRR(59,10,0)
      HRR(59,11,1)=HRR(59,11,1)+VRR(59,11,0)
      HRR(59,36,1)=HRR(59,36,1)+FnSpK*VRR(59,11,0)
      HRR(59,12,1)=HRR(59,12,1)+VRR(59,12,0)
      HRR(59,37,1)=HRR(59,37,1)+FnSpK*VRR(59,12,0)
      HRR(59,13,1)=HRR(59,13,1)+VRR(59,13,0)
      HRR(59,38,1)=HRR(59,38,1)+FnSpK*VRR(59,13,0)
      HRR(59,14,1)=HRR(59,14,1)+VRR(59,14,0)
      HRR(59,39,1)=HRR(59,39,1)+FnSpK*VRR(59,14,0)
      HRR(59,15,1)=HRR(59,15,1)+VRR(59,15,0)
      HRR(59,40,1)=HRR(59,40,1)+FnSpK*VRR(59,15,0)
      HRR(59,16,1)=HRR(59,16,1)+VRR(59,16,0)
      HRR(59,41,1)=HRR(59,41,1)+FnSpK*VRR(59,16,0)
      HRR(59,17,1)=HRR(59,17,1)+VRR(59,17,0)
      HRR(59,42,1)=HRR(59,42,1)+FnSpK*VRR(59,17,0)
      HRR(59,18,1)=HRR(59,18,1)+VRR(59,18,0)
      HRR(59,43,1)=HRR(59,43,1)+FnSpK*VRR(59,18,0)
      HRR(59,19,1)=HRR(59,19,1)+VRR(59,19,0)
      HRR(59,44,1)=HRR(59,44,1)+FnSpK*VRR(59,19,0)
      HRR(59,20,1)=HRR(59,20,1)+VRR(59,20,0)
      HRR(59,45,1)=HRR(59,45,1)+FnSpK*VRR(59,20,0)
      HRR(59,21,1)=HRR(59,21,1)+VRR(59,21,0)
      HRR(59,22,1)=HRR(59,22,1)+VRR(59,22,0)
      HRR(59,23,1)=HRR(59,23,1)+VRR(59,23,0)
      HRR(59,24,1)=HRR(59,24,1)+VRR(59,24,0)
      HRR(59,25,1)=HRR(59,25,1)+VRR(59,25,0)
      HRR(59,26,1)=HRR(59,26,1)+VRR(59,26,0)
      HRR(59,27,1)=HRR(59,27,1)+VRR(59,27,0)
      HRR(59,28,1)=HRR(59,28,1)+VRR(59,28,0)
      HRR(59,29,1)=HRR(59,29,1)+VRR(59,29,0)
      HRR(59,30,1)=HRR(59,30,1)+VRR(59,30,0)
      HRR(59,31,1)=HRR(59,31,1)+VRR(59,31,0)
      HRR(59,32,1)=HRR(59,32,1)+VRR(59,32,0)
      HRR(59,33,1)=HRR(59,33,1)+VRR(59,33,0)
      HRR(59,34,1)=HRR(59,34,1)+VRR(59,34,0)
      HRR(59,35,1)=HRR(59,35,1)+VRR(59,35,0)
      HRR(60,1,1)=HRR(60,1,1)+VRR(60,1,0)
      HRR(60,2,1)=HRR(60,2,1)+VRR(60,2,0)
      HRR(60,3,1)=HRR(60,3,1)+VRR(60,3,0)
      HRR(60,4,1)=HRR(60,4,1)+VRR(60,4,0)
      HRR(60,5,1)=HRR(60,5,1)+VRR(60,5,0)
      HRR(60,6,1)=HRR(60,6,1)+VRR(60,6,0)
      HRR(60,7,1)=HRR(60,7,1)+VRR(60,7,0)
      HRR(60,8,1)=HRR(60,8,1)+VRR(60,8,0)
      HRR(60,9,1)=HRR(60,9,1)+VRR(60,9,0)
      HRR(60,10,1)=HRR(60,10,1)+VRR(60,10,0)
      HRR(60,11,1)=HRR(60,11,1)+VRR(60,11,0)
      HRR(60,36,1)=HRR(60,36,1)+FnSpK*VRR(60,11,0)
      HRR(60,12,1)=HRR(60,12,1)+VRR(60,12,0)
      HRR(60,37,1)=HRR(60,37,1)+FnSpK*VRR(60,12,0)
      HRR(60,13,1)=HRR(60,13,1)+VRR(60,13,0)
      HRR(60,38,1)=HRR(60,38,1)+FnSpK*VRR(60,13,0)
      HRR(60,14,1)=HRR(60,14,1)+VRR(60,14,0)
      HRR(60,39,1)=HRR(60,39,1)+FnSpK*VRR(60,14,0)
      HRR(60,15,1)=HRR(60,15,1)+VRR(60,15,0)
      HRR(60,40,1)=HRR(60,40,1)+FnSpK*VRR(60,15,0)
      HRR(60,16,1)=HRR(60,16,1)+VRR(60,16,0)
      HRR(60,41,1)=HRR(60,41,1)+FnSpK*VRR(60,16,0)
      HRR(60,17,1)=HRR(60,17,1)+VRR(60,17,0)
      HRR(60,42,1)=HRR(60,42,1)+FnSpK*VRR(60,17,0)
      HRR(60,18,1)=HRR(60,18,1)+VRR(60,18,0)
      HRR(60,43,1)=HRR(60,43,1)+FnSpK*VRR(60,18,0)
      HRR(60,19,1)=HRR(60,19,1)+VRR(60,19,0)
      HRR(60,44,1)=HRR(60,44,1)+FnSpK*VRR(60,19,0)
      HRR(60,20,1)=HRR(60,20,1)+VRR(60,20,0)
      HRR(60,45,1)=HRR(60,45,1)+FnSpK*VRR(60,20,0)
      HRR(60,21,1)=HRR(60,21,1)+VRR(60,21,0)
      HRR(60,22,1)=HRR(60,22,1)+VRR(60,22,0)
      HRR(60,23,1)=HRR(60,23,1)+VRR(60,23,0)
      HRR(60,24,1)=HRR(60,24,1)+VRR(60,24,0)
      HRR(60,25,1)=HRR(60,25,1)+VRR(60,25,0)
      HRR(60,26,1)=HRR(60,26,1)+VRR(60,26,0)
      HRR(60,27,1)=HRR(60,27,1)+VRR(60,27,0)
      HRR(60,28,1)=HRR(60,28,1)+VRR(60,28,0)
      HRR(60,29,1)=HRR(60,29,1)+VRR(60,29,0)
      HRR(60,30,1)=HRR(60,30,1)+VRR(60,30,0)
      HRR(60,31,1)=HRR(60,31,1)+VRR(60,31,0)
      HRR(60,32,1)=HRR(60,32,1)+VRR(60,32,0)
      HRR(60,33,1)=HRR(60,33,1)+VRR(60,33,0)
      HRR(60,34,1)=HRR(60,34,1)+VRR(60,34,0)
      HRR(60,35,1)=HRR(60,35,1)+VRR(60,35,0)
      HRR(61,1,1)=HRR(61,1,1)+VRR(61,1,0)
      HRR(61,2,1)=HRR(61,2,1)+VRR(61,2,0)
      HRR(61,3,1)=HRR(61,3,1)+VRR(61,3,0)
      HRR(61,4,1)=HRR(61,4,1)+VRR(61,4,0)
      HRR(61,5,1)=HRR(61,5,1)+VRR(61,5,0)
      HRR(61,6,1)=HRR(61,6,1)+VRR(61,6,0)
      HRR(61,7,1)=HRR(61,7,1)+VRR(61,7,0)
      HRR(61,8,1)=HRR(61,8,1)+VRR(61,8,0)
      HRR(61,9,1)=HRR(61,9,1)+VRR(61,9,0)
      HRR(61,10,1)=HRR(61,10,1)+VRR(61,10,0)
      HRR(61,11,1)=HRR(61,11,1)+VRR(61,11,0)
      HRR(61,36,1)=HRR(61,36,1)+FnSpK*VRR(61,11,0)
      HRR(61,12,1)=HRR(61,12,1)+VRR(61,12,0)
      HRR(61,37,1)=HRR(61,37,1)+FnSpK*VRR(61,12,0)
      HRR(61,13,1)=HRR(61,13,1)+VRR(61,13,0)
      HRR(61,38,1)=HRR(61,38,1)+FnSpK*VRR(61,13,0)
      HRR(61,14,1)=HRR(61,14,1)+VRR(61,14,0)
      HRR(61,39,1)=HRR(61,39,1)+FnSpK*VRR(61,14,0)
      HRR(61,15,1)=HRR(61,15,1)+VRR(61,15,0)
      HRR(61,40,1)=HRR(61,40,1)+FnSpK*VRR(61,15,0)
      HRR(61,16,1)=HRR(61,16,1)+VRR(61,16,0)
      HRR(61,41,1)=HRR(61,41,1)+FnSpK*VRR(61,16,0)
      HRR(61,17,1)=HRR(61,17,1)+VRR(61,17,0)
      HRR(61,42,1)=HRR(61,42,1)+FnSpK*VRR(61,17,0)
      HRR(61,18,1)=HRR(61,18,1)+VRR(61,18,0)
      HRR(61,43,1)=HRR(61,43,1)+FnSpK*VRR(61,18,0)
      HRR(61,19,1)=HRR(61,19,1)+VRR(61,19,0)
      HRR(61,44,1)=HRR(61,44,1)+FnSpK*VRR(61,19,0)
      HRR(61,20,1)=HRR(61,20,1)+VRR(61,20,0)
      HRR(61,45,1)=HRR(61,45,1)+FnSpK*VRR(61,20,0)
      HRR(61,21,1)=HRR(61,21,1)+VRR(61,21,0)
      HRR(61,22,1)=HRR(61,22,1)+VRR(61,22,0)
      HRR(61,23,1)=HRR(61,23,1)+VRR(61,23,0)
      HRR(61,24,1)=HRR(61,24,1)+VRR(61,24,0)
      HRR(61,25,1)=HRR(61,25,1)+VRR(61,25,0)
      HRR(61,26,1)=HRR(61,26,1)+VRR(61,26,0)
      HRR(61,27,1)=HRR(61,27,1)+VRR(61,27,0)
      HRR(61,28,1)=HRR(61,28,1)+VRR(61,28,0)
      HRR(61,29,1)=HRR(61,29,1)+VRR(61,29,0)
      HRR(61,30,1)=HRR(61,30,1)+VRR(61,30,0)
      HRR(61,31,1)=HRR(61,31,1)+VRR(61,31,0)
      HRR(61,32,1)=HRR(61,32,1)+VRR(61,32,0)
      HRR(61,33,1)=HRR(61,33,1)+VRR(61,33,0)
      HRR(61,34,1)=HRR(61,34,1)+VRR(61,34,0)
      HRR(61,35,1)=HRR(61,35,1)+VRR(61,35,0)
      HRR(62,1,1)=HRR(62,1,1)+VRR(62,1,0)
      HRR(62,2,1)=HRR(62,2,1)+VRR(62,2,0)
      HRR(62,3,1)=HRR(62,3,1)+VRR(62,3,0)
      HRR(62,4,1)=HRR(62,4,1)+VRR(62,4,0)
      HRR(62,5,1)=HRR(62,5,1)+VRR(62,5,0)
      HRR(62,6,1)=HRR(62,6,1)+VRR(62,6,0)
      HRR(62,7,1)=HRR(62,7,1)+VRR(62,7,0)
      HRR(62,8,1)=HRR(62,8,1)+VRR(62,8,0)
      HRR(62,9,1)=HRR(62,9,1)+VRR(62,9,0)
      HRR(62,10,1)=HRR(62,10,1)+VRR(62,10,0)
      HRR(62,11,1)=HRR(62,11,1)+VRR(62,11,0)
      HRR(62,36,1)=HRR(62,36,1)+FnSpK*VRR(62,11,0)
      HRR(62,12,1)=HRR(62,12,1)+VRR(62,12,0)
      HRR(62,37,1)=HRR(62,37,1)+FnSpK*VRR(62,12,0)
      HRR(62,13,1)=HRR(62,13,1)+VRR(62,13,0)
      HRR(62,38,1)=HRR(62,38,1)+FnSpK*VRR(62,13,0)
      HRR(62,14,1)=HRR(62,14,1)+VRR(62,14,0)
      HRR(62,39,1)=HRR(62,39,1)+FnSpK*VRR(62,14,0)
      HRR(62,15,1)=HRR(62,15,1)+VRR(62,15,0)
      HRR(62,40,1)=HRR(62,40,1)+FnSpK*VRR(62,15,0)
      HRR(62,16,1)=HRR(62,16,1)+VRR(62,16,0)
      HRR(62,41,1)=HRR(62,41,1)+FnSpK*VRR(62,16,0)
      HRR(62,17,1)=HRR(62,17,1)+VRR(62,17,0)
      HRR(62,42,1)=HRR(62,42,1)+FnSpK*VRR(62,17,0)
      HRR(62,18,1)=HRR(62,18,1)+VRR(62,18,0)
      HRR(62,43,1)=HRR(62,43,1)+FnSpK*VRR(62,18,0)
      HRR(62,19,1)=HRR(62,19,1)+VRR(62,19,0)
      HRR(62,44,1)=HRR(62,44,1)+FnSpK*VRR(62,19,0)
      HRR(62,20,1)=HRR(62,20,1)+VRR(62,20,0)
      HRR(62,45,1)=HRR(62,45,1)+FnSpK*VRR(62,20,0)
      HRR(62,21,1)=HRR(62,21,1)+VRR(62,21,0)
      HRR(62,22,1)=HRR(62,22,1)+VRR(62,22,0)
      HRR(62,23,1)=HRR(62,23,1)+VRR(62,23,0)
      HRR(62,24,1)=HRR(62,24,1)+VRR(62,24,0)
      HRR(62,25,1)=HRR(62,25,1)+VRR(62,25,0)
      HRR(62,26,1)=HRR(62,26,1)+VRR(62,26,0)
      HRR(62,27,1)=HRR(62,27,1)+VRR(62,27,0)
      HRR(62,28,1)=HRR(62,28,1)+VRR(62,28,0)
      HRR(62,29,1)=HRR(62,29,1)+VRR(62,29,0)
      HRR(62,30,1)=HRR(62,30,1)+VRR(62,30,0)
      HRR(62,31,1)=HRR(62,31,1)+VRR(62,31,0)
      HRR(62,32,1)=HRR(62,32,1)+VRR(62,32,0)
      HRR(62,33,1)=HRR(62,33,1)+VRR(62,33,0)
      HRR(62,34,1)=HRR(62,34,1)+VRR(62,34,0)
      HRR(62,35,1)=HRR(62,35,1)+VRR(62,35,0)
      HRR(63,1,1)=HRR(63,1,1)+VRR(63,1,0)
      HRR(63,2,1)=HRR(63,2,1)+VRR(63,2,0)
      HRR(63,3,1)=HRR(63,3,1)+VRR(63,3,0)
      HRR(63,4,1)=HRR(63,4,1)+VRR(63,4,0)
      HRR(63,5,1)=HRR(63,5,1)+VRR(63,5,0)
      HRR(63,6,1)=HRR(63,6,1)+VRR(63,6,0)
      HRR(63,7,1)=HRR(63,7,1)+VRR(63,7,0)
      HRR(63,8,1)=HRR(63,8,1)+VRR(63,8,0)
      HRR(63,9,1)=HRR(63,9,1)+VRR(63,9,0)
      HRR(63,10,1)=HRR(63,10,1)+VRR(63,10,0)
      HRR(63,11,1)=HRR(63,11,1)+VRR(63,11,0)
      HRR(63,36,1)=HRR(63,36,1)+FnSpK*VRR(63,11,0)
      HRR(63,12,1)=HRR(63,12,1)+VRR(63,12,0)
      HRR(63,37,1)=HRR(63,37,1)+FnSpK*VRR(63,12,0)
      HRR(63,13,1)=HRR(63,13,1)+VRR(63,13,0)
      HRR(63,38,1)=HRR(63,38,1)+FnSpK*VRR(63,13,0)
      HRR(63,14,1)=HRR(63,14,1)+VRR(63,14,0)
      HRR(63,39,1)=HRR(63,39,1)+FnSpK*VRR(63,14,0)
      HRR(63,15,1)=HRR(63,15,1)+VRR(63,15,0)
      HRR(63,40,1)=HRR(63,40,1)+FnSpK*VRR(63,15,0)
      HRR(63,16,1)=HRR(63,16,1)+VRR(63,16,0)
      HRR(63,41,1)=HRR(63,41,1)+FnSpK*VRR(63,16,0)
      HRR(63,17,1)=HRR(63,17,1)+VRR(63,17,0)
      HRR(63,42,1)=HRR(63,42,1)+FnSpK*VRR(63,17,0)
      HRR(63,18,1)=HRR(63,18,1)+VRR(63,18,0)
      HRR(63,43,1)=HRR(63,43,1)+FnSpK*VRR(63,18,0)
      HRR(63,19,1)=HRR(63,19,1)+VRR(63,19,0)
      HRR(63,44,1)=HRR(63,44,1)+FnSpK*VRR(63,19,0)
      HRR(63,20,1)=HRR(63,20,1)+VRR(63,20,0)
      HRR(63,45,1)=HRR(63,45,1)+FnSpK*VRR(63,20,0)
      HRR(63,21,1)=HRR(63,21,1)+VRR(63,21,0)
      HRR(63,22,1)=HRR(63,22,1)+VRR(63,22,0)
      HRR(63,23,1)=HRR(63,23,1)+VRR(63,23,0)
      HRR(63,24,1)=HRR(63,24,1)+VRR(63,24,0)
      HRR(63,25,1)=HRR(63,25,1)+VRR(63,25,0)
      HRR(63,26,1)=HRR(63,26,1)+VRR(63,26,0)
      HRR(63,27,1)=HRR(63,27,1)+VRR(63,27,0)
      HRR(63,28,1)=HRR(63,28,1)+VRR(63,28,0)
      HRR(63,29,1)=HRR(63,29,1)+VRR(63,29,0)
      HRR(63,30,1)=HRR(63,30,1)+VRR(63,30,0)
      HRR(63,31,1)=HRR(63,31,1)+VRR(63,31,0)
      HRR(63,32,1)=HRR(63,32,1)+VRR(63,32,0)
      HRR(63,33,1)=HRR(63,33,1)+VRR(63,33,0)
      HRR(63,34,1)=HRR(63,34,1)+VRR(63,34,0)
      HRR(63,35,1)=HRR(63,35,1)+VRR(63,35,0)
      HRR(64,1,1)=HRR(64,1,1)+VRR(64,1,0)
      HRR(64,2,1)=HRR(64,2,1)+VRR(64,2,0)
      HRR(64,3,1)=HRR(64,3,1)+VRR(64,3,0)
      HRR(64,4,1)=HRR(64,4,1)+VRR(64,4,0)
      HRR(64,5,1)=HRR(64,5,1)+VRR(64,5,0)
      HRR(64,6,1)=HRR(64,6,1)+VRR(64,6,0)
      HRR(64,7,1)=HRR(64,7,1)+VRR(64,7,0)
      HRR(64,8,1)=HRR(64,8,1)+VRR(64,8,0)
      HRR(64,9,1)=HRR(64,9,1)+VRR(64,9,0)
      HRR(64,10,1)=HRR(64,10,1)+VRR(64,10,0)
      HRR(64,11,1)=HRR(64,11,1)+VRR(64,11,0)
      HRR(64,36,1)=HRR(64,36,1)+FnSpK*VRR(64,11,0)
      HRR(64,12,1)=HRR(64,12,1)+VRR(64,12,0)
      HRR(64,37,1)=HRR(64,37,1)+FnSpK*VRR(64,12,0)
      HRR(64,13,1)=HRR(64,13,1)+VRR(64,13,0)
      HRR(64,38,1)=HRR(64,38,1)+FnSpK*VRR(64,13,0)
      HRR(64,14,1)=HRR(64,14,1)+VRR(64,14,0)
      HRR(64,39,1)=HRR(64,39,1)+FnSpK*VRR(64,14,0)
      HRR(64,15,1)=HRR(64,15,1)+VRR(64,15,0)
      HRR(64,40,1)=HRR(64,40,1)+FnSpK*VRR(64,15,0)
      HRR(64,16,1)=HRR(64,16,1)+VRR(64,16,0)
      HRR(64,41,1)=HRR(64,41,1)+FnSpK*VRR(64,16,0)
      HRR(64,17,1)=HRR(64,17,1)+VRR(64,17,0)
      HRR(64,42,1)=HRR(64,42,1)+FnSpK*VRR(64,17,0)
      HRR(64,18,1)=HRR(64,18,1)+VRR(64,18,0)
      HRR(64,43,1)=HRR(64,43,1)+FnSpK*VRR(64,18,0)
      HRR(64,19,1)=HRR(64,19,1)+VRR(64,19,0)
      HRR(64,44,1)=HRR(64,44,1)+FnSpK*VRR(64,19,0)
      HRR(64,20,1)=HRR(64,20,1)+VRR(64,20,0)
      HRR(64,45,1)=HRR(64,45,1)+FnSpK*VRR(64,20,0)
      HRR(64,21,1)=HRR(64,21,1)+VRR(64,21,0)
      HRR(64,22,1)=HRR(64,22,1)+VRR(64,22,0)
      HRR(64,23,1)=HRR(64,23,1)+VRR(64,23,0)
      HRR(64,24,1)=HRR(64,24,1)+VRR(64,24,0)
      HRR(64,25,1)=HRR(64,25,1)+VRR(64,25,0)
      HRR(64,26,1)=HRR(64,26,1)+VRR(64,26,0)
      HRR(64,27,1)=HRR(64,27,1)+VRR(64,27,0)
      HRR(64,28,1)=HRR(64,28,1)+VRR(64,28,0)
      HRR(64,29,1)=HRR(64,29,1)+VRR(64,29,0)
      HRR(64,30,1)=HRR(64,30,1)+VRR(64,30,0)
      HRR(64,31,1)=HRR(64,31,1)+VRR(64,31,0)
      HRR(64,32,1)=HRR(64,32,1)+VRR(64,32,0)
      HRR(64,33,1)=HRR(64,33,1)+VRR(64,33,0)
      HRR(64,34,1)=HRR(64,34,1)+VRR(64,34,0)
      HRR(64,35,1)=HRR(64,35,1)+VRR(64,35,0)
      HRR(65,1,1)=HRR(65,1,1)+VRR(65,1,0)
      HRR(65,2,1)=HRR(65,2,1)+VRR(65,2,0)
      HRR(65,3,1)=HRR(65,3,1)+VRR(65,3,0)
      HRR(65,4,1)=HRR(65,4,1)+VRR(65,4,0)
      HRR(65,5,1)=HRR(65,5,1)+VRR(65,5,0)
      HRR(65,6,1)=HRR(65,6,1)+VRR(65,6,0)
      HRR(65,7,1)=HRR(65,7,1)+VRR(65,7,0)
      HRR(65,8,1)=HRR(65,8,1)+VRR(65,8,0)
      HRR(65,9,1)=HRR(65,9,1)+VRR(65,9,0)
      HRR(65,10,1)=HRR(65,10,1)+VRR(65,10,0)
      HRR(65,11,1)=HRR(65,11,1)+VRR(65,11,0)
      HRR(65,36,1)=HRR(65,36,1)+FnSpK*VRR(65,11,0)
      HRR(65,12,1)=HRR(65,12,1)+VRR(65,12,0)
      HRR(65,37,1)=HRR(65,37,1)+FnSpK*VRR(65,12,0)
      HRR(65,13,1)=HRR(65,13,1)+VRR(65,13,0)
      HRR(65,38,1)=HRR(65,38,1)+FnSpK*VRR(65,13,0)
      HRR(65,14,1)=HRR(65,14,1)+VRR(65,14,0)
      HRR(65,39,1)=HRR(65,39,1)+FnSpK*VRR(65,14,0)
      HRR(65,15,1)=HRR(65,15,1)+VRR(65,15,0)
      HRR(65,40,1)=HRR(65,40,1)+FnSpK*VRR(65,15,0)
      HRR(65,16,1)=HRR(65,16,1)+VRR(65,16,0)
      HRR(65,41,1)=HRR(65,41,1)+FnSpK*VRR(65,16,0)
      HRR(65,17,1)=HRR(65,17,1)+VRR(65,17,0)
      HRR(65,42,1)=HRR(65,42,1)+FnSpK*VRR(65,17,0)
      HRR(65,18,1)=HRR(65,18,1)+VRR(65,18,0)
      HRR(65,43,1)=HRR(65,43,1)+FnSpK*VRR(65,18,0)
      HRR(65,19,1)=HRR(65,19,1)+VRR(65,19,0)
      HRR(65,44,1)=HRR(65,44,1)+FnSpK*VRR(65,19,0)
      HRR(65,20,1)=HRR(65,20,1)+VRR(65,20,0)
      HRR(65,45,1)=HRR(65,45,1)+FnSpK*VRR(65,20,0)
      HRR(65,21,1)=HRR(65,21,1)+VRR(65,21,0)
      HRR(65,22,1)=HRR(65,22,1)+VRR(65,22,0)
      HRR(65,23,1)=HRR(65,23,1)+VRR(65,23,0)
      HRR(65,24,1)=HRR(65,24,1)+VRR(65,24,0)
      HRR(65,25,1)=HRR(65,25,1)+VRR(65,25,0)
      HRR(65,26,1)=HRR(65,26,1)+VRR(65,26,0)
      HRR(65,27,1)=HRR(65,27,1)+VRR(65,27,0)
      HRR(65,28,1)=HRR(65,28,1)+VRR(65,28,0)
      HRR(65,29,1)=HRR(65,29,1)+VRR(65,29,0)
      HRR(65,30,1)=HRR(65,30,1)+VRR(65,30,0)
      HRR(65,31,1)=HRR(65,31,1)+VRR(65,31,0)
      HRR(65,32,1)=HRR(65,32,1)+VRR(65,32,0)
      HRR(65,33,1)=HRR(65,33,1)+VRR(65,33,0)
      HRR(65,34,1)=HRR(65,34,1)+VRR(65,34,0)
      HRR(65,35,1)=HRR(65,35,1)+VRR(65,35,0)
      HRR(66,1,1)=HRR(66,1,1)+VRR(66,1,0)
      HRR(66,2,1)=HRR(66,2,1)+VRR(66,2,0)
      HRR(66,3,1)=HRR(66,3,1)+VRR(66,3,0)
      HRR(66,4,1)=HRR(66,4,1)+VRR(66,4,0)
      HRR(66,5,1)=HRR(66,5,1)+VRR(66,5,0)
      HRR(66,6,1)=HRR(66,6,1)+VRR(66,6,0)
      HRR(66,7,1)=HRR(66,7,1)+VRR(66,7,0)
      HRR(66,8,1)=HRR(66,8,1)+VRR(66,8,0)
      HRR(66,9,1)=HRR(66,9,1)+VRR(66,9,0)
      HRR(66,10,1)=HRR(66,10,1)+VRR(66,10,0)
      HRR(66,11,1)=HRR(66,11,1)+VRR(66,11,0)
      HRR(66,36,1)=HRR(66,36,1)+FnSpK*VRR(66,11,0)
      HRR(66,12,1)=HRR(66,12,1)+VRR(66,12,0)
      HRR(66,37,1)=HRR(66,37,1)+FnSpK*VRR(66,12,0)
      HRR(66,13,1)=HRR(66,13,1)+VRR(66,13,0)
      HRR(66,38,1)=HRR(66,38,1)+FnSpK*VRR(66,13,0)
      HRR(66,14,1)=HRR(66,14,1)+VRR(66,14,0)
      HRR(66,39,1)=HRR(66,39,1)+FnSpK*VRR(66,14,0)
      HRR(66,15,1)=HRR(66,15,1)+VRR(66,15,0)
      HRR(66,40,1)=HRR(66,40,1)+FnSpK*VRR(66,15,0)
      HRR(66,16,1)=HRR(66,16,1)+VRR(66,16,0)
      HRR(66,41,1)=HRR(66,41,1)+FnSpK*VRR(66,16,0)
      HRR(66,17,1)=HRR(66,17,1)+VRR(66,17,0)
      HRR(66,42,1)=HRR(66,42,1)+FnSpK*VRR(66,17,0)
      HRR(66,18,1)=HRR(66,18,1)+VRR(66,18,0)
      HRR(66,43,1)=HRR(66,43,1)+FnSpK*VRR(66,18,0)
      HRR(66,19,1)=HRR(66,19,1)+VRR(66,19,0)
      HRR(66,44,1)=HRR(66,44,1)+FnSpK*VRR(66,19,0)
      HRR(66,20,1)=HRR(66,20,1)+VRR(66,20,0)
      HRR(66,45,1)=HRR(66,45,1)+FnSpK*VRR(66,20,0)
      HRR(66,21,1)=HRR(66,21,1)+VRR(66,21,0)
      HRR(66,22,1)=HRR(66,22,1)+VRR(66,22,0)
      HRR(66,23,1)=HRR(66,23,1)+VRR(66,23,0)
      HRR(66,24,1)=HRR(66,24,1)+VRR(66,24,0)
      HRR(66,25,1)=HRR(66,25,1)+VRR(66,25,0)
      HRR(66,26,1)=HRR(66,26,1)+VRR(66,26,0)
      HRR(66,27,1)=HRR(66,27,1)+VRR(66,27,0)
      HRR(66,28,1)=HRR(66,28,1)+VRR(66,28,0)
      HRR(66,29,1)=HRR(66,29,1)+VRR(66,29,0)
      HRR(66,30,1)=HRR(66,30,1)+VRR(66,30,0)
      HRR(66,31,1)=HRR(66,31,1)+VRR(66,31,0)
      HRR(66,32,1)=HRR(66,32,1)+VRR(66,32,0)
      HRR(66,33,1)=HRR(66,33,1)+VRR(66,33,0)
      HRR(66,34,1)=HRR(66,34,1)+VRR(66,34,0)
      HRR(66,35,1)=HRR(66,35,1)+VRR(66,35,0)
      HRR(67,1,1)=HRR(67,1,1)+VRR(67,1,0)
      HRR(67,2,1)=HRR(67,2,1)+VRR(67,2,0)
      HRR(67,3,1)=HRR(67,3,1)+VRR(67,3,0)
      HRR(67,4,1)=HRR(67,4,1)+VRR(67,4,0)
      HRR(67,5,1)=HRR(67,5,1)+VRR(67,5,0)
      HRR(67,6,1)=HRR(67,6,1)+VRR(67,6,0)
      HRR(67,7,1)=HRR(67,7,1)+VRR(67,7,0)
      HRR(67,8,1)=HRR(67,8,1)+VRR(67,8,0)
      HRR(67,9,1)=HRR(67,9,1)+VRR(67,9,0)
      HRR(67,10,1)=HRR(67,10,1)+VRR(67,10,0)
      HRR(67,11,1)=HRR(67,11,1)+VRR(67,11,0)
      HRR(67,36,1)=HRR(67,36,1)+FnSpK*VRR(67,11,0)
      HRR(67,12,1)=HRR(67,12,1)+VRR(67,12,0)
      HRR(67,37,1)=HRR(67,37,1)+FnSpK*VRR(67,12,0)
      HRR(67,13,1)=HRR(67,13,1)+VRR(67,13,0)
      HRR(67,38,1)=HRR(67,38,1)+FnSpK*VRR(67,13,0)
      HRR(67,14,1)=HRR(67,14,1)+VRR(67,14,0)
      HRR(67,39,1)=HRR(67,39,1)+FnSpK*VRR(67,14,0)
      HRR(67,15,1)=HRR(67,15,1)+VRR(67,15,0)
      HRR(67,40,1)=HRR(67,40,1)+FnSpK*VRR(67,15,0)
      HRR(67,16,1)=HRR(67,16,1)+VRR(67,16,0)
      HRR(67,41,1)=HRR(67,41,1)+FnSpK*VRR(67,16,0)
      HRR(67,17,1)=HRR(67,17,1)+VRR(67,17,0)
      HRR(67,42,1)=HRR(67,42,1)+FnSpK*VRR(67,17,0)
      HRR(67,18,1)=HRR(67,18,1)+VRR(67,18,0)
      HRR(67,43,1)=HRR(67,43,1)+FnSpK*VRR(67,18,0)
      HRR(67,19,1)=HRR(67,19,1)+VRR(67,19,0)
      HRR(67,44,1)=HRR(67,44,1)+FnSpK*VRR(67,19,0)
      HRR(67,20,1)=HRR(67,20,1)+VRR(67,20,0)
      HRR(67,45,1)=HRR(67,45,1)+FnSpK*VRR(67,20,0)
      HRR(67,21,1)=HRR(67,21,1)+VRR(67,21,0)
      HRR(67,22,1)=HRR(67,22,1)+VRR(67,22,0)
      HRR(67,23,1)=HRR(67,23,1)+VRR(67,23,0)
      HRR(67,24,1)=HRR(67,24,1)+VRR(67,24,0)
      HRR(67,25,1)=HRR(67,25,1)+VRR(67,25,0)
      HRR(67,26,1)=HRR(67,26,1)+VRR(67,26,0)
      HRR(67,27,1)=HRR(67,27,1)+VRR(67,27,0)
      HRR(67,28,1)=HRR(67,28,1)+VRR(67,28,0)
      HRR(67,29,1)=HRR(67,29,1)+VRR(67,29,0)
      HRR(67,30,1)=HRR(67,30,1)+VRR(67,30,0)
      HRR(67,31,1)=HRR(67,31,1)+VRR(67,31,0)
      HRR(67,32,1)=HRR(67,32,1)+VRR(67,32,0)
      HRR(67,33,1)=HRR(67,33,1)+VRR(67,33,0)
      HRR(67,34,1)=HRR(67,34,1)+VRR(67,34,0)
      HRR(67,35,1)=HRR(67,35,1)+VRR(67,35,0)
      HRR(68,1,1)=HRR(68,1,1)+VRR(68,1,0)
      HRR(68,2,1)=HRR(68,2,1)+VRR(68,2,0)
      HRR(68,3,1)=HRR(68,3,1)+VRR(68,3,0)
      HRR(68,4,1)=HRR(68,4,1)+VRR(68,4,0)
      HRR(68,5,1)=HRR(68,5,1)+VRR(68,5,0)
      HRR(68,6,1)=HRR(68,6,1)+VRR(68,6,0)
      HRR(68,7,1)=HRR(68,7,1)+VRR(68,7,0)
      HRR(68,8,1)=HRR(68,8,1)+VRR(68,8,0)
      HRR(68,9,1)=HRR(68,9,1)+VRR(68,9,0)
      HRR(68,10,1)=HRR(68,10,1)+VRR(68,10,0)
      HRR(68,11,1)=HRR(68,11,1)+VRR(68,11,0)
      HRR(68,36,1)=HRR(68,36,1)+FnSpK*VRR(68,11,0)
      HRR(68,12,1)=HRR(68,12,1)+VRR(68,12,0)
      HRR(68,37,1)=HRR(68,37,1)+FnSpK*VRR(68,12,0)
      HRR(68,13,1)=HRR(68,13,1)+VRR(68,13,0)
      HRR(68,38,1)=HRR(68,38,1)+FnSpK*VRR(68,13,0)
      HRR(68,14,1)=HRR(68,14,1)+VRR(68,14,0)
      HRR(68,39,1)=HRR(68,39,1)+FnSpK*VRR(68,14,0)
      HRR(68,15,1)=HRR(68,15,1)+VRR(68,15,0)
      HRR(68,40,1)=HRR(68,40,1)+FnSpK*VRR(68,15,0)
      HRR(68,16,1)=HRR(68,16,1)+VRR(68,16,0)
      HRR(68,41,1)=HRR(68,41,1)+FnSpK*VRR(68,16,0)
      HRR(68,17,1)=HRR(68,17,1)+VRR(68,17,0)
      HRR(68,42,1)=HRR(68,42,1)+FnSpK*VRR(68,17,0)
      HRR(68,18,1)=HRR(68,18,1)+VRR(68,18,0)
      HRR(68,43,1)=HRR(68,43,1)+FnSpK*VRR(68,18,0)
      HRR(68,19,1)=HRR(68,19,1)+VRR(68,19,0)
      HRR(68,44,1)=HRR(68,44,1)+FnSpK*VRR(68,19,0)
      HRR(68,20,1)=HRR(68,20,1)+VRR(68,20,0)
      HRR(68,45,1)=HRR(68,45,1)+FnSpK*VRR(68,20,0)
      HRR(68,21,1)=HRR(68,21,1)+VRR(68,21,0)
      HRR(68,22,1)=HRR(68,22,1)+VRR(68,22,0)
      HRR(68,23,1)=HRR(68,23,1)+VRR(68,23,0)
      HRR(68,24,1)=HRR(68,24,1)+VRR(68,24,0)
      HRR(68,25,1)=HRR(68,25,1)+VRR(68,25,0)
      HRR(68,26,1)=HRR(68,26,1)+VRR(68,26,0)
      HRR(68,27,1)=HRR(68,27,1)+VRR(68,27,0)
      HRR(68,28,1)=HRR(68,28,1)+VRR(68,28,0)
      HRR(68,29,1)=HRR(68,29,1)+VRR(68,29,0)
      HRR(68,30,1)=HRR(68,30,1)+VRR(68,30,0)
      HRR(68,31,1)=HRR(68,31,1)+VRR(68,31,0)
      HRR(68,32,1)=HRR(68,32,1)+VRR(68,32,0)
      HRR(68,33,1)=HRR(68,33,1)+VRR(68,33,0)
      HRR(68,34,1)=HRR(68,34,1)+VRR(68,34,0)
      HRR(68,35,1)=HRR(68,35,1)+VRR(68,35,0)
      HRR(69,1,1)=HRR(69,1,1)+VRR(69,1,0)
      HRR(69,2,1)=HRR(69,2,1)+VRR(69,2,0)
      HRR(69,3,1)=HRR(69,3,1)+VRR(69,3,0)
      HRR(69,4,1)=HRR(69,4,1)+VRR(69,4,0)
      HRR(69,5,1)=HRR(69,5,1)+VRR(69,5,0)
      HRR(69,6,1)=HRR(69,6,1)+VRR(69,6,0)
      HRR(69,7,1)=HRR(69,7,1)+VRR(69,7,0)
      HRR(69,8,1)=HRR(69,8,1)+VRR(69,8,0)
      HRR(69,9,1)=HRR(69,9,1)+VRR(69,9,0)
      HRR(69,10,1)=HRR(69,10,1)+VRR(69,10,0)
      HRR(69,11,1)=HRR(69,11,1)+VRR(69,11,0)
      HRR(69,36,1)=HRR(69,36,1)+FnSpK*VRR(69,11,0)
      HRR(69,12,1)=HRR(69,12,1)+VRR(69,12,0)
      HRR(69,37,1)=HRR(69,37,1)+FnSpK*VRR(69,12,0)
      HRR(69,13,1)=HRR(69,13,1)+VRR(69,13,0)
      HRR(69,38,1)=HRR(69,38,1)+FnSpK*VRR(69,13,0)
      HRR(69,14,1)=HRR(69,14,1)+VRR(69,14,0)
      HRR(69,39,1)=HRR(69,39,1)+FnSpK*VRR(69,14,0)
      HRR(69,15,1)=HRR(69,15,1)+VRR(69,15,0)
      HRR(69,40,1)=HRR(69,40,1)+FnSpK*VRR(69,15,0)
      HRR(69,16,1)=HRR(69,16,1)+VRR(69,16,0)
      HRR(69,41,1)=HRR(69,41,1)+FnSpK*VRR(69,16,0)
      HRR(69,17,1)=HRR(69,17,1)+VRR(69,17,0)
      HRR(69,42,1)=HRR(69,42,1)+FnSpK*VRR(69,17,0)
      HRR(69,18,1)=HRR(69,18,1)+VRR(69,18,0)
      HRR(69,43,1)=HRR(69,43,1)+FnSpK*VRR(69,18,0)
      HRR(69,19,1)=HRR(69,19,1)+VRR(69,19,0)
      HRR(69,44,1)=HRR(69,44,1)+FnSpK*VRR(69,19,0)
      HRR(69,20,1)=HRR(69,20,1)+VRR(69,20,0)
      HRR(69,45,1)=HRR(69,45,1)+FnSpK*VRR(69,20,0)
      HRR(69,21,1)=HRR(69,21,1)+VRR(69,21,0)
      HRR(69,22,1)=HRR(69,22,1)+VRR(69,22,0)
      HRR(69,23,1)=HRR(69,23,1)+VRR(69,23,0)
      HRR(69,24,1)=HRR(69,24,1)+VRR(69,24,0)
      HRR(69,25,1)=HRR(69,25,1)+VRR(69,25,0)
      HRR(69,26,1)=HRR(69,26,1)+VRR(69,26,0)
      HRR(69,27,1)=HRR(69,27,1)+VRR(69,27,0)
      HRR(69,28,1)=HRR(69,28,1)+VRR(69,28,0)
      HRR(69,29,1)=HRR(69,29,1)+VRR(69,29,0)
      HRR(69,30,1)=HRR(69,30,1)+VRR(69,30,0)
      HRR(69,31,1)=HRR(69,31,1)+VRR(69,31,0)
      HRR(69,32,1)=HRR(69,32,1)+VRR(69,32,0)
      HRR(69,33,1)=HRR(69,33,1)+VRR(69,33,0)
      HRR(69,34,1)=HRR(69,34,1)+VRR(69,34,0)
      HRR(69,35,1)=HRR(69,35,1)+VRR(69,35,0)
      HRR(70,1,1)=HRR(70,1,1)+VRR(70,1,0)
      HRR(70,2,1)=HRR(70,2,1)+VRR(70,2,0)
      HRR(70,3,1)=HRR(70,3,1)+VRR(70,3,0)
      HRR(70,4,1)=HRR(70,4,1)+VRR(70,4,0)
      HRR(70,5,1)=HRR(70,5,1)+VRR(70,5,0)
      HRR(70,6,1)=HRR(70,6,1)+VRR(70,6,0)
      HRR(70,7,1)=HRR(70,7,1)+VRR(70,7,0)
      HRR(70,8,1)=HRR(70,8,1)+VRR(70,8,0)
      HRR(70,9,1)=HRR(70,9,1)+VRR(70,9,0)
      HRR(70,10,1)=HRR(70,10,1)+VRR(70,10,0)
      HRR(70,11,1)=HRR(70,11,1)+VRR(70,11,0)
      HRR(70,36,1)=HRR(70,36,1)+FnSpK*VRR(70,11,0)
      HRR(70,12,1)=HRR(70,12,1)+VRR(70,12,0)
      HRR(70,37,1)=HRR(70,37,1)+FnSpK*VRR(70,12,0)
      HRR(70,13,1)=HRR(70,13,1)+VRR(70,13,0)
      HRR(70,38,1)=HRR(70,38,1)+FnSpK*VRR(70,13,0)
      HRR(70,14,1)=HRR(70,14,1)+VRR(70,14,0)
      HRR(70,39,1)=HRR(70,39,1)+FnSpK*VRR(70,14,0)
      HRR(70,15,1)=HRR(70,15,1)+VRR(70,15,0)
      HRR(70,40,1)=HRR(70,40,1)+FnSpK*VRR(70,15,0)
      HRR(70,16,1)=HRR(70,16,1)+VRR(70,16,0)
      HRR(70,41,1)=HRR(70,41,1)+FnSpK*VRR(70,16,0)
      HRR(70,17,1)=HRR(70,17,1)+VRR(70,17,0)
      HRR(70,42,1)=HRR(70,42,1)+FnSpK*VRR(70,17,0)
      HRR(70,18,1)=HRR(70,18,1)+VRR(70,18,0)
      HRR(70,43,1)=HRR(70,43,1)+FnSpK*VRR(70,18,0)
      HRR(70,19,1)=HRR(70,19,1)+VRR(70,19,0)
      HRR(70,44,1)=HRR(70,44,1)+FnSpK*VRR(70,19,0)
      HRR(70,20,1)=HRR(70,20,1)+VRR(70,20,0)
      HRR(70,45,1)=HRR(70,45,1)+FnSpK*VRR(70,20,0)
      HRR(70,21,1)=HRR(70,21,1)+VRR(70,21,0)
      HRR(70,22,1)=HRR(70,22,1)+VRR(70,22,0)
      HRR(70,23,1)=HRR(70,23,1)+VRR(70,23,0)
      HRR(70,24,1)=HRR(70,24,1)+VRR(70,24,0)
      HRR(70,25,1)=HRR(70,25,1)+VRR(70,25,0)
      HRR(70,26,1)=HRR(70,26,1)+VRR(70,26,0)
      HRR(70,27,1)=HRR(70,27,1)+VRR(70,27,0)
      HRR(70,28,1)=HRR(70,28,1)+VRR(70,28,0)
      HRR(70,29,1)=HRR(70,29,1)+VRR(70,29,0)
      HRR(70,30,1)=HRR(70,30,1)+VRR(70,30,0)
      HRR(70,31,1)=HRR(70,31,1)+VRR(70,31,0)
      HRR(70,32,1)=HRR(70,32,1)+VRR(70,32,0)
      HRR(70,33,1)=HRR(70,33,1)+VRR(70,33,0)
      HRR(70,34,1)=HRR(70,34,1)+VRR(70,34,0)
      HRR(70,35,1)=HRR(70,35,1)+VRR(70,35,0)
      HRR(71,1,1)=HRR(71,1,1)+VRR(71,1,0)
      HRR(71,2,1)=HRR(71,2,1)+VRR(71,2,0)
      HRR(71,3,1)=HRR(71,3,1)+VRR(71,3,0)
      HRR(71,4,1)=HRR(71,4,1)+VRR(71,4,0)
      HRR(71,5,1)=HRR(71,5,1)+VRR(71,5,0)
      HRR(71,6,1)=HRR(71,6,1)+VRR(71,6,0)
      HRR(71,7,1)=HRR(71,7,1)+VRR(71,7,0)
      HRR(71,8,1)=HRR(71,8,1)+VRR(71,8,0)
      HRR(71,9,1)=HRR(71,9,1)+VRR(71,9,0)
      HRR(71,10,1)=HRR(71,10,1)+VRR(71,10,0)
      HRR(71,11,1)=HRR(71,11,1)+VRR(71,11,0)
      HRR(71,36,1)=HRR(71,36,1)+FnSpK*VRR(71,11,0)
      HRR(71,12,1)=HRR(71,12,1)+VRR(71,12,0)
      HRR(71,37,1)=HRR(71,37,1)+FnSpK*VRR(71,12,0)
      HRR(71,13,1)=HRR(71,13,1)+VRR(71,13,0)
      HRR(71,38,1)=HRR(71,38,1)+FnSpK*VRR(71,13,0)
      HRR(71,14,1)=HRR(71,14,1)+VRR(71,14,0)
      HRR(71,39,1)=HRR(71,39,1)+FnSpK*VRR(71,14,0)
      HRR(71,15,1)=HRR(71,15,1)+VRR(71,15,0)
      HRR(71,40,1)=HRR(71,40,1)+FnSpK*VRR(71,15,0)
      HRR(71,16,1)=HRR(71,16,1)+VRR(71,16,0)
      HRR(71,41,1)=HRR(71,41,1)+FnSpK*VRR(71,16,0)
      HRR(71,17,1)=HRR(71,17,1)+VRR(71,17,0)
      HRR(71,42,1)=HRR(71,42,1)+FnSpK*VRR(71,17,0)
      HRR(71,18,1)=HRR(71,18,1)+VRR(71,18,0)
      HRR(71,43,1)=HRR(71,43,1)+FnSpK*VRR(71,18,0)
      HRR(71,19,1)=HRR(71,19,1)+VRR(71,19,0)
      HRR(71,44,1)=HRR(71,44,1)+FnSpK*VRR(71,19,0)
      HRR(71,20,1)=HRR(71,20,1)+VRR(71,20,0)
      HRR(71,45,1)=HRR(71,45,1)+FnSpK*VRR(71,20,0)
      HRR(71,21,1)=HRR(71,21,1)+VRR(71,21,0)
      HRR(71,22,1)=HRR(71,22,1)+VRR(71,22,0)
      HRR(71,23,1)=HRR(71,23,1)+VRR(71,23,0)
      HRR(71,24,1)=HRR(71,24,1)+VRR(71,24,0)
      HRR(71,25,1)=HRR(71,25,1)+VRR(71,25,0)
      HRR(71,26,1)=HRR(71,26,1)+VRR(71,26,0)
      HRR(71,27,1)=HRR(71,27,1)+VRR(71,27,0)
      HRR(71,28,1)=HRR(71,28,1)+VRR(71,28,0)
      HRR(71,29,1)=HRR(71,29,1)+VRR(71,29,0)
      HRR(71,30,1)=HRR(71,30,1)+VRR(71,30,0)
      HRR(71,31,1)=HRR(71,31,1)+VRR(71,31,0)
      HRR(71,32,1)=HRR(71,32,1)+VRR(71,32,0)
      HRR(71,33,1)=HRR(71,33,1)+VRR(71,33,0)
      HRR(71,34,1)=HRR(71,34,1)+VRR(71,34,0)
      HRR(71,35,1)=HRR(71,35,1)+VRR(71,35,0)
      HRR(72,1,1)=HRR(72,1,1)+VRR(72,1,0)
      HRR(72,2,1)=HRR(72,2,1)+VRR(72,2,0)
      HRR(72,3,1)=HRR(72,3,1)+VRR(72,3,0)
      HRR(72,4,1)=HRR(72,4,1)+VRR(72,4,0)
      HRR(72,5,1)=HRR(72,5,1)+VRR(72,5,0)
      HRR(72,6,1)=HRR(72,6,1)+VRR(72,6,0)
      HRR(72,7,1)=HRR(72,7,1)+VRR(72,7,0)
      HRR(72,8,1)=HRR(72,8,1)+VRR(72,8,0)
      HRR(72,9,1)=HRR(72,9,1)+VRR(72,9,0)
      HRR(72,10,1)=HRR(72,10,1)+VRR(72,10,0)
      HRR(72,11,1)=HRR(72,11,1)+VRR(72,11,0)
      HRR(72,36,1)=HRR(72,36,1)+FnSpK*VRR(72,11,0)
      HRR(72,12,1)=HRR(72,12,1)+VRR(72,12,0)
      HRR(72,37,1)=HRR(72,37,1)+FnSpK*VRR(72,12,0)
      HRR(72,13,1)=HRR(72,13,1)+VRR(72,13,0)
      HRR(72,38,1)=HRR(72,38,1)+FnSpK*VRR(72,13,0)
      HRR(72,14,1)=HRR(72,14,1)+VRR(72,14,0)
      HRR(72,39,1)=HRR(72,39,1)+FnSpK*VRR(72,14,0)
      HRR(72,15,1)=HRR(72,15,1)+VRR(72,15,0)
      HRR(72,40,1)=HRR(72,40,1)+FnSpK*VRR(72,15,0)
      HRR(72,16,1)=HRR(72,16,1)+VRR(72,16,0)
      HRR(72,41,1)=HRR(72,41,1)+FnSpK*VRR(72,16,0)
      HRR(72,17,1)=HRR(72,17,1)+VRR(72,17,0)
      HRR(72,42,1)=HRR(72,42,1)+FnSpK*VRR(72,17,0)
      HRR(72,18,1)=HRR(72,18,1)+VRR(72,18,0)
      HRR(72,43,1)=HRR(72,43,1)+FnSpK*VRR(72,18,0)
      HRR(72,19,1)=HRR(72,19,1)+VRR(72,19,0)
      HRR(72,44,1)=HRR(72,44,1)+FnSpK*VRR(72,19,0)
      HRR(72,20,1)=HRR(72,20,1)+VRR(72,20,0)
      HRR(72,45,1)=HRR(72,45,1)+FnSpK*VRR(72,20,0)
      HRR(72,21,1)=HRR(72,21,1)+VRR(72,21,0)
      HRR(72,22,1)=HRR(72,22,1)+VRR(72,22,0)
      HRR(72,23,1)=HRR(72,23,1)+VRR(72,23,0)
      HRR(72,24,1)=HRR(72,24,1)+VRR(72,24,0)
      HRR(72,25,1)=HRR(72,25,1)+VRR(72,25,0)
      HRR(72,26,1)=HRR(72,26,1)+VRR(72,26,0)
      HRR(72,27,1)=HRR(72,27,1)+VRR(72,27,0)
      HRR(72,28,1)=HRR(72,28,1)+VRR(72,28,0)
      HRR(72,29,1)=HRR(72,29,1)+VRR(72,29,0)
      HRR(72,30,1)=HRR(72,30,1)+VRR(72,30,0)
      HRR(72,31,1)=HRR(72,31,1)+VRR(72,31,0)
      HRR(72,32,1)=HRR(72,32,1)+VRR(72,32,0)
      HRR(72,33,1)=HRR(72,33,1)+VRR(72,33,0)
      HRR(72,34,1)=HRR(72,34,1)+VRR(72,34,0)
      HRR(72,35,1)=HRR(72,35,1)+VRR(72,35,0)
      HRR(73,1,1)=HRR(73,1,1)+VRR(73,1,0)
      HRR(73,2,1)=HRR(73,2,1)+VRR(73,2,0)
      HRR(73,3,1)=HRR(73,3,1)+VRR(73,3,0)
      HRR(73,4,1)=HRR(73,4,1)+VRR(73,4,0)
      HRR(73,5,1)=HRR(73,5,1)+VRR(73,5,0)
      HRR(73,6,1)=HRR(73,6,1)+VRR(73,6,0)
      HRR(73,7,1)=HRR(73,7,1)+VRR(73,7,0)
      HRR(73,8,1)=HRR(73,8,1)+VRR(73,8,0)
      HRR(73,9,1)=HRR(73,9,1)+VRR(73,9,0)
      HRR(73,10,1)=HRR(73,10,1)+VRR(73,10,0)
      HRR(73,11,1)=HRR(73,11,1)+VRR(73,11,0)
      HRR(73,36,1)=HRR(73,36,1)+FnSpK*VRR(73,11,0)
      HRR(73,12,1)=HRR(73,12,1)+VRR(73,12,0)
      HRR(73,37,1)=HRR(73,37,1)+FnSpK*VRR(73,12,0)
      HRR(73,13,1)=HRR(73,13,1)+VRR(73,13,0)
      HRR(73,38,1)=HRR(73,38,1)+FnSpK*VRR(73,13,0)
      HRR(73,14,1)=HRR(73,14,1)+VRR(73,14,0)
      HRR(73,39,1)=HRR(73,39,1)+FnSpK*VRR(73,14,0)
      HRR(73,15,1)=HRR(73,15,1)+VRR(73,15,0)
      HRR(73,40,1)=HRR(73,40,1)+FnSpK*VRR(73,15,0)
      HRR(73,16,1)=HRR(73,16,1)+VRR(73,16,0)
      HRR(73,41,1)=HRR(73,41,1)+FnSpK*VRR(73,16,0)
      HRR(73,17,1)=HRR(73,17,1)+VRR(73,17,0)
      HRR(73,42,1)=HRR(73,42,1)+FnSpK*VRR(73,17,0)
      HRR(73,18,1)=HRR(73,18,1)+VRR(73,18,0)
      HRR(73,43,1)=HRR(73,43,1)+FnSpK*VRR(73,18,0)
      HRR(73,19,1)=HRR(73,19,1)+VRR(73,19,0)
      HRR(73,44,1)=HRR(73,44,1)+FnSpK*VRR(73,19,0)
      HRR(73,20,1)=HRR(73,20,1)+VRR(73,20,0)
      HRR(73,45,1)=HRR(73,45,1)+FnSpK*VRR(73,20,0)
      HRR(73,21,1)=HRR(73,21,1)+VRR(73,21,0)
      HRR(73,22,1)=HRR(73,22,1)+VRR(73,22,0)
      HRR(73,23,1)=HRR(73,23,1)+VRR(73,23,0)
      HRR(73,24,1)=HRR(73,24,1)+VRR(73,24,0)
      HRR(73,25,1)=HRR(73,25,1)+VRR(73,25,0)
      HRR(73,26,1)=HRR(73,26,1)+VRR(73,26,0)
      HRR(73,27,1)=HRR(73,27,1)+VRR(73,27,0)
      HRR(73,28,1)=HRR(73,28,1)+VRR(73,28,0)
      HRR(73,29,1)=HRR(73,29,1)+VRR(73,29,0)
      HRR(73,30,1)=HRR(73,30,1)+VRR(73,30,0)
      HRR(73,31,1)=HRR(73,31,1)+VRR(73,31,0)
      HRR(73,32,1)=HRR(73,32,1)+VRR(73,32,0)
      HRR(73,33,1)=HRR(73,33,1)+VRR(73,33,0)
      HRR(73,34,1)=HRR(73,34,1)+VRR(73,34,0)
      HRR(73,35,1)=HRR(73,35,1)+VRR(73,35,0)
      HRR(74,1,1)=HRR(74,1,1)+VRR(74,1,0)
      HRR(74,2,1)=HRR(74,2,1)+VRR(74,2,0)
      HRR(74,3,1)=HRR(74,3,1)+VRR(74,3,0)
      HRR(74,4,1)=HRR(74,4,1)+VRR(74,4,0)
      HRR(74,5,1)=HRR(74,5,1)+VRR(74,5,0)
      HRR(74,6,1)=HRR(74,6,1)+VRR(74,6,0)
      HRR(74,7,1)=HRR(74,7,1)+VRR(74,7,0)
      HRR(74,8,1)=HRR(74,8,1)+VRR(74,8,0)
      HRR(74,9,1)=HRR(74,9,1)+VRR(74,9,0)
      HRR(74,10,1)=HRR(74,10,1)+VRR(74,10,0)
      HRR(74,11,1)=HRR(74,11,1)+VRR(74,11,0)
      HRR(74,36,1)=HRR(74,36,1)+FnSpK*VRR(74,11,0)
      HRR(74,12,1)=HRR(74,12,1)+VRR(74,12,0)
      HRR(74,37,1)=HRR(74,37,1)+FnSpK*VRR(74,12,0)
      HRR(74,13,1)=HRR(74,13,1)+VRR(74,13,0)
      HRR(74,38,1)=HRR(74,38,1)+FnSpK*VRR(74,13,0)
      HRR(74,14,1)=HRR(74,14,1)+VRR(74,14,0)
      HRR(74,39,1)=HRR(74,39,1)+FnSpK*VRR(74,14,0)
      HRR(74,15,1)=HRR(74,15,1)+VRR(74,15,0)
      HRR(74,40,1)=HRR(74,40,1)+FnSpK*VRR(74,15,0)
      HRR(74,16,1)=HRR(74,16,1)+VRR(74,16,0)
      HRR(74,41,1)=HRR(74,41,1)+FnSpK*VRR(74,16,0)
      HRR(74,17,1)=HRR(74,17,1)+VRR(74,17,0)
      HRR(74,42,1)=HRR(74,42,1)+FnSpK*VRR(74,17,0)
      HRR(74,18,1)=HRR(74,18,1)+VRR(74,18,0)
      HRR(74,43,1)=HRR(74,43,1)+FnSpK*VRR(74,18,0)
      HRR(74,19,1)=HRR(74,19,1)+VRR(74,19,0)
      HRR(74,44,1)=HRR(74,44,1)+FnSpK*VRR(74,19,0)
      HRR(74,20,1)=HRR(74,20,1)+VRR(74,20,0)
      HRR(74,45,1)=HRR(74,45,1)+FnSpK*VRR(74,20,0)
      HRR(74,21,1)=HRR(74,21,1)+VRR(74,21,0)
      HRR(74,22,1)=HRR(74,22,1)+VRR(74,22,0)
      HRR(74,23,1)=HRR(74,23,1)+VRR(74,23,0)
      HRR(74,24,1)=HRR(74,24,1)+VRR(74,24,0)
      HRR(74,25,1)=HRR(74,25,1)+VRR(74,25,0)
      HRR(74,26,1)=HRR(74,26,1)+VRR(74,26,0)
      HRR(74,27,1)=HRR(74,27,1)+VRR(74,27,0)
      HRR(74,28,1)=HRR(74,28,1)+VRR(74,28,0)
      HRR(74,29,1)=HRR(74,29,1)+VRR(74,29,0)
      HRR(74,30,1)=HRR(74,30,1)+VRR(74,30,0)
      HRR(74,31,1)=HRR(74,31,1)+VRR(74,31,0)
      HRR(74,32,1)=HRR(74,32,1)+VRR(74,32,0)
      HRR(74,33,1)=HRR(74,33,1)+VRR(74,33,0)
      HRR(74,34,1)=HRR(74,34,1)+VRR(74,34,0)
      HRR(74,35,1)=HRR(74,35,1)+VRR(74,35,0)
      HRR(75,1,1)=HRR(75,1,1)+VRR(75,1,0)
      HRR(75,2,1)=HRR(75,2,1)+VRR(75,2,0)
      HRR(75,3,1)=HRR(75,3,1)+VRR(75,3,0)
      HRR(75,4,1)=HRR(75,4,1)+VRR(75,4,0)
      HRR(75,5,1)=HRR(75,5,1)+VRR(75,5,0)
      HRR(75,6,1)=HRR(75,6,1)+VRR(75,6,0)
      HRR(75,7,1)=HRR(75,7,1)+VRR(75,7,0)
      HRR(75,8,1)=HRR(75,8,1)+VRR(75,8,0)
      HRR(75,9,1)=HRR(75,9,1)+VRR(75,9,0)
      HRR(75,10,1)=HRR(75,10,1)+VRR(75,10,0)
      HRR(75,11,1)=HRR(75,11,1)+VRR(75,11,0)
      HRR(75,36,1)=HRR(75,36,1)+FnSpK*VRR(75,11,0)
      HRR(75,12,1)=HRR(75,12,1)+VRR(75,12,0)
      HRR(75,37,1)=HRR(75,37,1)+FnSpK*VRR(75,12,0)
      HRR(75,13,1)=HRR(75,13,1)+VRR(75,13,0)
      HRR(75,38,1)=HRR(75,38,1)+FnSpK*VRR(75,13,0)
      HRR(75,14,1)=HRR(75,14,1)+VRR(75,14,0)
      HRR(75,39,1)=HRR(75,39,1)+FnSpK*VRR(75,14,0)
      HRR(75,15,1)=HRR(75,15,1)+VRR(75,15,0)
      HRR(75,40,1)=HRR(75,40,1)+FnSpK*VRR(75,15,0)
      HRR(75,16,1)=HRR(75,16,1)+VRR(75,16,0)
      HRR(75,41,1)=HRR(75,41,1)+FnSpK*VRR(75,16,0)
      HRR(75,17,1)=HRR(75,17,1)+VRR(75,17,0)
      HRR(75,42,1)=HRR(75,42,1)+FnSpK*VRR(75,17,0)
      HRR(75,18,1)=HRR(75,18,1)+VRR(75,18,0)
      HRR(75,43,1)=HRR(75,43,1)+FnSpK*VRR(75,18,0)
      HRR(75,19,1)=HRR(75,19,1)+VRR(75,19,0)
      HRR(75,44,1)=HRR(75,44,1)+FnSpK*VRR(75,19,0)
      HRR(75,20,1)=HRR(75,20,1)+VRR(75,20,0)
      HRR(75,45,1)=HRR(75,45,1)+FnSpK*VRR(75,20,0)
      HRR(75,21,1)=HRR(75,21,1)+VRR(75,21,0)
      HRR(75,22,1)=HRR(75,22,1)+VRR(75,22,0)
      HRR(75,23,1)=HRR(75,23,1)+VRR(75,23,0)
      HRR(75,24,1)=HRR(75,24,1)+VRR(75,24,0)
      HRR(75,25,1)=HRR(75,25,1)+VRR(75,25,0)
      HRR(75,26,1)=HRR(75,26,1)+VRR(75,26,0)
      HRR(75,27,1)=HRR(75,27,1)+VRR(75,27,0)
      HRR(75,28,1)=HRR(75,28,1)+VRR(75,28,0)
      HRR(75,29,1)=HRR(75,29,1)+VRR(75,29,0)
      HRR(75,30,1)=HRR(75,30,1)+VRR(75,30,0)
      HRR(75,31,1)=HRR(75,31,1)+VRR(75,31,0)
      HRR(75,32,1)=HRR(75,32,1)+VRR(75,32,0)
      HRR(75,33,1)=HRR(75,33,1)+VRR(75,33,0)
      HRR(75,34,1)=HRR(75,34,1)+VRR(75,34,0)
      HRR(75,35,1)=HRR(75,35,1)+VRR(75,35,0)
      HRR(76,1,1)=HRR(76,1,1)+VRR(76,1,0)
      HRR(76,2,1)=HRR(76,2,1)+VRR(76,2,0)
      HRR(76,3,1)=HRR(76,3,1)+VRR(76,3,0)
      HRR(76,4,1)=HRR(76,4,1)+VRR(76,4,0)
      HRR(76,5,1)=HRR(76,5,1)+VRR(76,5,0)
      HRR(76,6,1)=HRR(76,6,1)+VRR(76,6,0)
      HRR(76,7,1)=HRR(76,7,1)+VRR(76,7,0)
      HRR(76,8,1)=HRR(76,8,1)+VRR(76,8,0)
      HRR(76,9,1)=HRR(76,9,1)+VRR(76,9,0)
      HRR(76,10,1)=HRR(76,10,1)+VRR(76,10,0)
      HRR(76,11,1)=HRR(76,11,1)+VRR(76,11,0)
      HRR(76,36,1)=HRR(76,36,1)+FnSpK*VRR(76,11,0)
      HRR(76,12,1)=HRR(76,12,1)+VRR(76,12,0)
      HRR(76,37,1)=HRR(76,37,1)+FnSpK*VRR(76,12,0)
      HRR(76,13,1)=HRR(76,13,1)+VRR(76,13,0)
      HRR(76,38,1)=HRR(76,38,1)+FnSpK*VRR(76,13,0)
      HRR(76,14,1)=HRR(76,14,1)+VRR(76,14,0)
      HRR(76,39,1)=HRR(76,39,1)+FnSpK*VRR(76,14,0)
      HRR(76,15,1)=HRR(76,15,1)+VRR(76,15,0)
      HRR(76,40,1)=HRR(76,40,1)+FnSpK*VRR(76,15,0)
      HRR(76,16,1)=HRR(76,16,1)+VRR(76,16,0)
      HRR(76,41,1)=HRR(76,41,1)+FnSpK*VRR(76,16,0)
      HRR(76,17,1)=HRR(76,17,1)+VRR(76,17,0)
      HRR(76,42,1)=HRR(76,42,1)+FnSpK*VRR(76,17,0)
      HRR(76,18,1)=HRR(76,18,1)+VRR(76,18,0)
      HRR(76,43,1)=HRR(76,43,1)+FnSpK*VRR(76,18,0)
      HRR(76,19,1)=HRR(76,19,1)+VRR(76,19,0)
      HRR(76,44,1)=HRR(76,44,1)+FnSpK*VRR(76,19,0)
      HRR(76,20,1)=HRR(76,20,1)+VRR(76,20,0)
      HRR(76,45,1)=HRR(76,45,1)+FnSpK*VRR(76,20,0)
      HRR(76,21,1)=HRR(76,21,1)+VRR(76,21,0)
      HRR(76,22,1)=HRR(76,22,1)+VRR(76,22,0)
      HRR(76,23,1)=HRR(76,23,1)+VRR(76,23,0)
      HRR(76,24,1)=HRR(76,24,1)+VRR(76,24,0)
      HRR(76,25,1)=HRR(76,25,1)+VRR(76,25,0)
      HRR(76,26,1)=HRR(76,26,1)+VRR(76,26,0)
      HRR(76,27,1)=HRR(76,27,1)+VRR(76,27,0)
      HRR(76,28,1)=HRR(76,28,1)+VRR(76,28,0)
      HRR(76,29,1)=HRR(76,29,1)+VRR(76,29,0)
      HRR(76,30,1)=HRR(76,30,1)+VRR(76,30,0)
      HRR(76,31,1)=HRR(76,31,1)+VRR(76,31,0)
      HRR(76,32,1)=HRR(76,32,1)+VRR(76,32,0)
      HRR(76,33,1)=HRR(76,33,1)+VRR(76,33,0)
      HRR(76,34,1)=HRR(76,34,1)+VRR(76,34,0)
      HRR(76,35,1)=HRR(76,35,1)+VRR(76,35,0)
      HRR(77,1,1)=HRR(77,1,1)+VRR(77,1,0)
      HRR(77,2,1)=HRR(77,2,1)+VRR(77,2,0)
      HRR(77,3,1)=HRR(77,3,1)+VRR(77,3,0)
      HRR(77,4,1)=HRR(77,4,1)+VRR(77,4,0)
      HRR(77,5,1)=HRR(77,5,1)+VRR(77,5,0)
      HRR(77,6,1)=HRR(77,6,1)+VRR(77,6,0)
      HRR(77,7,1)=HRR(77,7,1)+VRR(77,7,0)
      HRR(77,8,1)=HRR(77,8,1)+VRR(77,8,0)
      HRR(77,9,1)=HRR(77,9,1)+VRR(77,9,0)
      HRR(77,10,1)=HRR(77,10,1)+VRR(77,10,0)
      HRR(77,11,1)=HRR(77,11,1)+VRR(77,11,0)
      HRR(77,36,1)=HRR(77,36,1)+FnSpK*VRR(77,11,0)
      HRR(77,12,1)=HRR(77,12,1)+VRR(77,12,0)
      HRR(77,37,1)=HRR(77,37,1)+FnSpK*VRR(77,12,0)
      HRR(77,13,1)=HRR(77,13,1)+VRR(77,13,0)
      HRR(77,38,1)=HRR(77,38,1)+FnSpK*VRR(77,13,0)
      HRR(77,14,1)=HRR(77,14,1)+VRR(77,14,0)
      HRR(77,39,1)=HRR(77,39,1)+FnSpK*VRR(77,14,0)
      HRR(77,15,1)=HRR(77,15,1)+VRR(77,15,0)
      HRR(77,40,1)=HRR(77,40,1)+FnSpK*VRR(77,15,0)
      HRR(77,16,1)=HRR(77,16,1)+VRR(77,16,0)
      HRR(77,41,1)=HRR(77,41,1)+FnSpK*VRR(77,16,0)
      HRR(77,17,1)=HRR(77,17,1)+VRR(77,17,0)
      HRR(77,42,1)=HRR(77,42,1)+FnSpK*VRR(77,17,0)
      HRR(77,18,1)=HRR(77,18,1)+VRR(77,18,0)
      HRR(77,43,1)=HRR(77,43,1)+FnSpK*VRR(77,18,0)
      HRR(77,19,1)=HRR(77,19,1)+VRR(77,19,0)
      HRR(77,44,1)=HRR(77,44,1)+FnSpK*VRR(77,19,0)
      HRR(77,20,1)=HRR(77,20,1)+VRR(77,20,0)
      HRR(77,45,1)=HRR(77,45,1)+FnSpK*VRR(77,20,0)
      HRR(77,21,1)=HRR(77,21,1)+VRR(77,21,0)
      HRR(77,22,1)=HRR(77,22,1)+VRR(77,22,0)
      HRR(77,23,1)=HRR(77,23,1)+VRR(77,23,0)
      HRR(77,24,1)=HRR(77,24,1)+VRR(77,24,0)
      HRR(77,25,1)=HRR(77,25,1)+VRR(77,25,0)
      HRR(77,26,1)=HRR(77,26,1)+VRR(77,26,0)
      HRR(77,27,1)=HRR(77,27,1)+VRR(77,27,0)
      HRR(77,28,1)=HRR(77,28,1)+VRR(77,28,0)
      HRR(77,29,1)=HRR(77,29,1)+VRR(77,29,0)
      HRR(77,30,1)=HRR(77,30,1)+VRR(77,30,0)
      HRR(77,31,1)=HRR(77,31,1)+VRR(77,31,0)
      HRR(77,32,1)=HRR(77,32,1)+VRR(77,32,0)
      HRR(77,33,1)=HRR(77,33,1)+VRR(77,33,0)
      HRR(77,34,1)=HRR(77,34,1)+VRR(77,34,0)
      HRR(77,35,1)=HRR(77,35,1)+VRR(77,35,0)
      HRR(78,1,1)=HRR(78,1,1)+VRR(78,1,0)
      HRR(78,2,1)=HRR(78,2,1)+VRR(78,2,0)
      HRR(78,3,1)=HRR(78,3,1)+VRR(78,3,0)
      HRR(78,4,1)=HRR(78,4,1)+VRR(78,4,0)
      HRR(78,5,1)=HRR(78,5,1)+VRR(78,5,0)
      HRR(78,6,1)=HRR(78,6,1)+VRR(78,6,0)
      HRR(78,7,1)=HRR(78,7,1)+VRR(78,7,0)
      HRR(78,8,1)=HRR(78,8,1)+VRR(78,8,0)
      HRR(78,9,1)=HRR(78,9,1)+VRR(78,9,0)
      HRR(78,10,1)=HRR(78,10,1)+VRR(78,10,0)
      HRR(78,11,1)=HRR(78,11,1)+VRR(78,11,0)
      HRR(78,36,1)=HRR(78,36,1)+FnSpK*VRR(78,11,0)
      HRR(78,12,1)=HRR(78,12,1)+VRR(78,12,0)
      HRR(78,37,1)=HRR(78,37,1)+FnSpK*VRR(78,12,0)
      HRR(78,13,1)=HRR(78,13,1)+VRR(78,13,0)
      HRR(78,38,1)=HRR(78,38,1)+FnSpK*VRR(78,13,0)
      HRR(78,14,1)=HRR(78,14,1)+VRR(78,14,0)
      HRR(78,39,1)=HRR(78,39,1)+FnSpK*VRR(78,14,0)
      HRR(78,15,1)=HRR(78,15,1)+VRR(78,15,0)
      HRR(78,40,1)=HRR(78,40,1)+FnSpK*VRR(78,15,0)
      HRR(78,16,1)=HRR(78,16,1)+VRR(78,16,0)
      HRR(78,41,1)=HRR(78,41,1)+FnSpK*VRR(78,16,0)
      HRR(78,17,1)=HRR(78,17,1)+VRR(78,17,0)
      HRR(78,42,1)=HRR(78,42,1)+FnSpK*VRR(78,17,0)
      HRR(78,18,1)=HRR(78,18,1)+VRR(78,18,0)
      HRR(78,43,1)=HRR(78,43,1)+FnSpK*VRR(78,18,0)
      HRR(78,19,1)=HRR(78,19,1)+VRR(78,19,0)
      HRR(78,44,1)=HRR(78,44,1)+FnSpK*VRR(78,19,0)
      HRR(78,20,1)=HRR(78,20,1)+VRR(78,20,0)
      HRR(78,45,1)=HRR(78,45,1)+FnSpK*VRR(78,20,0)
      HRR(78,21,1)=HRR(78,21,1)+VRR(78,21,0)
      HRR(78,22,1)=HRR(78,22,1)+VRR(78,22,0)
      HRR(78,23,1)=HRR(78,23,1)+VRR(78,23,0)
      HRR(78,24,1)=HRR(78,24,1)+VRR(78,24,0)
      HRR(78,25,1)=HRR(78,25,1)+VRR(78,25,0)
      HRR(78,26,1)=HRR(78,26,1)+VRR(78,26,0)
      HRR(78,27,1)=HRR(78,27,1)+VRR(78,27,0)
      HRR(78,28,1)=HRR(78,28,1)+VRR(78,28,0)
      HRR(78,29,1)=HRR(78,29,1)+VRR(78,29,0)
      HRR(78,30,1)=HRR(78,30,1)+VRR(78,30,0)
      HRR(78,31,1)=HRR(78,31,1)+VRR(78,31,0)
      HRR(78,32,1)=HRR(78,32,1)+VRR(78,32,0)
      HRR(78,33,1)=HRR(78,33,1)+VRR(78,33,0)
      HRR(78,34,1)=HRR(78,34,1)+VRR(78,34,0)
      HRR(78,35,1)=HRR(78,35,1)+VRR(78,35,0)
      HRR(79,1,1)=HRR(79,1,1)+VRR(79,1,0)
      HRR(79,2,1)=HRR(79,2,1)+VRR(79,2,0)
      HRR(79,3,1)=HRR(79,3,1)+VRR(79,3,0)
      HRR(79,4,1)=HRR(79,4,1)+VRR(79,4,0)
      HRR(79,5,1)=HRR(79,5,1)+VRR(79,5,0)
      HRR(79,6,1)=HRR(79,6,1)+VRR(79,6,0)
      HRR(79,7,1)=HRR(79,7,1)+VRR(79,7,0)
      HRR(79,8,1)=HRR(79,8,1)+VRR(79,8,0)
      HRR(79,9,1)=HRR(79,9,1)+VRR(79,9,0)
      HRR(79,10,1)=HRR(79,10,1)+VRR(79,10,0)
      HRR(79,11,1)=HRR(79,11,1)+VRR(79,11,0)
      HRR(79,36,1)=HRR(79,36,1)+FnSpK*VRR(79,11,0)
      HRR(79,12,1)=HRR(79,12,1)+VRR(79,12,0)
      HRR(79,37,1)=HRR(79,37,1)+FnSpK*VRR(79,12,0)
      HRR(79,13,1)=HRR(79,13,1)+VRR(79,13,0)
      HRR(79,38,1)=HRR(79,38,1)+FnSpK*VRR(79,13,0)
      HRR(79,14,1)=HRR(79,14,1)+VRR(79,14,0)
      HRR(79,39,1)=HRR(79,39,1)+FnSpK*VRR(79,14,0)
      HRR(79,15,1)=HRR(79,15,1)+VRR(79,15,0)
      HRR(79,40,1)=HRR(79,40,1)+FnSpK*VRR(79,15,0)
      HRR(79,16,1)=HRR(79,16,1)+VRR(79,16,0)
      HRR(79,41,1)=HRR(79,41,1)+FnSpK*VRR(79,16,0)
      HRR(79,17,1)=HRR(79,17,1)+VRR(79,17,0)
      HRR(79,42,1)=HRR(79,42,1)+FnSpK*VRR(79,17,0)
      HRR(79,18,1)=HRR(79,18,1)+VRR(79,18,0)
      HRR(79,43,1)=HRR(79,43,1)+FnSpK*VRR(79,18,0)
      HRR(79,19,1)=HRR(79,19,1)+VRR(79,19,0)
      HRR(79,44,1)=HRR(79,44,1)+FnSpK*VRR(79,19,0)
      HRR(79,20,1)=HRR(79,20,1)+VRR(79,20,0)
      HRR(79,45,1)=HRR(79,45,1)+FnSpK*VRR(79,20,0)
      HRR(79,21,1)=HRR(79,21,1)+VRR(79,21,0)
      HRR(79,22,1)=HRR(79,22,1)+VRR(79,22,0)
      HRR(79,23,1)=HRR(79,23,1)+VRR(79,23,0)
      HRR(79,24,1)=HRR(79,24,1)+VRR(79,24,0)
      HRR(79,25,1)=HRR(79,25,1)+VRR(79,25,0)
      HRR(79,26,1)=HRR(79,26,1)+VRR(79,26,0)
      HRR(79,27,1)=HRR(79,27,1)+VRR(79,27,0)
      HRR(79,28,1)=HRR(79,28,1)+VRR(79,28,0)
      HRR(79,29,1)=HRR(79,29,1)+VRR(79,29,0)
      HRR(79,30,1)=HRR(79,30,1)+VRR(79,30,0)
      HRR(79,31,1)=HRR(79,31,1)+VRR(79,31,0)
      HRR(79,32,1)=HRR(79,32,1)+VRR(79,32,0)
      HRR(79,33,1)=HRR(79,33,1)+VRR(79,33,0)
      HRR(79,34,1)=HRR(79,34,1)+VRR(79,34,0)
      HRR(79,35,1)=HRR(79,35,1)+VRR(79,35,0)
      HRR(80,1,1)=HRR(80,1,1)+VRR(80,1,0)
      HRR(80,2,1)=HRR(80,2,1)+VRR(80,2,0)
      HRR(80,3,1)=HRR(80,3,1)+VRR(80,3,0)
      HRR(80,4,1)=HRR(80,4,1)+VRR(80,4,0)
      HRR(80,5,1)=HRR(80,5,1)+VRR(80,5,0)
      HRR(80,6,1)=HRR(80,6,1)+VRR(80,6,0)
      HRR(80,7,1)=HRR(80,7,1)+VRR(80,7,0)
      HRR(80,8,1)=HRR(80,8,1)+VRR(80,8,0)
      HRR(80,9,1)=HRR(80,9,1)+VRR(80,9,0)
      HRR(80,10,1)=HRR(80,10,1)+VRR(80,10,0)
      HRR(80,11,1)=HRR(80,11,1)+VRR(80,11,0)
      HRR(80,36,1)=HRR(80,36,1)+FnSpK*VRR(80,11,0)
      HRR(80,12,1)=HRR(80,12,1)+VRR(80,12,0)
      HRR(80,37,1)=HRR(80,37,1)+FnSpK*VRR(80,12,0)
      HRR(80,13,1)=HRR(80,13,1)+VRR(80,13,0)
      HRR(80,38,1)=HRR(80,38,1)+FnSpK*VRR(80,13,0)
      HRR(80,14,1)=HRR(80,14,1)+VRR(80,14,0)
      HRR(80,39,1)=HRR(80,39,1)+FnSpK*VRR(80,14,0)
      HRR(80,15,1)=HRR(80,15,1)+VRR(80,15,0)
      HRR(80,40,1)=HRR(80,40,1)+FnSpK*VRR(80,15,0)
      HRR(80,16,1)=HRR(80,16,1)+VRR(80,16,0)
      HRR(80,41,1)=HRR(80,41,1)+FnSpK*VRR(80,16,0)
      HRR(80,17,1)=HRR(80,17,1)+VRR(80,17,0)
      HRR(80,42,1)=HRR(80,42,1)+FnSpK*VRR(80,17,0)
      HRR(80,18,1)=HRR(80,18,1)+VRR(80,18,0)
      HRR(80,43,1)=HRR(80,43,1)+FnSpK*VRR(80,18,0)
      HRR(80,19,1)=HRR(80,19,1)+VRR(80,19,0)
      HRR(80,44,1)=HRR(80,44,1)+FnSpK*VRR(80,19,0)
      HRR(80,20,1)=HRR(80,20,1)+VRR(80,20,0)
      HRR(80,45,1)=HRR(80,45,1)+FnSpK*VRR(80,20,0)
      HRR(80,21,1)=HRR(80,21,1)+VRR(80,21,0)
      HRR(80,22,1)=HRR(80,22,1)+VRR(80,22,0)
      HRR(80,23,1)=HRR(80,23,1)+VRR(80,23,0)
      HRR(80,24,1)=HRR(80,24,1)+VRR(80,24,0)
      HRR(80,25,1)=HRR(80,25,1)+VRR(80,25,0)
      HRR(80,26,1)=HRR(80,26,1)+VRR(80,26,0)
      HRR(80,27,1)=HRR(80,27,1)+VRR(80,27,0)
      HRR(80,28,1)=HRR(80,28,1)+VRR(80,28,0)
      HRR(80,29,1)=HRR(80,29,1)+VRR(80,29,0)
      HRR(80,30,1)=HRR(80,30,1)+VRR(80,30,0)
      HRR(80,31,1)=HRR(80,31,1)+VRR(80,31,0)
      HRR(80,32,1)=HRR(80,32,1)+VRR(80,32,0)
      HRR(80,33,1)=HRR(80,33,1)+VRR(80,33,0)
      HRR(80,34,1)=HRR(80,34,1)+VRR(80,34,0)
      HRR(80,35,1)=HRR(80,35,1)+VRR(80,35,0)
      HRR(81,1,1)=HRR(81,1,1)+VRR(81,1,0)
      HRR(81,2,1)=HRR(81,2,1)+VRR(81,2,0)
      HRR(81,3,1)=HRR(81,3,1)+VRR(81,3,0)
      HRR(81,4,1)=HRR(81,4,1)+VRR(81,4,0)
      HRR(81,5,1)=HRR(81,5,1)+VRR(81,5,0)
      HRR(81,6,1)=HRR(81,6,1)+VRR(81,6,0)
      HRR(81,7,1)=HRR(81,7,1)+VRR(81,7,0)
      HRR(81,8,1)=HRR(81,8,1)+VRR(81,8,0)
      HRR(81,9,1)=HRR(81,9,1)+VRR(81,9,0)
      HRR(81,10,1)=HRR(81,10,1)+VRR(81,10,0)
      HRR(81,11,1)=HRR(81,11,1)+VRR(81,11,0)
      HRR(81,36,1)=HRR(81,36,1)+FnSpK*VRR(81,11,0)
      HRR(81,12,1)=HRR(81,12,1)+VRR(81,12,0)
      HRR(81,37,1)=HRR(81,37,1)+FnSpK*VRR(81,12,0)
      HRR(81,13,1)=HRR(81,13,1)+VRR(81,13,0)
      HRR(81,38,1)=HRR(81,38,1)+FnSpK*VRR(81,13,0)
      HRR(81,14,1)=HRR(81,14,1)+VRR(81,14,0)
      HRR(81,39,1)=HRR(81,39,1)+FnSpK*VRR(81,14,0)
      HRR(81,15,1)=HRR(81,15,1)+VRR(81,15,0)
      HRR(81,40,1)=HRR(81,40,1)+FnSpK*VRR(81,15,0)
      HRR(81,16,1)=HRR(81,16,1)+VRR(81,16,0)
      HRR(81,41,1)=HRR(81,41,1)+FnSpK*VRR(81,16,0)
      HRR(81,17,1)=HRR(81,17,1)+VRR(81,17,0)
      HRR(81,42,1)=HRR(81,42,1)+FnSpK*VRR(81,17,0)
      HRR(81,18,1)=HRR(81,18,1)+VRR(81,18,0)
      HRR(81,43,1)=HRR(81,43,1)+FnSpK*VRR(81,18,0)
      HRR(81,19,1)=HRR(81,19,1)+VRR(81,19,0)
      HRR(81,44,1)=HRR(81,44,1)+FnSpK*VRR(81,19,0)
      HRR(81,20,1)=HRR(81,20,1)+VRR(81,20,0)
      HRR(81,45,1)=HRR(81,45,1)+FnSpK*VRR(81,20,0)
      HRR(81,21,1)=HRR(81,21,1)+VRR(81,21,0)
      HRR(81,22,1)=HRR(81,22,1)+VRR(81,22,0)
      HRR(81,23,1)=HRR(81,23,1)+VRR(81,23,0)
      HRR(81,24,1)=HRR(81,24,1)+VRR(81,24,0)
      HRR(81,25,1)=HRR(81,25,1)+VRR(81,25,0)
      HRR(81,26,1)=HRR(81,26,1)+VRR(81,26,0)
      HRR(81,27,1)=HRR(81,27,1)+VRR(81,27,0)
      HRR(81,28,1)=HRR(81,28,1)+VRR(81,28,0)
      HRR(81,29,1)=HRR(81,29,1)+VRR(81,29,0)
      HRR(81,30,1)=HRR(81,30,1)+VRR(81,30,0)
      HRR(81,31,1)=HRR(81,31,1)+VRR(81,31,0)
      HRR(81,32,1)=HRR(81,32,1)+VRR(81,32,0)
      HRR(81,33,1)=HRR(81,33,1)+VRR(81,33,0)
      HRR(81,34,1)=HRR(81,34,1)+VRR(81,34,0)
      HRR(81,35,1)=HRR(81,35,1)+VRR(81,35,0)
      HRR(82,1,1)=HRR(82,1,1)+VRR(82,1,0)
      HRR(82,2,1)=HRR(82,2,1)+VRR(82,2,0)
      HRR(82,3,1)=HRR(82,3,1)+VRR(82,3,0)
      HRR(82,4,1)=HRR(82,4,1)+VRR(82,4,0)
      HRR(82,5,1)=HRR(82,5,1)+VRR(82,5,0)
      HRR(82,6,1)=HRR(82,6,1)+VRR(82,6,0)
      HRR(82,7,1)=HRR(82,7,1)+VRR(82,7,0)
      HRR(82,8,1)=HRR(82,8,1)+VRR(82,8,0)
      HRR(82,9,1)=HRR(82,9,1)+VRR(82,9,0)
      HRR(82,10,1)=HRR(82,10,1)+VRR(82,10,0)
      HRR(82,11,1)=HRR(82,11,1)+VRR(82,11,0)
      HRR(82,36,1)=HRR(82,36,1)+FnSpK*VRR(82,11,0)
      HRR(82,12,1)=HRR(82,12,1)+VRR(82,12,0)
      HRR(82,37,1)=HRR(82,37,1)+FnSpK*VRR(82,12,0)
      HRR(82,13,1)=HRR(82,13,1)+VRR(82,13,0)
      HRR(82,38,1)=HRR(82,38,1)+FnSpK*VRR(82,13,0)
      HRR(82,14,1)=HRR(82,14,1)+VRR(82,14,0)
      HRR(82,39,1)=HRR(82,39,1)+FnSpK*VRR(82,14,0)
      HRR(82,15,1)=HRR(82,15,1)+VRR(82,15,0)
      HRR(82,40,1)=HRR(82,40,1)+FnSpK*VRR(82,15,0)
      HRR(82,16,1)=HRR(82,16,1)+VRR(82,16,0)
      HRR(82,41,1)=HRR(82,41,1)+FnSpK*VRR(82,16,0)
      HRR(82,17,1)=HRR(82,17,1)+VRR(82,17,0)
      HRR(82,42,1)=HRR(82,42,1)+FnSpK*VRR(82,17,0)
      HRR(82,18,1)=HRR(82,18,1)+VRR(82,18,0)
      HRR(82,43,1)=HRR(82,43,1)+FnSpK*VRR(82,18,0)
      HRR(82,19,1)=HRR(82,19,1)+VRR(82,19,0)
      HRR(82,44,1)=HRR(82,44,1)+FnSpK*VRR(82,19,0)
      HRR(82,20,1)=HRR(82,20,1)+VRR(82,20,0)
      HRR(82,45,1)=HRR(82,45,1)+FnSpK*VRR(82,20,0)
      HRR(82,21,1)=HRR(82,21,1)+VRR(82,21,0)
      HRR(82,22,1)=HRR(82,22,1)+VRR(82,22,0)
      HRR(82,23,1)=HRR(82,23,1)+VRR(82,23,0)
      HRR(82,24,1)=HRR(82,24,1)+VRR(82,24,0)
      HRR(82,25,1)=HRR(82,25,1)+VRR(82,25,0)
      HRR(82,26,1)=HRR(82,26,1)+VRR(82,26,0)
      HRR(82,27,1)=HRR(82,27,1)+VRR(82,27,0)
      HRR(82,28,1)=HRR(82,28,1)+VRR(82,28,0)
      HRR(82,29,1)=HRR(82,29,1)+VRR(82,29,0)
      HRR(82,30,1)=HRR(82,30,1)+VRR(82,30,0)
      HRR(82,31,1)=HRR(82,31,1)+VRR(82,31,0)
      HRR(82,32,1)=HRR(82,32,1)+VRR(82,32,0)
      HRR(82,33,1)=HRR(82,33,1)+VRR(82,33,0)
      HRR(82,34,1)=HRR(82,34,1)+VRR(82,34,0)
      HRR(82,35,1)=HRR(82,35,1)+VRR(82,35,0)
      HRR(83,1,1)=HRR(83,1,1)+VRR(83,1,0)
      HRR(83,2,1)=HRR(83,2,1)+VRR(83,2,0)
      HRR(83,3,1)=HRR(83,3,1)+VRR(83,3,0)
      HRR(83,4,1)=HRR(83,4,1)+VRR(83,4,0)
      HRR(83,5,1)=HRR(83,5,1)+VRR(83,5,0)
      HRR(83,6,1)=HRR(83,6,1)+VRR(83,6,0)
      HRR(83,7,1)=HRR(83,7,1)+VRR(83,7,0)
      HRR(83,8,1)=HRR(83,8,1)+VRR(83,8,0)
      HRR(83,9,1)=HRR(83,9,1)+VRR(83,9,0)
      HRR(83,10,1)=HRR(83,10,1)+VRR(83,10,0)
      HRR(83,11,1)=HRR(83,11,1)+VRR(83,11,0)
      HRR(83,36,1)=HRR(83,36,1)+FnSpK*VRR(83,11,0)
      HRR(83,12,1)=HRR(83,12,1)+VRR(83,12,0)
      HRR(83,37,1)=HRR(83,37,1)+FnSpK*VRR(83,12,0)
      HRR(83,13,1)=HRR(83,13,1)+VRR(83,13,0)
      HRR(83,38,1)=HRR(83,38,1)+FnSpK*VRR(83,13,0)
      HRR(83,14,1)=HRR(83,14,1)+VRR(83,14,0)
      HRR(83,39,1)=HRR(83,39,1)+FnSpK*VRR(83,14,0)
      HRR(83,15,1)=HRR(83,15,1)+VRR(83,15,0)
      HRR(83,40,1)=HRR(83,40,1)+FnSpK*VRR(83,15,0)
      HRR(83,16,1)=HRR(83,16,1)+VRR(83,16,0)
      HRR(83,41,1)=HRR(83,41,1)+FnSpK*VRR(83,16,0)
      HRR(83,17,1)=HRR(83,17,1)+VRR(83,17,0)
      HRR(83,42,1)=HRR(83,42,1)+FnSpK*VRR(83,17,0)
      HRR(83,18,1)=HRR(83,18,1)+VRR(83,18,0)
      HRR(83,43,1)=HRR(83,43,1)+FnSpK*VRR(83,18,0)
      HRR(83,19,1)=HRR(83,19,1)+VRR(83,19,0)
      HRR(83,44,1)=HRR(83,44,1)+FnSpK*VRR(83,19,0)
      HRR(83,20,1)=HRR(83,20,1)+VRR(83,20,0)
      HRR(83,45,1)=HRR(83,45,1)+FnSpK*VRR(83,20,0)
      HRR(83,21,1)=HRR(83,21,1)+VRR(83,21,0)
      HRR(83,22,1)=HRR(83,22,1)+VRR(83,22,0)
      HRR(83,23,1)=HRR(83,23,1)+VRR(83,23,0)
      HRR(83,24,1)=HRR(83,24,1)+VRR(83,24,0)
      HRR(83,25,1)=HRR(83,25,1)+VRR(83,25,0)
      HRR(83,26,1)=HRR(83,26,1)+VRR(83,26,0)
      HRR(83,27,1)=HRR(83,27,1)+VRR(83,27,0)
      HRR(83,28,1)=HRR(83,28,1)+VRR(83,28,0)
      HRR(83,29,1)=HRR(83,29,1)+VRR(83,29,0)
      HRR(83,30,1)=HRR(83,30,1)+VRR(83,30,0)
      HRR(83,31,1)=HRR(83,31,1)+VRR(83,31,0)
      HRR(83,32,1)=HRR(83,32,1)+VRR(83,32,0)
      HRR(83,33,1)=HRR(83,33,1)+VRR(83,33,0)
      HRR(83,34,1)=HRR(83,34,1)+VRR(83,34,0)
      HRR(83,35,1)=HRR(83,35,1)+VRR(83,35,0)
      HRR(84,1,1)=HRR(84,1,1)+VRR(84,1,0)
      HRR(84,2,1)=HRR(84,2,1)+VRR(84,2,0)
      HRR(84,3,1)=HRR(84,3,1)+VRR(84,3,0)
      HRR(84,4,1)=HRR(84,4,1)+VRR(84,4,0)
      HRR(84,5,1)=HRR(84,5,1)+VRR(84,5,0)
      HRR(84,6,1)=HRR(84,6,1)+VRR(84,6,0)
      HRR(84,7,1)=HRR(84,7,1)+VRR(84,7,0)
      HRR(84,8,1)=HRR(84,8,1)+VRR(84,8,0)
      HRR(84,9,1)=HRR(84,9,1)+VRR(84,9,0)
      HRR(84,10,1)=HRR(84,10,1)+VRR(84,10,0)
      HRR(84,11,1)=HRR(84,11,1)+VRR(84,11,0)
      HRR(84,36,1)=HRR(84,36,1)+FnSpK*VRR(84,11,0)
      HRR(84,12,1)=HRR(84,12,1)+VRR(84,12,0)
      HRR(84,37,1)=HRR(84,37,1)+FnSpK*VRR(84,12,0)
      HRR(84,13,1)=HRR(84,13,1)+VRR(84,13,0)
      HRR(84,38,1)=HRR(84,38,1)+FnSpK*VRR(84,13,0)
      HRR(84,14,1)=HRR(84,14,1)+VRR(84,14,0)
      HRR(84,39,1)=HRR(84,39,1)+FnSpK*VRR(84,14,0)
      HRR(84,15,1)=HRR(84,15,1)+VRR(84,15,0)
      HRR(84,40,1)=HRR(84,40,1)+FnSpK*VRR(84,15,0)
      HRR(84,16,1)=HRR(84,16,1)+VRR(84,16,0)
      HRR(84,41,1)=HRR(84,41,1)+FnSpK*VRR(84,16,0)
      HRR(84,17,1)=HRR(84,17,1)+VRR(84,17,0)
      HRR(84,42,1)=HRR(84,42,1)+FnSpK*VRR(84,17,0)
      HRR(84,18,1)=HRR(84,18,1)+VRR(84,18,0)
      HRR(84,43,1)=HRR(84,43,1)+FnSpK*VRR(84,18,0)
      HRR(84,19,1)=HRR(84,19,1)+VRR(84,19,0)
      HRR(84,44,1)=HRR(84,44,1)+FnSpK*VRR(84,19,0)
      HRR(84,20,1)=HRR(84,20,1)+VRR(84,20,0)
      HRR(84,45,1)=HRR(84,45,1)+FnSpK*VRR(84,20,0)
      HRR(84,21,1)=HRR(84,21,1)+VRR(84,21,0)
      HRR(84,22,1)=HRR(84,22,1)+VRR(84,22,0)
      HRR(84,23,1)=HRR(84,23,1)+VRR(84,23,0)
      HRR(84,24,1)=HRR(84,24,1)+VRR(84,24,0)
      HRR(84,25,1)=HRR(84,25,1)+VRR(84,25,0)
      HRR(84,26,1)=HRR(84,26,1)+VRR(84,26,0)
      HRR(84,27,1)=HRR(84,27,1)+VRR(84,27,0)
      HRR(84,28,1)=HRR(84,28,1)+VRR(84,28,0)
      HRR(84,29,1)=HRR(84,29,1)+VRR(84,29,0)
      HRR(84,30,1)=HRR(84,30,1)+VRR(84,30,0)
      HRR(84,31,1)=HRR(84,31,1)+VRR(84,31,0)
      HRR(84,32,1)=HRR(84,32,1)+VRR(84,32,0)
      HRR(84,33,1)=HRR(84,33,1)+VRR(84,33,0)
      HRR(84,34,1)=HRR(84,34,1)+VRR(84,34,0)
      HRR(84,35,1)=HRR(84,35,1)+VRR(84,35,0)
    END SUBROUTINE CNTRCT1010102
